function [A, X, iContours]=TA_Pb_affinities(I)
%
%[A, X, iC]=TA_Pb_affinities(I, R)
%  
%  TA_Pb_affinities computes the affinities A, the feature vectors X and the 
%  contour map iC using the 'Berkeley Segmentation Engine' code.
%  (see www.cs.berkeley.edu/~fowlkes/BSE)
%  
%  Marco Andreetto, Caltech 2009
% 

% =========================================================================
%  check if input image is valid (class double and 0-1 values)
% =========================================================================
if ~isa(I, 'double')
  error('TA_Pb_affinities: I must be of class double');
end

if max(max(max(I)))>1
  error('TA_Pb_affinities: values in I must be in the 0-1 range');
end


% =========================================================================
%  compute the affinities and the Pb contour map
% =========================================================================

% get a random label for the tmp files. Usefull if multiple instances are
% running on the same machine
rl=genRandomLabel(8);

% tmp files used in the computation
tmpImageFile=['/tmp/bse_input_image_', rl, '.png'];
tmpAffFile=['/tmp/bse_aff_data_', rl, '.aff'];
tmpPbFile=['/tmp/bse_aff_data_', rl, '.pb'];

% save the image for the external program 'segment' and 'pixelfeatures'
imwrite(I, tmpImageFile);

% execute the command 
[s, w]=system(['/home/marco/research/libraries/BSE-1.2/segment -dthresh 9 ', ...
               '-image ', tmpImageFile, ' -smatrixfile ', tmpAffFile]);

% check for possible errors
if (s~=0), error(w); end

% load the data
A=read_smatrix(tmpAffFile);

% reorder the matrix rows and columns
A=reorder_rows_and_columns(A, rows(I), cols(I));

% finally we have to compute the Pb image
[s, w]=system(['/home/marco/research/libraries/BSE-1.2/pixelfeatures ', ...
               '-image ', tmpImageFile, ' -pbfile ', tmpPbFile]);

% check for possible errors
if (s~=0), error(w); end

% load the contour map
iContours=read_array(tmpPbFile); 

% remove temp files
try
  delete(tmpImageFile);
  delete(tmpAffFile);
  delete(tmpPbFile);
catch expId
  disp('Unable to cancel tmp files');
end


% =========================================================================
%  get the feature vectors
% =========================================================================

X=getFeatureVectors(I);


function A=reorder_rows_and_columns(A, M, N)

% given the matrix A reorder the rows and column of the matrix such that
% they are in MATLAB column order.

T=rows(A); % total number of elements.

% check that everything is coherent
if (T~=M*N), error('dimension mismatch'); end;

% obtain the row and column indeces for the current listing of the
% observations. These values are in C convention.
r=floor((0:(T-1))/N);
c=mod((0:(T-1)), N);

% these are the new labels for the observations
p=(M*c+r)+1;

[i, j, a]=find(A);
A=sparse(p(i), p(j), a, rows(A), cols(A));

% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
