function [A, X, iContours]=TA_ucm_affinities(I, L, iContours)
%
%[A, X, iC]=TA_ucm_affinities(I, R, iC)
%  
%  TA_ucm_affinities computes the affinities and the observation vectors using 
%  the ucm by the Berkeley group (see P. Arbelaez et al. CVPR2009) see: 
%  www.eecs.berkeley.edu/Research/Projects/CS/vision/grouping/gpb/grouping.zip
%  'R' is the radius consider for computing the intervening
%  contours: pixels further away than R are not consider and their affinity
%  is set to zero. Finally, if the input iC is given then the countour map
%  is not computed.
%
%  Marco Andreetto, Caltech 2009
% 

% check the input
if ~isa(I, 'double')
  error('TA_ucm_affinities: I must be double');
end

if max(max(max(I)))>1
  error('TA_ucm_affinities: values in I must be in the 0-1 range');
end

% by defualt we consider only pixels that have a distance no larger than 5
if nargin<2, L=5; end

% if no contour information is given then we have to compute it
if nargin<3

  % temp files for the computation
  rl=genRandomLabel(10);
  tmpImageFile=['/tmp/tmp_TA_ucm_', rl, '.jpg'];
  tmpResFile=['/tmp/tmp_TA_ucm_', rl, '.mat'];

  % compute ultrametric contour map
  imwrite(I, tmpImageFile, 'Quality', 100);
  globalPb(tmpImageFile, tmpResFile);
  load(tmpResFile, 'gPb_orient');
  ucm = im2double(contours2ucm(gPb_orient, 'imageSize'));
  iContours=ucm;
  
end

% dimension of the image
M=rows(I);
N=cols(I);

% Compute the grid points.
[ix, iy, ix_range, iy_range]=getGridPositions(N, M, 1);
idx=sub2ind([M, N], iy, ix);

gridPoint=zeros(M, N);
gridPoint(idx)=1;

% compute the linear index of each pixel used in the following computation
indImage=reshape(1:(M*N), M, N);
indImage=indImage.*gridPoint;

indImage=indImage(iy_range, :);
indImage=indImage(:, ix_range);

% compute the neighbors for each point
LL=2*L+1;

nn=im2col(addborder(indImage, L, L, 0), [LL LL], 'sliding');
valid=find(nn(:)>0); 

nnSize=LL*LL;
centerPixel=(nnSize+1)/2;

cp=repmat(nn(centerPixel, :), nnSize, 1);

ind_col = nn(valid); % linear index for the pixels destination of the ic.
ind_row = cp(valid); % linear index for the pixels origin of the ic.

% Compute intervening countours for the ucm
[Py Px]=ind2sub([M, N], ind_row);
[Qy Qx]=ind2sub([M, N], ind_col);
intContourUCM=compIntContour(iContours, [Px Qx], [Py Qy]);

% Create the affinity matrix. We use a gaussian kernel and a scale factor
% of 0.1 as in the Berkeley code.
a=exp(-(intContourUCM/0.1).^2);
A=sparse(ind_row, ind_col, a, M*N, M*N);


% get the feature vectors
X=getFeatureVectors(I);

  
% remove the temp file
try
  delete(tmpImageFile);
  delete(tmpResFile);
catch expId
  if nargin<3
    disp('Unable to cancel tmp files');
  end
end

% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
