/* AUTORIGHTS
Copyright (C) 2008-2012 Marco Andreetto
      
This file is part of AB_SAMPLER.

AB_SAMPLER is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#include <iostream>
#include <exception>

#include "mex.h"

#include "my_distributions.h"
#include "ab_log.h"
#include "ab_sampler.h"

using namespace std;

// gataway function for mex version
void mexFunction(int n_out, mxArray *out[],
                 int n_in, const mxArray *in[])
{ 
  //
  // The calling paramenters are:
  //   A: the affinity matrix
  //   X: the features of each observation (for global statistcs)
  //   ALPHA: the prior for the Dirichlet distribution, which also gives 
  //          the number of clusters K
  //   NI: number of (complete) iterations for the sampler for each of 
  //       the returned samples
  //   C: the initial state of the sampler, i.e. the labels of each observation
  //      if it is the empty matrix [] then the state is randomly generated
  //   PI: the prior information: fixed labels for certain observations
  //   LAMBDA: The mixture coefficient for the global term in each cluster
  //   W: the vector of visual words associated to the observations
  //   V: the size of the dictionary
  //   ETA: the prior for the Dirichlet prior of the topic distributions.
  //        This is a scalar quantity.
  //   SEED: The seed of the random number generator used in the code
  //
  
  try { 
    // Start logging
    ab_init_log();  
    
    CheckArguments(n_out, out, n_in, const_cast<mxArray **>(in));
    flog<<"Arguments OK"<<endl;
    
    ABSamplerType sampler;
    SetUpSampler(n_in, const_cast<mxArray **>(in), sampler);

    // this is the number of samples
    unsigned S=mxGetN(in[iNI])*mxGetM(in[iNI]);

    // this is the number of observations
    unsigned N=mxGetM(in[iAFF]); // number of observations

    // allocate memory for the output
    out[oSAMPLES]=mxCreateDoubleMatrix(N, S, mxREAL);
    double *p=mxGetPr(out[oSAMPLES]); // output vector
    
    // allocate the memory for the log-likelihood of the samples if required
    double *ll=0;
    if (n_out>oLL){
      out[oLL]=mxCreateDoubleMatrix(1, S, mxREAL);
      ll=mxGetPr(out[oLL]); // output likelihood
    }
     
    // generate the samples
    for(unsigned s=0; s<S; ++s, p+=N, ++ll) {
      unsigned NI=static_cast<unsigned>(mxGetPr(in[iNI])[s]);
      for(unsigned h=0; h<NI; ++h)
        sampler.Run(); // run the sampler for NI iteration

      // collect the sample
      sampler.GetSample(p); // p is the pointer to the output vector
      
      // collect the likelihood if necessary
      if (n_out>oLL)
        (*ll)=sampler.ComputeLogLikelihood();
    }
    
    // return the seed if required
    if (n_out>oSEED) {
      out[oSEED]=mxCreateDoubleMatrix(1, 1, mxREAL);
      (mxGetPr(out[oSEED]))[0]=sampler.GetSeed();
    }
  }
  
  // this catches my exceptions
  catch (const char *exp_msg) { mexErrMsgTxt(exp_msg); }
  
  // this catches all the exceptions from standard libraries
  catch (exception &e) { mexErrMsgTxt(e.what()); }

  // this catches whatever remains
  catch (...) { mexErrMsgTxt("Unidentified exception"); }

  // close the logging
  ab_close_log();
}
