function [SC, L]=ab_gibbs_sampling(A, X, alpha, NI, C, si, lambda, seed)  %#ok<INUSD,STOUT>
%AB_GIBBS_SAMPLING
% Gibbs sampler for the clustering.
%
% [SC, L]=ab_gibbs_sampling(A, X, alpha, NI, C, si, lambda, seed)
% 
% A: set of affinities (NxN)
%
% X: set of observation to clusters (NxF). Each row is a associated with an
%    observation and each entry is a different features. The affinitieas
%    are possibly computed using a larger set of observations. The one in X
%    are only those ones that are used to compute the global term that
%    describes the cluster, and the volume used for the uniform distribution
%    used as garbage collector.
%
% alpha: hyperparameter of the Dirichlet prior distribution.
%        its lenght is the number of clusters. 
%
% NI: number of iterations of the sampler for generating a new sample. If
%     it is a vector then length of NI samples are generated and the number
%     of iterations before creating sample s is NI(s).
%
% C: Initial labeling of the observations. If not present or equal to []
%    then it is randomly generated.
%
% si: side information for the clustering. It is a vector with as many 
%     entries as C. If an entry is zero the corresponding observation is
%     not assigned to any cluster. If an entry is label k then the
%     corresponding observation id affigned to cluster k.
%     If it is not present ot it is equal to [] then no side information 
%     are used by the sampler.
%
% lambda: weights of the global (Gaussian) term in each cluster. 0 no
%         global term, 1 only global term.
%
% iW: the visual words associated with the observations. They are not going
%     to be used if this is empty.
%
% V: the number of words in the dictionary
%
% eta: the hyperparameters for the Dirichet distributions over the visual
%      words. This is a scalar value.
%
% seed: random generator seed. To obtain repeatable exeriments 
%
% SC: new labeling of the date after NI iteration of the sampler
%
% L: loglikelihood of the samples.
%
%
% Notes:
% One iteration of the sampler considers all the N observations
%

% %% check for the coherence of the inputs
% if (rows(A)~=cols(A)) 
%   error('AB:SAMPLER', 'A must be square');
% end
% 
% if (rows(X)~=rows(A))
%   error('AB:SAMPLER', 'Wrong number of observations');
% end
% 
% if (any(alpha<0))
%   error('AB:SAMPLER', 'alpha must be positive');
% end
% 
% N=rows(A);  % number of points and dimension of the space
% K=length(alpha); % number of clusters
% 
% 
% %% set the default arguments:
% 
% % no starting labeling
% if (nargin<5), C=[]; end 
% 
% % no partial labeling
% if (nargin<6), si=[]; end
% 
% % no global term
% if (nargin<7), lambda=0; end
% 
% % fix the state of the random generator if seed is specified
% if (nargin==8), rand('state', seed); end 
% 
% % init the sampler
% if (isempty(C))
%   C=unidrnd(K, N, 1); % vector of assignment;
% else
%   % the labels are provided as input. we need to check that 
%   % they make sense
%   if ( (any(C<1)) || (any(C>K) ) )
%     error('AB:SAMPLER', 'incoherent assignment C');
%   end
% end
% 
% % partial labeling
% if isempty(si), si=zeros(N,1); end  
% 
% if ((rows(si)~=N) || (cols(si)~=1))
%   error('AB:SAMPLER', 'si must be a column vector of length N');
% end
% 
% if any( (si<0) | (si>K) )
%   error('AB:SAMPLER', 'improper prior labeling in si');
% end
% 
% C(si~=0)=si(si~=0); % enforce coherence between the initial labeling and
%                     % the prior information
% 
% 
% %% Run the sampler
% 
% % set the count
% counts=zeros(K, 1);
% for i=1:K, counts(i)=sum(C==i); end
% 
% % we want to have at least a point assigned to each cluster
% if (any(counts==0))
%   error('AB:SAMPLER', 'Unassigned cluster');
% end
% 
% % compute the global statistics for each cluster give the assignment
% if lambda~=0
%   [mu, covariances]=computeGlobalStat(X, C, K);
% else
%   mu=0;
%   covariances=0;
% end
% 
% % start the sampling
% SC=zeros(N, length(NI)); % memory for the samples
% 
% for s=1:length(NI) % number of samples we want to generate
%   
%   for h=1:NI(s) % number of iteration before generating the sample
%   
%     for i=1:N % iterate over all the observations
%     
%       if (si(i)==0) % sample only observation without fixed label
%         counts(C(i))=counts(C(i))-1; % remove the current label
%         
%         dist=CompConditionalDist(A, C, X, i, alpha, counts, mu, covariances, lambda, 0);
%         C(i)=MultinomialRnd(dist); % sample the new value
% 
%         counts(C(i))=counts(C(i))+1; % update the corresponding value
%       end % if
%       
%     end % for
%     
%     if (lambda~=0)
%       % update the global model
%       [mu, covariances]=computeGlobalStat(X, C, K);
%     end
%     
%   end
% 
%   SC(:,s)=C; % add the current state of the label
% end
% 
% 
% %%
% function post=CompConditionalDist(A, C, X, i, alpha, counts, mu, covariances, lambda, V)
% 
% prior=zeros(length(alpha), 1);
% likelihood=zeros(length(alpha), 1);
% 
% for k=1:length(alpha)
%   if (counts(k)==0)
%     likelihood(k)=0;
%   else
%     if lambda==0
%       likelihood(k)=sum(A(C==k, i))/counts(k);
%     else
%       localTerm=sum(A(C==k, i))/counts(k);
%       globalTerm=gaussian(X(i,:), mu(k,:), covariances(:,:,k));
%       likelihood(k)=(1-lambda)*localTerm+lambda*globalTerm;
%     end
%   end
%   prior(k)=counts(k)+alpha(k);
% end
% 
% if V~=0, likelihood(1)=1/V; end
% 
% prior=prior./sum(prior);
% post=likelihood.*prior;
% 
% post=post/sum(post);
% 
% 
% %%
% function [mu, covariances]=computeGlobalStat(X, C, K)
% 
% % we assume a single gaussian distribution with isotropic variance
% 
% Dim=size(X);
% 
% mu=zeros(K, 1);
% covariances=zeros(Dim, Dim , K);
% 
% for k=1:K
%   Xk=X(C==k,:); % entries asigned to the k-th cluster
%   
%   mu(k)=mean(Xk);
%   covariances(:, :, k)=cov(Xk, 1);
% end
% 
% 
% %%
% function p=gaussian(x, mu, S)
% 
% % p=gaussian(x, mu,Sigma,x)
% % x: calculate the probability of x
% % mu, sigma: parameters of the Gaussian
% 
% N=length(mu); % dimension of the space
% y=x(:)-mu; % centered observations
% 
% d2 = (y' *inv(S) * y);
% p = exp(-d2/2) / sqrt((2*pi)^N * det(S));

% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
