function [Cs, Thetas]=ab_gibbs_sampling_ext(A, X, alpha, NI, C, si, lambda)
%AB_GIBBS_SAMPLING_EXT
% Gibbs sampler for the affinity based clustering 
%
% [Cs, Thetas]=ab_gibbs_sampling_ext(A, X, alpha, NI, C,lambda)
% 
% A: set of affinities (NxN)
%
% X: set of observation to clusters (NxF). Each row is a associated with an
%    observation and each entry is a different features. The affinitieas
%    are possibly computed using a larger set of observations. The one in X
%    are only those ones that are used to compute the global term that
%    describes the cluster, and the volume used for the uniform distribution
%    used as garbage collector.
%
% alpha: hyperparameter of the dirichlet prior distribution.
%        its lenght is the number of clusters. 
%
% NI: number of iterations of the sampler for generating a new sample. If
%     it is a vector then length of NI samples are generated and the number
%     of iterations before creating sample s is NI(s).
%
% C: Initial labeling of the observations. If not present or equal to []
%    then it is randomly generated.
%
% si: side information for the clustering. It is a vector with as many 
%     entries as C. If an entry is zero the corresponding observation is
%     not assigned to any cluster. If an entry is label k then the
%     corresponding observation id affigned to cluster k.
%     If it is not present ot it is equal to [] then no side information 
%     are used by the sampler.
%
% lambda: weights of the global (Gaussian) term in each cluster. 0 no
%         global term, 1 only global term.
%
% seed: random generator seed. To obtain repeatable exeriments 
%
% Cs: new labeling of the date after NI iteration of the sampler.
%
% Thetas: samples for the mixing coefficients
%
% Notes:
% This version of the sampler applies a different sampling scheme than the
% one proposed in ICCV. In this case the latent variables \theta are
% sampled as well. This sampling scheme is more suitable to parallel
% implementation.
%


%% check for the coherence of the inputs
if (rows(A)~=cols(A)) 
  error('AB:SAMPLER', 'A must be square');
end

if (rows(X)~=rows(A))
  error('AB:SAMPLER', 'Wrong number of observations');
end

if (any(alpha<0))
  error('AB:SAMPLER', 'alpha must be positive');
end

N=rows(A);  % number of points and dimension of the space
K=length(alpha); % number of clusters


%% set the default arguments:

% no starting labeling
if (nargin<5), C=[]; end 

% no fixed labels
if (nargin<6), si=[]; end

% no global term
if (nargin<7), lambda=zeros(1, K); end


% init the sampler
if (isempty(C))
  C=unidrnd(K, N, 1); % vector of assignment;
else
  % the labels are provided as input. we need to check that 
  % they make sense
  if ( (any(C<1)) || (any(C>K) ) )
    error('AB:SAMPLER', 'incoherent assignment C');
  end
end


% set the side information if necessary
if (~isempty(si))
  if all(si==0)
    % all the si are equal to zero, we do not need them
    si=[];
  else
    for k=1:K, C(si==k)=k; end
  end
end


%% Run the sampler

S=length(NI); % number of samples to drawn
Cs=zeros(N, S); % preallocate the samples of the assignments
Thetas=zeros(K, S); % preallocate the samples of the mixing coeff.

% dimension of the model
K=length(alpha); % number of clusters
N=rows(A); % number of observations


% start the sampling
for s=1:S % number of samples we want to generate
  
  for h=1:NI(s) % number of iteration before generating the sample
    
    % construct the indicator variables and the counts
    U=labels2ind(C, K);
    counts=sum(U);
  
    % sample the new mixture coefficient.
    theta=dirichlet_sample(alpha+counts, 1);
    
    % Given the observations and the mixture coefficients sample a new 
    % assignment of the observations
    
    % likelihood for the non-parameteric term
    Fk=A*U;
    Fk=Fk./repmat(counts, N, 1);

    % compute the parametric term contribution (if present)
    if (lambda~=0)
      for k=1:K
        % Compute the parameters of the Gaussians
        Xk=X(logical(U(:, k)), :);
        mu=mean(Xk);
        S=cov(Xk, 1);

        Fk(:, k)=(1-lambda(k))*Fk(:, k)+lambda(k)*(gauss(mu, S, X));
      end
    end

    % (unnormalized) posterior
    P=Fk.*repmat(theta, N, 1);

    % sample from P multinomial
    C=sample_vector(P')';
    
    % if we have side information then we need to reset them to their
    % fixed value
    if (~isempty(si))
      for k=1:K, C(si==k)=k; end
    end
  end
 
  Cs(:,s)=C; % add the current state of the label
  disp(['Collect sample ', num2str(s)]);
end


% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
