/* AUTORIGHTS
Copyright (C) 2008-2012 Marco Andreetto
      
This file is part of AB_SAMPLER.

AB_SAMPLER is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#include <iostream>
#include <vector>

#include <boost/timer.hpp>
#include <boost/progress.hpp>
#include <boost/format.hpp>

#include "mex.h"

#include "my_distributions.h"
#include "ab_log.h"
#include "ab_sampler.h"
#include "ab_sampler_list.h"

using namespace std;

// gateway function for mex version
void mexFunction(int n_out, mxArray *out[],
                 int n_in, const mxArray *in[])
{ 
  //
  // The calling paramenters are:
  //   A: the affinity matrix
  //   X: the features of each observation (for global statistcs)
  //   ALPHA: the prior for the Dirichlet distribution, which also gives 
  //          the number of clusters K
  //   NI: number of (complete) iterations for the sampler for each of 
  //       the returned samples
  //   C: the initial state of the sampler, i.e. the labels of each observation
  //      if it is the empty matrix [] then the state is randomly generated
  //   PI: the prior information: fixed labels for certain observations
  //   LAMBDA: The mixture coefficient for the global term in each cluster
  //   W: visual words for each pixel
  //   V: number of words in the dictionary
  //   ETA: prior huperparameter for the topic distributions.
  //   SEED: The seed of the random number generator ised in the code
  //

  try {
    // Start logging
    ab_init_log();
    
    CheckListArguments(n_out, out, n_in, const_cast<mxArray **>(in));
    flog<<"Arguments OK"<<endl;
  
#ifdef _USE_BACKGROUND_
    flog<<"Using background segment"<<endl;
#endif

    vector<ABSamplerTypePr> samplerImages; // contains all the pointers to
                                           // the sampler of each image
    // set all the sampler
    SetUpAllSamplers(n_in, const_cast<mxArray **>(in), samplerImages);
    flog<<"All samplers are (almost) ready"<<endl;
    
    unsigned L=(samplerImages[0])->GetL(); // number of visual words per observation
    unsigned V=(samplerImages[0])->GetV(); // number of visual words in all dictionary
    unsigned K=(samplerImages[0])->GetK(); // number of topic
    
    flog<<K<<" topics, and "<<V<<" visual words (all types)."<<endl;
    flog<<L<<" types of words per observations."<<endl;

    // we have to allocate the set of tables and vector. One element for 
    // each type of visual word. After creating this pointers we need 
    // to create the actual objects.
    vector<vector<unsigned> *> allTopicTables(L);
    vector<table<unsigned> *> allWordTopicTables(L);
    for(unsigned l=0; l<L; ++l) {
      allWordTopicTables[l]=new table<unsigned>(V, K);
      allTopicTables[l]=new vector<unsigned>(K, 0);
    }
    
    // setup the topic tables if necessary
    if ( (samplerImages[0])->UseVisualWords() ) {
      SetTopicTables(samplerImages, allWordTopicTables, allTopicTables);
      flog<<"The topic tables are ready"<<endl;
    }
    else flog<<"Not using the patches"<<endl;
        
    // allocate memory for the output. The output is a cell array 
    // containing all the samples.
    reserveOutput(const_cast<mxArray **>(in), out[oSAMPLES]);
    flog<<"Memory reserved for output results"<<endl;
    flog<<endl;
    
    // this is the number of samples.
    unsigned S=mxGetN(in[iNI])*mxGetM(in[iNI]);
        
    // allocate the memory for the log-likelihood of the samples and compute 
    // the log-likelihood of the initian random assignment if required
    double *ll=0;
    if (n_out>oLL){
      out[oLL]=mxCreateDoubleMatrix(1, S, mxREAL);
      ll=mxGetPr(out[oLL]); // output likelihood
    }
    
    flog<<"Start the sampler"<<endl;
    unsigned iter=0; 
    boost::timer sampler_timer;
    
    // iterate over all the samples
    for(unsigned s=0; s<S; ++s, ++ll){

      // number of iterations before collecting a samples
      unsigned NI=static_cast<unsigned>(mxGetPr(in[iNI])[s]);
      
      flog<<"Generating sample "<<(s+1)<<"/"<<S
          <<". Number of iterations "<<NI;
      // Timing and progress for this iteration.
      // restart the timer for this iterations
      sampler_timer.restart(); 
      // create a new progress_display object for this iterations
      boost::progress_display progress_bar(NI, flog);
      
      for(unsigned h=0; h<NI; ++h, ++progress_bar, ++iter){
        // Run all the sampler for one iteration
        for(unsigned l=0; l<samplerImages.size(); ++l)
          (samplerImages[l])->Run();
      }

      CollectSamplesForAllImages(samplerImages, s, out[oSAMPLES]);

      flog<<"Total time: "<<sampler_timer.elapsed()<<endl<<endl; 
    }
    
    // check if we have to return also the random seeds of each sampler
    if (n_out>oSEED)
      CollectSeedsFromSamplers(samplerImages, out[oSEED]);
    
    /*if (n_out>oTABLE)
      GetWordTopicTable(wordTopicTable, out[oTABLE]);*/
    
    // deallocate all the sampler objects
    for(unsigned h=0; h<samplerImages.size(); ++h)
      delete (samplerImages[h]);
  }
  
  // this catches my exceptions
  catch (const char *msg) { mexErrMsgTxt(msg); }

  // exception from the standard library
  catch (exception &e) { mexErrMsgTxt(e.what()); }

  // this catches whatever remains
  catch (...) { mexErrMsgTxt("Unidentified exception"); }
  
  // close the logging
  ab_close_log();
}
