function Cs_all=ab_gibbs_sampling_list_gc(As, Ws, alpha, NI, V_s, eta_s)

%AB_GIBBS_SAMPLING_LIST_EXT
%
% Non-collapesed Gibbs sampler for segmenting multiple images 
% (collections). for each image a "clutter" topic is given to collect the
% observations that are not shared across the collection. The topic
% distribution for the clutter is sampled from a Dirichlet distribution of
% hyperparameter 'eta'.
%
% Cs_all=ab_gibbs_sampling_list_ext(A, W, alpha, NI, Vs, eta_s)
% 
% All the input, except NI, are cell arrays. Each cell contains the
% corresponding value for the ab_gibbs_sampling function. For example A{1}
% contains the affinity matrix for the first image in the collection
%
% Use 'help ab_gibbs_sampling' for more details about the input.
%
% NI: number of iterations of the sampler for generating a new sample. If
%     it is a vector then length of NI samples are generated and the number
%     of iterations before creating sample s is NI(s).
%
% Cs_all: is a cell array containing all the samples for all the images. For 
%         example SC{1} contains the samples for the first image.
%

M=length(As);   % number of images in the collection
V=V_s{1};
  

%% initial assignment of observations to clusters (Random)
Cs_all=cell(1, M);
Cs=cell(1, M);
thetas=cell(1, M);

% dimension of the model
K=length(alpha{1}); % number of clusters, the last one is the "clutter".

% initial segmentation
for m=1:M, Cs{m}=unidrnd(K, rows(As{m}), 1); end

% initial value for the topic hidden variables
[phi, rho]=sampleTopics(Cs, Ws, V, K, eta_s{1});


%% Run the sampler

S=length(NI); % number of samples to drawn

% start the sampling
for s=1:S % number of samples we want to generate
  
  tic;
  for h=1:NI(s) % number of iteration before generating the sample
    
    % since the topic distribution over visual words are not integrated out
    % we can sample images independently. 
    
    for m=1:M
      % sample image
      [Cs{m}, thetas{m}]=AB_sample_single_image(As{m}, Ws{m}, Cs{m}, ...
                                                alpha{m}, phi, rho{m});
    end
    
    % sample the topic distribution
    phi=sampleTopics(Cs, Ws, V, K, eta_s{1});
    
    putDot(h);
  end
  disp(' ');
  ttt=toc;
  
  Cs_all=compose_samples(Cs_all, Cs);
  disp(['Generated sample ', num2str(s), '/', num2str(S), ...
    ' in ', num2str(NI(s)), ' iter. Total time: ', num2str(ttt), ' sec.']);
end
disp('Sampling done');



function [C, theta]=AB_sample_single_image(A, W, C, alpha, phi, rho)

% construct the indicator variables and the label counts
K=length(alpha);
N=rows(A);

U=labels2ind(C, K);
counts=sum(U);

% sample the new mixture coefficient for this image
theta=dirichlet_sample(alpha+counts, 1);

% Given the observations and the mixture coefficients sample a new 
% assignment of the observations

% likelihood for the non-parameteric term
Fk=A*U;

% (unnormalized) posterior
phi=[rho(:) phi(:, 2:K)];
P=Fk.*phi(W, :).*repmat(theta./(counts+eps), N, 1)+eps;

% sample from P multinomial
C=sample_vector(P')';



function [phi, rho]=sampleTopics(Cs, Ws, V, K, eta)

topic_counts=compTopicCounts(Ws, Cs, V, K);

% iterate over all the images and get the sample from the posterior of rho.
rho=cell(1, length(Ws));
for m=1:length(Ws)
  W_clutter=Ws{m}(Cs{m}==(K+1));
  rho{m}=dirichlet_sample(hist(W_clutter, 1:V)+eta)';
end

% sample the shared topics.
phi=zeros(V, K);
for k=1:K
  phi(:, k)=dirichlet_sample(topic_counts(k, :)+eta)';
end



function Cs_all=compose_samples(Cs_all, Cs)

M=length(Cs_all);
if (M ~= length(Cs))
  error('Dimension mismatch for the number of samples');
end

for m=1:length(Cs)
  Cs_all{m}=[Cs_all{m}, Cs{m}];
end

% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
