/* AUTORIGHTS
Copyright (C) 2008-2012 Marco Andreetto
      
This file is part of AB_SAMPLER.

AB_SAMPLER is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#include <iostream>
#include <cstdlib>
#include <cstring>
#include <vector>
#include <string>
#include <iterator>
#include <sstream>
#include <boost/asio.hpp>

#include <boost/numeric/ublas/io.hpp>

#include "ublas_utils.h"

#include "my_parallel_library.h"

//
//define the types from ublas
//
typedef boost::numeric::ublas::matrix<double> ublas_matrix;
typedef boost::numeric::ublas::vector<double> ublas_vector;
typedef boost::numeric::ublas::zero_vector<double> ublas_zero_vector;
typedef boost::numeric::ublas::zero_matrix<double> ublas_zero_matrix;


// 
// Macros for the command line arguments
// 

#define COMMAND 0
#define NSLAVES 1

#define NARG NSLAVES+1 // total number of arguments

// 
// TCP port number for all the communications. This is fixed to 5173 because 
// it is the only port currently not used by any other service and not under
// firewall control. The constant is a string since we were passing a string 
// as command argument.
//

const char *PORT="5173";



// all the remaining arguments are the names of the servers and their ID

using namespace std;
using boost::asio::ip::tcp; // we use a tcp socket


const char *usageMessage=
"Usage: <command> <port> <num_client> ";

void CheckArguments(int argc, char **argv)
{
  if (argc<NARG){
    cerr<<"Too few arguments"<<endl;
    cerr<<usageMessage<<endl;
    exit(-1);
  }
  
  cerr<<"Arguments OK"<<endl;
}


void AcceptSlaveConnections(boost::asio::io_service &io_service, unsigned port,
                                vector<socket_pr> &slave_socket)
{
  // create an acceptor for the port specified by the user
  tcp::acceptor acceptor(io_service, tcp::endpoint(tcp::v4(), port));
  
  // create as many connections with the slaves as needed
  for(unsigned h=0; h<slave_socket.size(); ++h) {
    // create a new socket
    socket_pr sp(new tcp::socket(io_service));
    
    // accept the a new connection from a slave
    acceptor.accept(*sp);
    
    // put the new connection (the socket pointer) in 'slave_socket'
    slave_socket[h]=sp;
  
    // repeat the loop for a new connection
    cerr<<"Get connection number : "<<h<<endl;
  }
  
  // here we have all the connection we need. At the end of the function the
  // acceptor will be destroyed. This could be a possible problem

  cerr<<"All the connections are opened"<<endl;
}


unsigned GetSpaceDimension(vector<socket_pr> &slave_socket)
{
  //
  // collect the dataspace dimensionality for all the slaves
  //
  vector<unsigned> slaveDim(slave_socket.size());
  for(unsigned h=0; h<slave_socket.size(); ++h){
    
    // read from the h-th slave the unsigned number that represent the number 
    // of dimensions.
    unsigned dim;
    boost::asio::read(*slave_socket[h], 
        boost::asio::buffer(reinterpret_cast<char *>(&dim), sizeof(unsigned)));
    
    slaveDim[h]=dim; // save the results
  }
  
  
  //
  // Print the dimension for each slave and check that there are 
  // all the same (equal to the first one).
  //
        
  for(unsigned h=0; h<slaveDim.size(); ++h){
    cerr<<"Slave number "<<h
        <<", dimension of the space "<<slaveDim[h]<<endl;
    
    if (slaveDim[h]!=slaveDim[0])
      throw exception();
  }

  cerr<<"Dimension for this experiment: "<<slaveDim[0]<<endl;
  
  // return the dimension of the space
  return slaveDim[0];
}


bool ReceiveAndCombineData(vector<socket_pr> &slave_socket, 
                           vector<unsigned> &slaveSeqNum, 
                           unsigned masterSeqNum,
                           unsigned dim)
{
  //
  // Allocate variable for the data from the slaves and the for the 
  // accumulator. All the vriables are initialized to 0.
  //
  ublas_vector SX_all=ublas_zero_vector(dim);
  ublas_vector SX_slave=ublas_zero_vector(dim);
               
  ublas_matrix SX2_all=ublas_zero_matrix(dim);
  ublas_matrix SX2_slave=ublas_zero_matrix(dim);
               
  unsigned Nd_all(0), Nd_slave(0);

  messageType msg;
  
  //cerr<<"========================================================="<<endl;
  
  //
  // Iterate over all the slave, get all their messages, and combine them
  //
  unsigned active_connections=slave_socket.size();
  for(unsigned h=0; h<slave_socket.size(); ++h){
    
    try {
 
      // 
      // read a message from the socket
      //
        
      messageType msg;
      boost::asio::read(*slave_socket[h], 
          boost::asio::buffer(reinterpret_cast<char *>(&msg), MSG_SIZE));
          
      /*cerr<<"Receiving message "<<msg.seqNumber<<" from client "<<h<<endl;
      cerr<<"Expecting message "<<slaveSeqNum[h]<<endl;
      cerr<<"Master sequence number "<<masterSeqNum<<endl;*/

      // check that the sequence number is the expected one.
      if (msg.seqNumber!=slaveSeqNum[h]) {
         cerr<<"Invalid sequence number!!!"<<endl;
         cerr<<msg.seqNumber<<" "<<slaveSeqNum[h]<<endl;
         cerr<<"A message has been lost"<<endl;
      }
    
      if (msg.seqNumber!=masterSeqNum) {
         cerr<<"Invalid sequence number!!!"<<endl;
         cerr<<"Expecting "<<masterSeqNum<<endl;
      }

      //
      // Extract statistcs from the message and add them to the all the others
      // 
      
      ExtractStats(msg, SX_slave, SX2_slave, Nd_slave);

      SX_all+=SX_slave;
      SX2_all+=SX2_slave;
      Nd_all+=Nd_slave;


      // increase the slave sequence number
      ++slaveSeqNum[h];
    }
    
    catch (exception &e)
    {
      // Check if we got and end of file exception. 
      // This means the socket has been closed
      if (strcmp(e.what(), "End of file") == 0){
        cerr<<e.what()<<", from slave "<<h<<endl;
        active_connections--;
      }
      else throw; // rethrow the exception if it is not EOF
    }
  }
  
  // 
  // Send back the global statistics.
  //

  // Compact the statistics for all the data in a single message
  msg.seqNumber=masterSeqNum;
  CompactStats(SX_all, SX2_all, Nd_all, msg);
  
  // broadcast the message to all the clients
  for(unsigned h=0; h<slave_socket.size(); ++h)
    boost::asio::write(*slave_socket[h], 
        boost::asio::buffer(reinterpret_cast<const char *>(&msg), MSG_SIZE));  


  // increase the master sequence number
  ++masterSeqNum;
  
  return (active_connections!=0);
}


int main(int argc, char **argv)
{
  try
  {
    // check for consistency call
    CheckArguments(argc, argv);
        
    // main object for the asio io_service
    boost::asio::io_service io_service;

    // Accept as many connections as the number specified by the user
    vector<socket_pr> slave_socket(atoi(argv[NSLAVES]));
    AcceptSlaveConnections(io_service, atoi(PORT), slave_socket);

    // Get a message from all the slaves indicating the dimension of the 
    // space where the data is defined. If there is any inconsistency then
    // we generate an error and quit.
    unsigned dim=GetSpaceDimension(slave_socket);
    
    // create a vector for the message sequences. This is used just as a
    // sanity check that the communications are in sync.
    vector<unsigned> slaveSeqNum(slave_socket.size(), 0);
    
    // enter the main loop. 
    unsigned seqNumber=0; // initial sequence number 
    
    // iterate until all the client has terminated
    while (ReceiveAndCombineData(slave_socket, slaveSeqNum, seqNumber++, dim));
  }
  
  catch (exception& e)
  {
    cerr << e.what() << endl;
  }
  
  return 0;
}

