/* AUTORIGHTS
Copyright (C) 2008-2012 Marco Andreetto
      
This file is part of AB_SAMPLER.

AB_SAMPLER is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#include <iostream>

#include <vector>
#include <string>
#include <sstream>

#include "ublas_utils.h"

#include "mex.h"
#include "mat.h"

#include "my_distributions.h"
#include "my_parallel_library.h"
#include "ab_sampler.h"
#include "ab_sampler_list.h"

using namespace std;
using boost::asio::ip::tcp; // we use a tcp socket

// global const variable for the number of shared segments
const unsigned NUM_SHARED_SEG=1;


// 
// Macros for the command line arguments
// 

#define COMMAND 0
#define MASTER  1
#define PROC_ID 2
#define IN_DATA 3
#define OUT_RES 4

#define NARG 5

const char* usageMessage=
"Usage: <command> <master> <ID> <input_mat_file> <output_mat_file>";


// 
// TCP port number for all the communications. This is fixed to 5173 because 
// it is the only port currently not used by any other service and not under
// firewall control. The constant is a string since we were passing a string 
// as command argument.
//

const char *PORT="5173";

// global variable for the sequence number of the master
unsigned seqNumber=0;

mxArray *LoadMxArrayFromMATFile(MATFile *matFile, const char *name)
{
  mxArray *data=matGetVariable(matFile, name);
  if (data==NULL) {
    cerr<<"Unable to read variable '"<<name<<"' from mat file"<<endl;
    exit(-1);
  }
  
  return data;
}


void DisplayCallingArguments(int argc, char **argv)
{
  // Display the calling arguments of the program
  
  cerr<<"Command"<<endl;
  cerr<<argv[0]<<endl<<endl;
  
  for(int h=1; h<argc; ++h){
    cerr<<"Argument "<<h<<endl;
    cerr<<argv[h]<<endl;
  }
  
  cerr<<"------------------------------------------------------------"<<endl;
  cerr<<endl<<endl;
}


// This function creates the mxArray vectors for the input output.
void SetInputOutput(int argc, char **argv, 
                   int &n_in, mxArray **&in, 
                   int &n_out, mxArray **&out)
{
  // 
  // Check the command line arguments
  // 
  
  if (argc!=NARG){
    cerr<<usageMessage<<endl;
    exit(-1);
  }
  
  //
  // Read from a mat file the input arguments and store them in vector of 
  // mxArray. In this way we can use the same code of before.
  // 
    
  // the data for all the sampler are in a mat file.
  // The name of this file in in argv[IN_DATA].
  MATFile *matDataFile=matOpen(argv[IN_DATA], "r");
  if (matDataFile==NULL){
    cerr<<"Unable to open MAT file: "<<argv[IN_DATA]<<endl;
    exit(-1);
  }
  
  // all the input variables
  vector<mxArray *> inputArg;
  
  inputArg.push_back(
      LoadMxArrayFromMATFile(matDataFile, "As"));      // affinities
  inputArg.push_back(
      LoadMxArrayFromMATFile(matDataFile, "Xs"));      // feature vector
  inputArg.push_back(
      LoadMxArrayFromMATFile(matDataFile, "alpha"));   // alpha prior
  inputArg.push_back(
      LoadMxArrayFromMATFile(matDataFile, "NI"));      // Number of iter.
  inputArg.push_back(
      LoadMxArrayFromMATFile(matDataFile, "Ci"));      // Initial state
  inputArg.push_back(
      LoadMxArrayFromMATFile(matDataFile, "si"));      // side info
  inputArg.push_back(
      LoadMxArrayFromMATFile(matDataFile, "lambda"));  // weight of the global term

  // close the mat file
  matClose(matDataFile);
  
  
  //
  // Create multiple sampler and set them using the different values of 
  // the affinities and observation cell arrays.
  //
  
  // create and fill the input vector
  n_in=inputArg.size();
  in=new mxArray*[n_in];
  copy(inputArg.begin(), inputArg.end(), in);
  
  
  //
  // Set the output arguments
  //
  
  n_out=1; // for the moment we use just one output (the samples)
  out=new mxArray *[1];
}


void SendDimensionToMaster(const vector<ABSamplerTypePr> &samplerImages, 
                           tcp::socket &socket)
{
  //
  // iterate over all the sampler and check that they have the same 
  // space dimesion.
  //
  
  unsigned F=(samplerImages[0])->GetF();
  for(unsigned h=0; h<samplerImages.size(); ++h)
    if (F!=(samplerImages[h])->GetF())
      throw "Mismatching of feature space";
  
  //
  // Send this value to the master
  // 
  boost::asio::write(socket, 
      boost::asio::buffer(reinterpret_cast<const char *>(&F), sizeof(unsigned)));
}


void ShareStatsAmongImages(const vector<ABSamplerTypePr> &samplerImages, 
                           tcp::socket &socket)
{
  //
  // Collect the observations for the shared clusters of all the images
  //
  
  //const unsigned commonSeg=0;

  ublas_vector SX_all=(samplerImages[0])->GetParametricTerm_SX(0);
  ublas_matrix SX2_all=(samplerImages[0])->GetParametricTerm_SX2(0);
  unsigned Nd=(samplerImages[0])->GetParametricTerm_Nd(0);
  
  messageType msg;
  
  for(unsigned commonSeg=0; commonSeg<NUM_SHARED_SEG; ++commonSeg){
    
    // iterate over all the segments that are shared

    // In each sampler there is the sum of the observation 'SX=sum(X_i)' and
    // the sum of all the observations squared 'SX2=sum( (X_i)' * (X_i) ).
    // We can use this information to merge the statistics from all the 
    // images in a fast way.

    // init the statistics for all the images with the ones form the first
    // image.
    ublas_vector SX_local=(samplerImages[0])->GetParametricTerm_SX(commonSeg);
    ublas_matrix SX2_local=(samplerImages[0])->GetParametricTerm_SX2(commonSeg);
    unsigned Nd_local=(samplerImages[0])->GetParametricTerm_Nd(commonSeg);

    // get all the other images (from 1 to the last one)
    for(unsigned h=1; h<samplerImages.size(); ++h){
      SX_local+=(samplerImages[h])->GetParametricTerm_SX(commonSeg);
      SX2_local+=(samplerImages[h])->GetParametricTerm_SX2(commonSeg);
      Nd_local+=(samplerImages[h])->GetParametricTerm_Nd(commonSeg);
    }
    
    //
    // Communication with the master
    //
    
    // Send all the local values to the Master for combining them.
    msg.seqNumber=seqNumber;
    CompactStats(SX_local, SX2_local, Nd_local, msg); // Marshalling
    boost::asio::write(socket, 
        boost::asio::buffer(reinterpret_cast<const char *>(&msg), MSG_SIZE));  

    // Receive all the information from the master about the global terms
    boost::asio::read(socket, 
       boost::asio::buffer(reinterpret_cast<char *>(&msg), MSG_SIZE));
    ExtractStats(msg, SX_all, SX2_all, Nd); // Marshalling
       
    // check that the message we got back has the correct sequence number
    if (msg.seqNumber!=seqNumber) {
      cerr<<"Error communication out of sequence"<<endl;
      exit(-1);
    }
    
    // increase the seqNumber
    ++seqNumber;

    //
    // End of communication with the server
    //
    
    // set the global term of the common cluster for each image (each 
    // sampler object). The common cluster has the usual common covariance 
    // matrix but the mean component is specific of each component
    
    for(unsigned h=0; h<samplerImages.size(); ++h) {
      // set the global term using the mean of the h-th sampler
      (samplerImages[h])->SetParametricTerm(commonSeg, SX_all, SX2_all, Nd);      
    }
  }
}


void SaveOutputSample(int argc, char **argv, int n_out, mxArray **out)
{
  // save all the sample in argv[OUT_RES]
  
  // open the file
  MATFile *outSampleFile=matOpen(argv[OUT_RES], "w");
  if (outSampleFile==NULL){
    cerr<<"Unable open file"<<argv[OUT_RES]<<endl;
    exit(-1);
  }
  
  // save the variable
  matPutVariable(outSampleFile, "CSample", out[oSAMPLES]);
  
  // close the file
  matClose(outSampleFile);
}


// main function 
int main(int argc, char **argv)
{ 
  // Display the calling arguments on the error stream. This stream is saved
  // in the LOG file.
  DisplayCallingArguments(argc, argv);


  // There are only two differences between this 'slave' version and the
  // stand alone one of 'ab_sampling_list':
  // 
  // 1 - the input output arguments are read from and written in MAT files
  // 
  // 2 - the sharing part require network comunication with the slave
  //
  
  int n_in, n_out; // how many input and output arguments
  mxArray **in, **out; // the two vectors of the input and output arguments
  SetInputOutput(argc, argv, n_in, in, n_out, out);
  
#ifdef _NO_SHARE_PARAM_
  cerr<<"Running multiple sampler WITHOUT sharing"<<endl;
#endif

  //
  // Set up the communication with the master.
  //
  
  // main object for the io_service
  boost::asio::io_service io_service;

  // resolve the connection to the server/port
  tcp::resolver resolver(io_service);
  tcp::resolver::query query(tcp::v4(), argv[MASTER], PORT);
  
  // connect to the master using 'socket'
  tcp::socket socket(io_service);
  socket.connect(*resolver.resolve(query));
    
  //
  // The calling paramenters are:
  //   A: the affinity matrix
  //   X: the features of each observation (for global statistcs)
  //   ALPHA: the prior for the Dirichlet distribution, which also gives 
  //          the number of clusters K
  //   NI: number of (complete) iterations for the sampler for each of 
  //       the returned samples
  //   C: the initial state of the sampler, i.e. the labels of each observation
  //      if it is the empty matrix [] then the state is randomly generated
  //   PI: the prior information: fixed labels for certain observations
  //   LAMBDA: The mixture coefficient for the global term in each cluster
  //   USEGB: A flag indicating if the first cluster must be used as garbage
  //          collector (uniform distribtion over the space of observation)
  //   SEED: The seed of the random number generator ised in the code
  //
  
  CheckListArguments(n_out, out, n_in, in);
  cerr<<"Arguments correct"<<endl;

  try {
    vector<ABSamplerTypePr> samplerImages; // contains all the pointers to
                                           // the sampler of each image
    // set all the sampler
    SetUpAllSamplers(n_in, in, samplerImages);
    cerr<<"All the samplers are ready"<<endl<<endl;
    
    // comunicates the dimenson to the master
    SendDimensionToMaster(samplerImages, socket);
    cerr<<"The master has accepted the proposed number of dimensions"<<endl;
    
    // allocate memory for the output. The output is a cell array 
    // containing all the samples.
    reserveOutput(in, out[oSAMPLES]);
    cerr<<"Memory for output allocated"<<endl<<endl;
    
    // this is the number of samples. 
    unsigned S=mxGetN(in[iNI])*mxGetM(in[iNI]);
    
    cerr<<"Start all the samplers"<<endl;
    for(unsigned s=0; s<S; ++s){
      
      // number of iterations before collecting a samples NI(s)
      unsigned NI=static_cast<unsigned>(mxGetPr(in[iNI])[s]);
      cerr<<"Generating sample "<<s+1<<" out of "
          <<S<<" in "<<NI<<" iterations. Total time :";
      double t=clock();
      for(unsigned h=0; h<NI; ++h){
        // Run all the sampler for one iteration
        for(unsigned l=0; l<samplerImages.size(); ++l) {
          (samplerImages[l])->Run();
        }
        
        // Compute the statistics of the global term for the segment 
        // common to all the images. And fix them in the all the samplers.
        
#ifndef _NO_SHARE_PARAM_
        // This switch enables the sharing of statistics between different 
        // samplers.
        ShareStatsAmongImages(samplerImages, socket);
#endif
      }
      cerr<<(clock()-t*1.0)/CLOCKS_PER_SEC<<" sec."<<endl;
      
      CollectSamplesForAllImages(samplerImages, s, out[oSAMPLES]);
      cerr<<"Collected sample "<<s+1<<"/"<<S<<endl;
    }
    
    // deallocate all the sampler objects
    for(unsigned h=0; h<samplerImages.size(); ++h)
      delete (samplerImages[h]);
    
    cerr<<"Sampler Run succesfully"<<endl;
  }

  catch (const char *msg) { std::cerr<<"Exception: "<<msg<<endl; }
  
  catch (std::exception &e) { std::cerr<<"Catch all: "<<e.what()<<std::endl; }
  
  // save the sample results
  SaveOutputSample(argc, argv, n_out, out);
  
  return 0;
}
