/* AUTORIGHTS
Copyright (C) 2008-2012 Marco Andreetto
      
This file is part of AB_SAMPLER.

AB_SAMPLER is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#include <iostream>
#include <fstream>
#include <ctime>

#include <vector>
#include <iterator>
#include <algorithm>
#include <numeric>

#include <boost/math/special_functions/gamma.hpp>

#include "mex.h"
#include "matrix.h"

#include "my_distributions.h"
#include "ab_log.h"
#include "ab_sampler.h"

using namespace std;
using namespace boost::numeric;


// 
// Definition of the methods of the class ABSampler
// 

void ABSamplerType::SetSeed(unsigned seed)
{
  used_seed=seed;
  
  // construct the random generator.
  // The static_cast avoids overloading problems under windows.
  if (rg_engine_pt==0)
    rg_engine_pt=new rndGenType(static_cast<unsigned long>(used_seed));
   else
    rg_engine_pt->seed(static_cast<unsigned long>(seed));
  
  // and the uniform random variable used by the sampler
  delete u01; 
  u01=new u01RndVar(*rg_engine_pt);
}


void ABSamplerType::SetPriorHyperParameters(const double *inAlpha, unsigned n)
{
  K=n; // set the number of clusters
  
  // allocate memory for the Dirichlet hyperparameter, the cumulative 
  // posterior and the likelihood. This allocation is done one for all.
  // All this vectors have now all the components they need
  alpha.resize(K, 0);
  allCounts.resize(K, 0);
  
  cumPost.resize(K, 0); 
  likelihood.resize(K, 0); 
  prior.resize(K, 0);
  
  sum_alpha=0;
  
  // check the the input is non-negative
  if (count_if(inAlpha, inAlpha+K, bind2nd(less<double>(), 0)) > 0)
    throw "The Dirichlet hyperparameter alpha must be non-negative";
  
  // copy the input 
  copy(inAlpha, inAlpha+K, alpha.begin());
  
  // accumulate the values in alpha
  sum_alpha=accumulate(alpha.begin(), alpha.end(), 0.0);
}


// Set the Gaussian Inv-Wishart prior of the parametric term. 
// See "Bayesian Data Analysis"
void ABSamplerType::SetInvWishart(const ublas_vector &inMu, double inPrec, 
                                  unsigned inNu, const ublas_matrix &inS)
{
  //
  // Set the hyperparameters into the sampler object.
  //
  
  mu_prior=inMu;
  precision_prior=inPrec;
  nu_prior=inNu;
  S_prior=inS;
  
  
  //
  // Check that all the hyperparameters of the Normal-Wishart prior are 
  // valid.
  // 
  
  if (precision_prior<=0)
    throw "Non-positive precision for Normal-Wishart prior";
  
  if (nu_prior<=0)
    throw "Invalid number of degree of freedom for Wishart prior";
  
  if (ublas_utils::det(S_prior)<=0)
    throw "Invalid scale matrix for InvWishart prior";
  
  
  //
  // set the Normal-InvWishart hyperparameter inside each 
  // parametric distribution
  //
  
  // create the parametric terms objects
  paramDist.resize(K);
  
  // set the hyperparamteres
  for(unsigned k=0; k<K; ++k)
    paramDist[k].SetInvWishart(mu_prior, precision_prior, nu_prior, S_prior);
    
  // The parametric terms are ready. We need to fix their statistics before we
  // can use them.
}


void ABSamplerType::SetDiagonalAffinities()
{
  // save the diagonal elelments for computing the likelihood
  
  // allocate the memory
  Ad.resize(N, 0);
  
  if (sparseAffinity) {
    //scan all the cols
    for(unsigned h=0; h<N; ++h) {
    
      // scan the rows with non zero elements
      for(unsigned n=0; n<col_dim[h]; ++n)
        if (row_info[h][n]==static_cast<mwIndex>(h))
          Ad[h]=A[h][n];
    }
  }
  else {
    // scan all the cols
    for(unsigned h=0; h<N; ++h)
      Ad[h]=A[h][h];
  }
}


void ABSamplerType::SetAffinityMatrix(const mxArray *am)
{
  // Set the number of observations
  N=mxGetN(am);
  
  // allocate the memory for the pointers
  A.resize(N);
  
  // fill in the entries
  if (!mxIsSparse(am)){

    for(unsigned i=0; i<N; ++i)
      A[i]=mxGetPr(am)+i*N;
    
    sparseAffinity=false;
  }
  else{

    // auxiliary vectors for the sparse case
    col_dim.resize(N); 
    row_info.resize(N);
    
    mwIndex *Jc=mxGetJc(am);
    for(unsigned i=0; i<N; ++i){
      A[i]=mxGetPr(am)+Jc[i];
      row_info[i]=mxGetIr(am)+Jc[i];
      col_dim[i]=Jc[i+1]-Jc[i];
    }
    
    sparseAffinity=true;
  }
  
  // Given the affinity matrix we need to extract the diagonal elements and 
  // store them in the 'Ad' vector.
  SetDiagonalAffinities();
  
  // the affinities are ready
  affinitiesReady=true;
}


inline 
void ABSamplerType::ComputeCounts()
{
  //init to zero all the entries
  fill(allCounts.begin(), allCounts.end(), 0);
 
  // set the counts
  for(unsigned i=0; i<N; ++i) ++allCounts[C[i]];
}


inline
void ABSamplerType::AddObservation(unsigned i)
{
  // update all the counts
  ++allCounts[C[i]];
  
  if (useVisualWords)

    for(unsigned l=0; l<L; ++l) {
      // update the counts of the word/topic tables
      ++(*wordTopicPr[l])(W(l, i), C[i]);
      ++(*topicCountPr[l])[C[i]];

      // if we are updating the topic with 0 label
      if (C[i]==0) {
        ++((backgroundHist[l])[W(l, i)]);
        ++backgroundSize[l];
      }
    }
}


inline
void ABSamplerType::RemoveObservation(unsigned i)
{
  // update all the counts
  allCounts[C[i]]--;
  
  if (useVisualWords)
    for(unsigned l=0; l<L; ++l) {
      // update the counts of the word/topic tables
      --(*wordTopicPr[l])(W(l, i), C[i]);
      --(*topicCountPr[l])[C[i]];

      // if we are updating the topic with 0 label
      if (C[i]==0) {
        --backgroundHist[l][W(l, i)];
        --backgroundSize[l];
      }
    }
}  


void ABSamplerType::SetObservationVectors(const mxArray *x)
{ 
  // if a null pointer is given then we do not set the observations
  if (x==0) {
    return;
  }
  
  // if the mxArray if empty then we do not have any observation vector
  if (mxGetM(x)*mxGetN(x)==0) {
    return;
  }

  F=mxGetN(x); // dimension of the space
  ublas_matrix tmp=ublas_utils::ConvertMxArray(x);
  
  // copy the data to the 
  X.resize(N);
  for(unsigned i=0; i<N; ++i){ // iterate over all the observations
    X[i].resize(F);
    for(unsigned j=0; j<F; ++j)
      (X[i])[j]=tmp(i, j);
  }
  
  observationsSet=true;
}


void ABSamplerType::SetVisualWords(const mxArray *x)
{
  W.resize(0, 0);

  // no data specified, we do not use the visual words in the sampler.
  if (x==0) return; 
  
  // check if the mxArray is empty: in this case we do not use the visual words
  if (mxGetN(x)*mxGetM(x)==0) return;
 
  
  //
  // we use the visual words in the sampler
  //
  
  useVisualWords=true; 
  
  // get the number of different visual words for each observation. This is 
  // the number of columns in the input matrix for the visual words.
  L=mxGetN(x); // set the number of types of visual word
  
  
  W.resize(L, N);
  double *d=mxGetPr(x);
  
  for(unsigned l=0; l<L; ++l) {
    // iterate over all the columns and insert the corresponding 
    // vector of visual word into a row of the table W.
    for(unsigned h=0; h<N; ++h, ++d) {
      // subtruct one because of the different convention between MATLAB and C.
      if ((*d)<=0)
        throw "All the visual words labels must be positive";
      
      W(l, h)=static_cast<unsigned>(*d)-1;
    }
  }
}


void ABSamplerType::SetDictionarySize(const double dictionarySize)
{
  V=static_cast<unsigned>(dictionarySize);
  
  if ( (!useVisualWords) && (V>0) )
    flog<<"Warning: we are not using the visual words, but the dictionary size is positive"<<endl;
  
  // verify that all the words are in the dictionary, i.e. that no word 
  // has a label larger or equal to the dictionary size. We do this only if 
  // we have the visual words.
  if (useVisualWords)
    for(unsigned l=0; l<L; ++l)
      for(unsigned h=0; h<N; ++h)
        if (W(l, h)>=V)
          throw "Word not in the dictionary";
}


void ABSamplerType::SetEta(const double inEta)
{
  if ( (!useVisualWords) && (inEta>0) )
    flog<<"Warning: we are not using the visual words, "
        <<"but the topic prior is positive"<<endl;
  
  eta=inEta; // set the hyperparameter for the word distributions
  sumEta=eta*V; // set the sum of all the hyperparameter
}


void ABSamplerType::UpdateParametricTerm()
{
  vector<bool> mask(N);
  
  for(unsigned k=0; k<K; ++k){
    for(unsigned n=0; n<N; ++n)
      mask[n]=(C[n]==k);
   
    paramDist[k].UpdateParam(X, mask);
  }
}


void ABSamplerType::SetParametricTerm()
{
  // we should not use this term if there are no observations
  if (!observationsSet)
    throw "Using the parametric term without observations";
  
  // allocate K parametric distributions
  paramDist.resize(K);
  
  // set the Normal-InvWishart hyperparameter in each parametric term
  for(unsigned k=0; k<K; ++k)
    paramDist[k].SetInvWishart(mu_prior, precision_prior, nu_prior, S_prior);
  
  // set the statitics
  vector<bool> mask(N);
  for(unsigned k=0; k<K; ++k){
    for(unsigned n=0; n<N; ++n) 
      mask[n]=(C[n]==k);
    
    // init the k-th cluster
    paramDist[k].Init(X, mask);
  }
}


void ABSamplerType::SetParametricTerm(const unsigned k, 
                                      const ublas_vector &newSX, 
                                      const ublas_matrix &newSX2, 
                                      unsigned newNd)
{
  // we should not use this term if there are no observations
  if (!observationsSet)
    throw "Using the parametric term without observations";

  //
  // Set the parametric term of cluster 'k' using the sufficient statistics
  // 'newSX', 'newSX2', and 'newNd'. This is used when working on image
  // collections.
  //
  
  paramDist[k].SumX()=newSX;
  paramDist[k].SumX2()=newSX2;
  paramDist[k].NumObs()=newNd;
  
  // update the internal structure of the parametric term
  paramDist[k].UpdateParam();
}


void ABSamplerType::SetInitialState(const double *p, unsigned iN)
{
  C.resize(N); // allocate memory for the labels;

  if (iN==0){
    // empty vector. Use random initialization for the state
    // uniform discrete random variable.
    uiRndVar uid(0, K-1);
    
    // binding between the random number generator and the random variable
    boost::variate_generator<rndGenType, uiRndVar> uidGen(*rg_engine_pt, uid);
    
    // generate the sequence
    generate(C.begin(), C.end(), uidGen);
  }
  else{
    // Set the labels to the specified input values
    
    for(unsigned i=0; i<C.size(); ++i){
      // we have to subtruct one because of the different convention
      // for the indeces in C++ and MATLAB
      C[i]=static_cast<unsigned>(p[i])-1; 
      
      // a sanity check
      if ( (C[i]<0) || (C[i]>=K) ) throw "Invalid label in vector C";
    }
  }
      
  ComputeCounts(); // we have to compute the counts of the input labeling
  
  emptyState=false; // the state is ready
}


void ABSamplerType::SetLambda(const double *v)
{
  lambda.resize(K, 0);
  useParametricTerm=false;
  
  if (v==0) return;
  
  // copy the input vector into lambda
  copy(v, v+K, lambda.begin());
  
  // check that the input lambdas are between 0 and 1.
  if (count_if(lambda.begin(), lambda.end(), 
               bind2nd(less<double>(), 0.0)) > 0)
    throw "Negative value for lambda weights";
    
  if (count_if(lambda.begin(), lambda.end(), 
               bind2nd(greater<double>(), 1.0)) > 0)
    throw "Value greater then 1 for lambda weights";
  
  // check if all the specified weights are zero. In this case we do not
  // use the parametric term
  useParametricTerm=(count_if(lambda.begin(), lambda.end(), 
                              bind2nd(greater<double>(), 0.0)) > 0);
  
  // we are going to use the parametric term. Check that the observation
  // vectors are available.
  if ( (useParametricTerm) && (!observationsSet) )
    throw "No observations specified for the parametric term";

  if (useParametricTerm) {
    SetParametricTerm(); // we must set the parametric term, 
                         // since we have to use it.
  }
}


void ABSamplerType::SetPriorInformation(const double *p, unsigned L)
{
  if (L==0) return; // no side information

  priorLabels.resize(N, K); // all the labels are free
  for(unsigned i=0; i<N; ++i){
    // we have to subtruct one because of the diffenrent convention
    // for the indeces in C++ and MATLAB
    if (p[i]>0){ // preassigned label
      priorLabels[i]=static_cast<unsigned>(p[i])-1;
      C[i]=priorLabels[i];
      
      // a sanity check
      if (priorLabels[i]>=K) throw "Invalid labels in PI";
    }
  }
  
  // we have to recompute the counts again since we do not assume coherence
  // between the input state and the labeling information provided in PI
  ComputeCounts();

  usePI=true; // set the flag
}


inline
void ABSamplerType::ComputeCondDist(unsigned i)
{
  //
  // First compute the likelihoods for observation i to be in each cluster
  //
  
#ifndef _PATCHES_ONLY_
  // init the likelihood vector to zero
  fill(likelihood.begin(), likelihood.end(), 0.0);
  
  double *a=A[i]; // beginning of the i-th column
  if (sparseAffinity){
    mwIndex *row=row_info[i]; // row indeces of each non-zero element in i
    unsigned Ne=col_dim[i];

    for(unsigned n=0; n<Ne; ++n, ++a, ++row)
      likelihood[C[(*row)]]+=(*a);
  }
  else{
    for(unsigned n=0; n<N; ++n, ++a)
      likelihood[C[n]]+=(*a);
  }
  
  // we remove the diagonal value because the current observation is not 
  // used to compute the KDE.
  likelihood[C[i]]-=Ad[i];
  
  for(unsigned k=0; k<K; ++k) // we need to normalize
    if (allCounts[k]==0)
      likelihood[k]=0; // if no other element is assigned to cluster k
                       // we assume the likelihood to be zero. This may be
                       // problem for the sampler, because once a cluster
                       // does not have observations it disappear forever.
    else
      likelihood[k]/=allCounts[k];
  
  // Here we "compose" the likelihood of the "global structure", represented by
  // the parametric term, with the "local structure",  represented by the 
  // non-parametric term.
  //flog<<"Comparisons between different terms of the likelihood:"<<endl;
  if (useParametricTerm) {
    for(unsigned k=0; k<K; ++k){
      likelihood[k]=(1-lambda[k])*likelihood[k]+
                     lambda[k]*paramDist[k].Evaluate(X[i]); // parametric term
                                                            // likelihood
    }
  }
  //flog<<endl;
  
#else

  fill(likelihood.begin(), likelihood.end(), 1.0);
  
  // A sanity check! If we are using only the patches, then the sampler should
  // be set to use them. If that's not the case we throw an error.
  if (!useVisualWords)
    throw "Not using the visual words in LDA sampler";
  
#endif // end of the switch between the two different version of the sampler.

  // Here we multiply the likelihood for the features with the likelihod for 
  // the visual word associated with the patch.
  //flog<<"----------------------------------------------------------------"<<endl;
  if (useVisualWords) {
    // just a little check
    if (!topicTablesReady)
      throw "Trying to use visual words, but the topic tables are not ready";

    for(unsigned l=0; l<L; ++l) {
      const table<unsigned> &topicTable=*(wordTopicPr[l]);
      const vector<unsigned> &topicVector=*(topicCountPr[l]);

#ifdef _USE_BACKGROUND_
      // the topic 0 is the background so we have to use the local vector
      likelihood[0]*=(backgroundHist[l][W(l,i)]+eta)/(backgroundSize[l]+sumEta);
      for(unsigned k=1; k<K; ++k)
        likelihood[k]*=(topicTable(W(l,i), k)+eta)/(topicVector[k]+sumEta);
#else
      for(unsigned k=0; k<K; ++k)
        likelihood[k]*=(topicTable(W(l,i), k)+eta)/(topicVector[k]+sumEta);
#endif
    }
  }
  
  //
  // Then we compute the prior
  //
  
  for(unsigned k=0; k<K; ++k)
    prior[k]=allCounts[k]+alpha[k];
   
  
  // 
  // Finally we compute the cumulative posterior (in normalized form)
  //
  
  // compute the first elements (k=0)
  cumPost[0]=likelihood[0]*prior[0];
  
  for(unsigned k=1; k<K; ++k){
    cumPost[k]=cumPost[k-1]+(likelihood[k]*prior[k]);
  }
  
  // normalize the cumulative posterior
  unsigned last=K-1;
  for(unsigned k=0; k<last; ++k)
    cumPost[k]/=cumPost[last];
  cumPost[last]=1;
}


void ABSamplerType::SetTopicTables(
                    vector<table<unsigned> *> inWordTopicTablePr, 
                    vector<vector<unsigned> *> inWordTopicCountPr)
{
  wordTopicPr=inWordTopicTablePr;
  topicCountPr=inWordTopicCountPr;
}


void ABSamplerType::FillTopicTables()
{
  // we assume that the table is already created and set either to zero
  // or to the count given by the other images
  
  // allocate the vector for the hist
  backgroundSize.resize(L, 0);
  backgroundHist.resize(L, vector<unsigned>(V, 0));
  
  for(unsigned l=0; l<L; ++l) {
  
    // iterate over all the observation and increment the table entries
    for(unsigned n=0; n<N; ++n){
      ++(*(wordTopicPr[l]))(W(l, n), C[n]);
      ++(*topicCountPr[l])[C[n]];

      // we have to update the local histogram
      if (C[n]==0) {
        ++backgroundHist[l][W(l,n)];
        ++backgroundSize[l];
      }
    }
  }
  
  // the topics are ready for this sampler
  topicTablesReady=true;
}


void ABSamplerType::Run()
{
  if (usePI) {
    // if we are using fixed labels we have to check
    // which points have fixed labels
    for(unsigned i=0; i<N; ++i){
      // iterate over all observations      
      if (priorLabels[i]==K){
        // if the label is K (the number of clusters) than it is a valid
        // point to resample (the labels are from 0 to K-1)
        RemoveObservation(i);
        ComputeCondDist(i);
        C[i]=MultGenRnd(cumPost, *u01);
        AddObservation(i);
      }
    }
  }
  else {
    for(unsigned i=0; i<N; ++i){
      RemoveObservation(i);
      ComputeCondDist(i);
      C[i]=MultGenRnd(cumPost, *u01);
      AddObservation(i);
    }
  }
  
  // at the end of the sampling we update the parameters of the parametric term.
  if (useParametricTerm)
    UpdateParametricTerm();
}


void ABSamplerType::RunBlockSampling()
{
  // given the current state and the affinity matrix we have a set of graphs
}
  

double ABSamplerType::GenerateBlock()
{
  // TODO
  return 0.0;
}


double ABSamplerType::ComputeLogLikelihood() const
{
  // compute the log-likelihood of the current pair (x, z), with z being
  // the cluster assignment.
  
  
#ifndef _PATCHES_ONLY_
  //
  // First term: log p(x|z) = sum_n ( log p(x_i | z_i) ) 
  //                        = sum_n ( log ( sum_{j \in C_k} A_{n, l}/N_k ) )
  //
  
  // iterate over all the observations
  double ll_term1=0.0; // this is log p(x|z)
  for(unsigned i=0; i<N; ++i) {
    double l=0.0; // likelihood for the i-th observation 
    unsigned k=C[i]; // current cluster label: z_i=k
    
    // compute p(x_i| z_i=k ) = sum_{l \in C_k} (A_ij)
    double *a=A[i]; // beginning of the i-th column
    if (sparseAffinity){
      mwIndex *row=row_info[i]; // row indeces of each non-zero element in i
      unsigned Ne=col_dim[i];

      // iterate over all the affinities and collect the ones in cluster k
      for(unsigned n=0; n<Ne; ++n, ++a, ++row)
        if (C[(*row)]==k) l+=(*a);
    }
    else{
      for(unsigned n=0; n<N; ++n, ++a)
        if (C[n]==k) l+=(*a);
    }
  
    if (allCounts[k]==0)
      l=1E-300; // if no other element is assigned to cluster k we assume the 
                // likelihood to be (almost) zero. We do not use zero to avoid
                // possible problems when we compute the logarithm.
    else
      l/=allCounts[k]; // divide by the number of elements currently in the cluster
  
    // Here we "compose" the likelihood of the "parametric term", 
    if (useParametricTerm) {
      l=(1-lambda[k])*l+lambda[k]*paramDist[k].Evaluate(X[i]); 
        // parametric term likelihood
    }
    
    // add the log-likelihood for the observation i to the whole log likelihood
    ll_term1+=log(l);
  }

#else
  double ll_term1=0;

#endif

  //
  // Now we have the first term of the likelihood (which depends on the 
  // observations vectors). We have to compute the second term, which depend 
  // on the prior parameters. p(z)
  //
 
  double ll_term2=boost::math::lgamma(sum_alpha)
                 -boost::math::lgamma(N+sum_alpha);
  for(unsigned k=0; k<K; ++k)
    ll_term2+=(boost::math::lgamma(alpha[k]+allCounts[k])
              -boost::math::lgamma(alpha[k]));
  
  // return the likelihood
  return (ll_term1+ll_term2);
}

//
// end of the methods for the ABSamplerType class
//



//
// This function check that the arguments passed to the sampler are correct,
// and coherent.
//

void CheckArguments(int n_out, mxArray *out[], int n_in, mxArray *in[])
{
  // all the imputs should be of class double
  for(int h=0; h<n_in; ++h)
    if (mxGetClassID(in[h])!=mxDOUBLE_CLASS)
      mexErrMsgTxt("Input argument not of class double");

  // Check the number of inputs
  if ((n_in<MIN_IN_ARG) || (n_in>MAX_IN_ARG)) 
    mexErrMsgTxt("Wrong number of inputs.");
  
  // We have certain integrity constraints:
  // size(A)=[N, N]
  if ( mxGetM(in[iAFF])!=mxGetN(in[iAFF]) )
    mexErrMsgTxt("The affinity matrix A is not square");
  
  // rows(X)=N
  if ( (mxGetM(in[iX])!=mxGetN(in[iAFF])) && 
       (mxGetM(in[iX])*mxGetM(in[iX])>0) )
    mexErrMsgTxt(
      "The number of observations does not match affinity matrix dimensions");
  
  // alpha must be a vector 
  if ( ((mxGetM(in[iALPHA])>1) && (mxGetN(in[iALPHA])>1) ) || 
       ((mxGetM(in[iALPHA])==1) && (mxGetN(in[iALPHA])==1) ) )
    mexErrMsgTxt("The hyperparameter alpha must be a vector");
  
  // NI must be either a scalar or a vector
  if ((mxGetM(in[iNI])>1)  && (mxGetN(in[iNI])>1))
    mexErrMsgTxt("NI must be a vector or a scalar");

  // If C is given then it is either a column vector of N elements or []
  if (n_in>iC)
    // we have also the intial labeling C (column vector)
    if ( (mxGetM(in[iC])!=0) || (mxGetN(in[iC])!=0) ) 
      if ( (mxGetM(in[iC])!=mxGetM(in[iAFF])) || (mxGetN(in[iC])!=1) )
        mexErrMsgTxt(
        "The (column) vector C must have N elements or be the empty matrix");
  
  // if si is given the is must be a column vector of N elements or []
  if (n_in>iSI)
  // we have also the prior labeling si (column vector)
    if ( (mxGetM(in[iSI])!=0) || (mxGetN(in[iSI])!=0) )
      if ( (mxGetM(in[iSI])!=mxGetM(in[iAFF])) || (mxGetN(in[iSI])!=1) )
        mexErrMsgTxt(
        "The (column) vector si must have N elements or be the empty matrix");
  
  // if lambda is give then it must have the same length of alpha
  if (n_in>iLAMBDA)
    if ( (mxGetM(in[iLAMBDA])*mxGetN(in[iLAMBDA])) != 
       (mxGetM(in[iALPHA]) *mxGetN(in[iALPHA])) )
      mexErrMsgTxt(
      "The vector lambda must have the same number of elements of alpha");
  
  if (n_in>iW) {
    // if the visual words are provided we must have also the dictionary size 
    // and the Dirichlet hyperparmeter 'eta' 
    if (n_in<=iETA)
      mexErrMsgTxt("Not enough inputs for the visual words");
  
    // check input iW
    if ( ( (mxGetM(in[iW])) != mxGetM(in[iAFF]) ) && 
         ( (mxGetM(in[iW])*mxGetN(in[iW])) != 0 ) )
      mexErrMsgTxt("The label vector must have N rows or be empty");
  
    // iV must be a scalar 
    if ( (mxGetM(in[iV])*mxGetN(in[iV])) != 1 )
      mexErrMsgTxt("V must be a scalar");

    // iETA must be a scalar 
    if ( (mxGetM(in[iETA])*mxGetN(in[iETA])) != 1 )
      mexErrMsgTxt("Eta must be a scalar");
  }
}


void SetUpSampler(int n_in, mxArray *in[], ABSamplerType &ABsampler)
{  
  // we have to setup the random number generator as first 
  // thing otherwise we won't be able to generate the initial 
  // labeling  
  unsigned rndSeed=time(0)+clock();
  //unsigned rndSeed=100;
  if (n_in>iSEED) {
    rndSeed=static_cast<unsigned>(mxGetScalar(in[iSEED]));
    flog<<"Using "<<rndSeed<<" as seed for the random number generator "<<endl;
  }
  
  ABsampler.SetSeed(rndSeed); // set the seed

  // set the dirichlet hyperparameters
  unsigned K=mxGetM(in[iALPHA])*mxGetN(in[iALPHA]);
  ABsampler.SetPriorHyperParameters(mxGetPr(in[iALPHA]), K);
  
  // set the data used by the sampler
  ABsampler.SetAffinityMatrix(in[iAFF]);
  ABsampler.SetObservationVectors(in[iX]);
  
  // 
  // This is a work around for the hyperparameters of the prior of the 
  // parametric term. We should have them specified as input arguments.
  //
  unsigned F=ABsampler.GetF();
  ublas_vector m0=ublas_zero_vector(F);
  ABsampler.SetInvWishart(m0, 0.001, 9, ublas_identity_matrix(F)*0.01);
  // the parametric term distribution are now ready

  // here we set the initial state
  if (n_in>iC)
    ABsampler.SetInitialState(mxGetPr(in[iC]), mxGetM(in[iC]));
  else
    ABsampler.SetInitialState(0, 0);
  
  if (n_in>iSI) 
    ABsampler.SetPriorInformation(mxGetPr(in[iSI]), mxGetM(in[iSI]));
  
  if (n_in>iLAMBDA)
    ABsampler.SetLambda(mxGetPr(in[iLAMBDA]));
  else
    ABsampler.SetLambda(0);

  if (n_in>iW) {
    // we have also information for visual words.
    ABsampler.SetVisualWords(in[iW]);
    ABsampler.SetDictionarySize(mxGetScalar(in[iV]));
    ABsampler.SetEta(mxGetScalar(in[iETA]));
  }
  else {
    ABsampler.SetVisualWords(0); // this will avoid using the visual words
    ABsampler.SetDictionarySize(0);
    ABsampler.SetEta(0);
  }
  
/*  // a simple check for the use of the visual words in the sampler
  if (ABsampler.UseVisualWords())
    flog<<"Using the visual words"<<endl;
  else
    flog<<"Not using the visual words"<<endl;
      
#ifdef _PATCHES_ONLY_
   flog<<"Using only the visual words (LDA model)"<<endl;
#endif
  flog<<endl;*/
}
