/* AUTORIGHTS
Copyright (C) 2008-2012 Marco Andreetto
      
This file is part of AB_SAMPLER.

AB_SAMPLER is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#ifndef _AB_SAMPLER_H_
#define _AB_SAMPLER_H_

// This header file defines the class for a Gibbs sampler over a single 
// image. This can also be used to sample an entire collection. It is 
// sufficient to create a sampler object for each image in the collection

#include <vector>

#include <boost/random.hpp>
#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/vector.hpp>

// define some additional functions for the boost::ublas library
#include "ublas_utils.h"

// MATLAB interface
#include "mex.h"

// define a gaussian distribution
#include "my_distributions.h"

// define a table datastructure for the word/topic counting
#include "table.h"


// aliases for boost::random classes
typedef boost::mt19937 rndGenType;
typedef boost::uniform_01<rndGenType> u01RndVar;
typedef boost::uniform_int<unsigned> uiRndVar;

// aliases for the ABSampler
typedef std::vector<unsigned> SampleType;
typedef GaussianDist DistributionType;


//
// definition of the class for the Gibbs sampler used in 
// the affinity based grouping
//
class ABSamplerType
{
  // parameters of the prior
  std::vector<double> alpha; // hyperparameter for the Dirichlet prior
  double sum_alpha;          // sum of the alphas
  
  // hyperparameters for the Inv-Wishart prior of the parametric term
  ublas_vector mu_prior;    // mean of the gaussian prior 
  double precision_prior;   // precision for the gaussian prior 
  unsigned nu_prior;        // Degree of freedom of the Inv-Wishart
  ublas_matrix S_prior;     // ublas_matrix of the Inv-Wishart
  
  // hyperparameters for the word-topic distribution
  double eta;       // hyperparameter for the symmetric Dirichlet distribution
  double sumEta;    // product eta*V;
  
  // Data for the sampler: affinities and feature vectors
  std::vector<double*> A; // The affinity matrix is stored as a vector 
                          // of pointers to doubles. Each pointer refers
                          // to a different column. The pointers refer
                          // to the MATLAB data.
  
  std::vector<double> Ad; // The values of the diagonal of the Affinity matrix.
                          // These values are used for computing the likelihood 
                          // of the current state of the sampler.
  
  std::vector<ublas_vector> X;
  // the set of features used to compute the parametric term in each cluster. 
  // At the moment each parametric term is assumed to be a single Gaussian.
  
  table<unsigned> W; // The visual words associated to each observation.
                     // It is a table since we have many words for each
                     // observations.
  
  // auxiliary data structure to handle sparse representation of the affinities
  std::vector<unsigned> col_dim; // number of nonzero elements in each row
  std::vector<mwIndex *> row_info; // indeces of the rows
  
  // Dimensions of the model
  unsigned N;  // number of observations
  unsigned K;  // number of clusters
  unsigned F;  // dimension of the feature space
  unsigned V;  // number of words in the all the dictionary
  unsigned L;  // number of visual word for each observations
  
  // sampler state
  SampleType C; // current labeling (state of the sampler)
  std::vector<unsigned> allCounts; // number of observations in each cluster
  
  // there are two additional tables: the table of the topic vector 
  // assignment and the vector of the topic assignment. These are shared 
  // between all the images in the collection. Therefore we store pointers
  // to this global tables.
  std::vector<table<unsigned> *> wordTopicPr; // a vector of pointers to all the 
                                             // tables for each type of visual
                                             // words.
                                             
  std::vector<std::vector<unsigned> *> topicCountPr; // vectors to the pointers
                                                     // for each visual word.
  // histogram background for this image
  std::vector<std::vector<unsigned> > backgroundHist;
  std::vector<unsigned> backgroundSize;

  // parametric part of the sampler (each component is a Gaussian dist.)
  std::vector<DistributionType> paramDist;
                                      
  std::vector<double> lambda;   // weights of the parametric term (1 only 
                                // parametric term 0 only non-parametric term)
    
  // distributions used by the sampler
  std::vector<double> cumPost;     // cummulative posterior of the i-th label
  std::vector<double> likelihood;  // likelihood of the i-th observation
  std::vector<double> prior;       // prior on the i-th label
  
  // preassigned observations. These observations are never sampled.
  std::vector<unsigned> priorLabels;

  // random generator and uniform random variable.
  rndGenType *rg_engine_pt;
  u01RndVar *u01;
  unsigned used_seed; // the seed used to intialize the ranom number generator.
                      // It is important to save it to obtain reproducible
                      // results.

  // flags for consistency of the data structures and for their proper use
  bool emptyState, affinitiesReady, sparseAffinity, useParametricTerm, 
       usePI, useVisualWords, topicTablesReady, observationsSet;
  
  public: 
  
  // default constructors.
  ABSamplerType() : N(0), K(0), F(5), V(0), L(0),
                    wordTopicPr(0), topicCountPr(0),
                    rg_engine_pt(0), u01(0),
                    
                    emptyState(true), affinitiesReady(false), 
                    sparseAffinity(false), useParametricTerm(false), 
                    usePI(false), useVisualWords(false), topicTablesReady(false),
                    observationsSet(false)
  {}
  
  
  // set up all the data of the sampler
  void SetPriorHyperParameters(const double *, unsigned);
  void SetAffinityMatrix(const mxArray *);
  void SetDiagonalAffinities();
  void SetObservationVectors(const mxArray *);
  void SetInitialState(const double *, unsigned);
  void SetPriorInformation(const double *, unsigned); 
  void SetLambda(const double *);
  
  void SetVisualWords(const mxArray *);
  void SetDictionarySize(const double);
  void SetEta(const double);
  void SetTopicTables(std::vector<table<unsigned> *>, 
                      std::vector<std::vector<unsigned> *>);
  void FillTopicTables();

  // Set the Gaussian/Inv-Wishart prior
  void SetInvWishart(const ublas_vector &, double, unsigned, 
                     const ublas_matrix &);
  
  // store the seed for all the following random operations
  void SetSeed(unsigned);
  unsigned GetSeed() const { return used_seed; }

  // Generate sample from the current state. This is done by running 
  // the Gibbs sampler over all the observations once.
  void Run();
  
  // Generate a new sample from the current state using block sampling.
  void RunBlockSampling();
  
  // Compute the likelihood of the current label assignment
  double ComputeLogLikelihood() const;
    
  // Read the current state of the sampler
  void GetSample(double *p) const
  { for(unsigned i=0; i<C.size(); ++i) p[i]=C[i]+1; }

  void GetSample(std::vector<unsigned> &out) const
  { out.resize(N); for(unsigned i=0; i<C.size(); ++i) out[i]=C[i]+1; }

  // begin and end const iterators for accessing to the sampler state C
  SampleType::const_iterator beginState() const { return C.begin(); }
  SampleType::const_iterator endState() const { return C.end(); }

  // access other information inside the sampler
  unsigned GetN()  const { return N; }
  unsigned GetK()  const { return K; }
  unsigned GetF()  const { return F; }
  unsigned GetV()  const { return V; }
  unsigned GetL()  const { return L; }
  double   GetEta() const { return eta; }
  
  // access to the statistics of parametric term of the sampler.
  const ublas_vector & GetParametricTerm_SX(unsigned k) const 
  { return paramDist[k].SumX(); }

  const ublas_matrix GetParametricTerm_SX2(unsigned k) const 
  { return paramDist[k].SumX2(); }

  unsigned GetParametricTerm_Nd(unsigned k) 
  { return paramDist[k].NumObs(); }
  
  void SetParametricTerm(const unsigned, const ublas_vector &, 
                         const ublas_matrix &, unsigned Nd);  
  
  // check if we are using the patches or not
  bool UseVisualWords() const { return useVisualWords; }
  
  // destructor
  ~ABSamplerType() 
  { delete rg_engine_pt; delete u01; }
  
  private:
  
  void ComputeCounts();
  void AddObservation(unsigned i);
  void RemoveObservation(unsigned i);
  void ComputeCondDist(const unsigned);
  void SetParametricTerm();
  void UpdateParametricTerm();
  
  
  // set of auxiliary functions for "block" sampling.
  double GenerateBlock();

  
  //
  // never to be used
  //
  ABSamplerType(const ABSamplerType &); // no copy constructor defined
  const ABSamplerType & operator=(const ABSamplerType &); 
};

// define a type pointer to an ABSamplerType class
typedef ABSamplerType* ABSamplerTypePr;


// function for setting up a sampler using the parameters from MATLAB
void SetUpSampler(int, mxArray **, ABSamplerType &);

// function to check the coherence of the MATLAB arguments
void CheckArguments(int, mxArray **out, int , mxArray **);


// A simple function to sample from a multinomial distribution.
// The input parameters are the cumulative distribution (for efficiency
// reason), and the random generator object.
inline
unsigned MultGenRnd(const std::vector<double> &cumDist, u01RndVar &u01)
{
  // generate a random number
  double v=u01();
  
  for(unsigned r=0; r<cumDist.size(); ++r)
    if ( v<=cumDist[r] )
      return r;
  
  throw "Detected an invalid posterior distribution";

  return cumDist.size();
}


// MACROS for the MATLAB parameters. They define the interface for the sampler.


#define iAFF     0
#define iX       1
#define iALPHA   2
#define iNI      3
#define iC       4
#define iSI      5
#define iLAMBDA  6 
#define iW       7
#define iV       8
#define iETA     9
#define iSEED   10

#define oSAMPLES 0
#define oLL      1
#define oSEED    2
#define oTABLE   3

#define MIN_IN_ARG (iNI+1)
#define MAX_IN_ARG (iSEED+1)

#define MIN_OUT_ARG (oSAMPLES+1)
#define MAX_OUT_ARG (oTABLE+1)


#endif // _AB_SAMPLER_H_
