/* AUTORIGHTS
Copyright (C) 2008-2012 Marco Andreetto
      
This file is part of AB_SAMPLER.

AB_SAMPLER is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#include <iostream>
#include <fstream>
#include <vector>
#include <cmath>

#include <boost/math/special_functions/gamma.hpp>
#include <boost/format.hpp>

#include "mex.h"

#include "my_distributions.h"
#include "ab_log.h"
#include "ab_sampler.h"
#include "ab_sampler_list.h"

using namespace std;

void CheckListArguments(int n_out, mxArray *out[], int n_in, mxArray *in[])
{
  //
  // All the arguments of the list sampler are cell array, but the vector that 
  // represents the number of iterations. This argument indicates the number  
  // of iterations for all the sampler and is therefore global to all of them.
  // All the other arguments are in one to one correspondence with the 
  // arguments of the single sampler.
  // Each cell in each argument contains the corresponding single argument
  // for the single sampler.
  //
  
  // Check the number of inputs
  if ((n_in<MIN_IN_ARG) || (n_in>MAX_IN_ARG)) 
    mexErrMsgTxt("Wrong number of inputs.");

  // iterate over all the available inputs and check if they are cell arrays
  for(int h=0; h<n_in; ++h) {
    if (h==iNI) continue; // skip the number of iterations
    if (!mxIsCell(in[h]))
      mexErrMsgTxt("The input of the list sampler must be a cell array");
  }
  
  // get the number of images (and samplers) from the first argument
  unsigned T=mxGetN(in[0]);

  // iterate over all the cell arrays and check they all heve the same 
  // dimensions: 1xN with N number of images.
  for(int h=0; h<n_in; ++h){
    if (h==iNI) continue; // skip the number of iteration
  
    // check all the others
    if (mxGetN(in[h])!=T)
      mexErrMsgTxt("Incoherent dimension of the cell array");
      
    // check that the cell array is a row
    if (mxGetM(in[h])!=1)
      mexErrMsgTxt("The cell array must be a row (1xNI)");
  }

  // iterate over all the NI images, construct a temporary array for the 
  // input and check it using the checking function for the single sampler
  for(unsigned ni=0; ni<T; ++ni){
    mxArray **tmp_in; // tmp mxArray vector
    tmp_in=new mxArray *[n_in]; 

    // get all the cell in position 'ni', for all the arguments
    for(int h=0; h<n_in; ++h){
      if (h==iNI)
        tmp_in[iNI]=in[iNI];
      else
        tmp_in[h]=mxGetCell(in[h], ni);
    }
    
    // pass the tmp_in vector to the checking function 
    CheckArguments(n_out, out, n_in, tmp_in);
    
    // release the memory allocated for the array
    delete []tmp_in;
  }
  
  // if we are here the arguments are all coherent and we are done.
}


void SetUpAllSamplers(int n_in, mxArray *in[], 
                      vector<ABSamplerTypePr> &samplerImages)
{
  //
  // create multiple sampler and set them using the different values of 
  // the affinities and observation cell arrays.
  //

  unsigned T=mxGetN(in[iAFF]); // number of images in the collection
  samplerImages.resize(T);     // preallocation

  // tmp list for the arguments of each sampler
  mxArray **single_sampler_param;
  single_sampler_param=new mxArray *[n_in];
  
  for(unsigned t=0; t<T; ++t){
    // create an object sampler and store it into the container
    samplerImages[t]=new ABSamplerType;
        
    // construct the list of arguments for the t-th sampler
    for(int h=0; h<n_in; ++h){
      if (h==iNI)
        single_sampler_param[h]=in[h];
      else
        single_sampler_param[h]=mxGetCell(in[h], t);
    }
  
    // set up the t-th sampler
//     flog<<"Setting up sampler "<<(t+1)<<endl;
    SetUpSampler(n_in, single_sampler_param, (*samplerImages[t]));
  }
  
  delete []single_sampler_param;
}


void SetTopicTables(vector<ABSamplerTypePr> &samplerImages, 
               vector<table<unsigned> *> &allWordTopicTables, 
               vector<vector<unsigned> *> &allTopicTables)
{
  for(unsigned l=0; l<samplerImages.size(); ++l){
    (samplerImages[l])->SetTopicTables(allWordTopicTables, allTopicTables);
    (samplerImages[l])->FillTopicTables();
  }

  /*// At this point the word/topic table is ready. We check if all the words
  // in the dictionary have been used. If not we issue a warning

  unsigned numUsedWords=0, maxLabel=0;

  unsigned V=(samplerImages[0])->GetV();
  unsigned K=(samplerImages[0])->GetK();
  
  for(unsigned h=0; h<V; ++h) {
    unsigned c=0;

    // sum over all the topics
    for(unsigned k=0; k<K; ++k)
      c+=wordTopicTable(h, k);

    if (c!=0) {
      ++numUsedWords;
      maxLabel=max(h, maxLabel);
    }
  }

  if (numUsedWords<V) {
    flog<<"Warning: "<<(V-numUsedWords)<<" of the "<<V
        <<" visual words are not used"<<endl;
    flog<<"Warning: max visual word label: "<<maxLabel<<endl;
  }*/
}


void GetPriorHyperparam(int nin, mxArray **in, ublas_vector &m0, 
                        double &tau0, unsigned &nu0, ublas_matrix &S0)
{
  //
  // Read the hyperparameters from the input. If no hyperparameters are
  // specifiend then the default values are used.
  //   
  
  vector<double> hyperParam(5, 0);  
  /*if (nin>iNIW_HP)
    // use the hyperparameters specified as input
    copy(mxGetPr(in[iNIW_HP]), mxGetPr(in[iNIW_HP])+5, hyperParam.begin());
  else*/ {
    // use the default values. They should give vague 
    //(non-informative) prior
  
    hyperParam[0]=0; // m0=[0 0] the center of the image
    hyperParam[1]=0; 
    hyperParam[2]=0.01; // tau0=0.01 the precision of the mean dist.
    hyperParam[3]=6;    // nu0=6 number of degree of freedom (D+4, with D=2)
    hyperParam[4]=0.01; // S0=0.01*eye(D) scale matrix
  }
  
  // check if the observations are specified. It they are not fix the 
  // dimensions to 5
  unsigned D;
  if (mxGetM(mxGetCell(in[iX], 0))*mxGetN(mxGetCell(in[iX], 0))==0)
    D=5;
  else
    D=mxGetN(mxGetCell(in[iX], 0));
  
  
  //
  // Set the hyperparameters
  // 
  
  m0=ublas_zero_vector(D); // all the coordinates are zero
  m0(D-2)=hyperParam[0]; // change only the positions part
  m0(D-1)=hyperParam[1]; 
  tau0=hyperParam[2]; // scale of the prior gaussian
  nu0=static_cast<unsigned>(hyperParam[3]); //  number of degree of freedom
  S0=ublas_identity_matrix(D)*hyperParam[4];     // set the scale matrix
}


void reserveOutput(mxArray *in[], mxArray *&outSamples)
{
  //
  // Reserve the memory for the output samples: both the number of images
  // and the number of pixels and samples
  //
    
  // this is the number of samples (common to all the images)
  unsigned S=mxGetN(in[iNI])*mxGetM(in[iNI]);

  // this is the number of images
  unsigned T=mxGetN(in[iAFF]);

  // allocate the cell array for the samples
  outSamples=mxCreateCellMatrix(1, T);
  
  // allocate the mxarrays for each image
  for(unsigned h=0; h<T; ++h){
    // get the number of pixels in image h
    unsigned N=mxGetN(mxGetCell(in[iAFF], h));
    
    //allocate the mxArray and put the pointer into the cell array
    mxSetCell(outSamples, h, mxCreateDoubleMatrix(N, S, mxREAL));
  }
}


void CollectSamplesForAllImages(const vector<ABSamplerTypePr> &samplerImages, 
                                unsigned s, mxArray *outSamples)
{
  //
  // Collect the current label assignment (a sample) for all the images.
  // 
  
  mxArray *pi;
  unsigned N;
  double *d;
  for(unsigned h=0; h<samplerImages.size(); ++h){
    
    // pointer to the samples of the h-th image
    pi=mxGetCell(outSamples, h);
    
    // number of pixels in the current image
    N=(samplerImages[h])->GetN();
    
    // copy the sample into the mxArray
    d=mxGetPr(pi)+N*s; // p is the pointer to the starting position to fill
    (samplerImages[h])->GetSample(d);
  }
}


double ComputeLogLikelihoodForAllImages(const vector<ABSamplerTypePr> &samplerImages, 
                                        const vector<unsigned> &topicTable,
                                        const table<unsigned> &wordTopicTable)
{
  //
  // Compute the log-likelihood of the current state of the sampler.
  // The log-likelihood is computed by summing the log-likelihood of each image 
  // and finally adding the term for the topic/words distributions.
  //
  
  
  //
  // Iterate over all the images and collect the log-likelihood for those images.
  // This will result in computing the term log p(x|z) + log(z)
  //
  double logLikelihood=0, ll_term1=0, ll_term2=0;
  for(unsigned m=0; m<samplerImages.size(); ++m){
    double ll=(samplerImages[m])->ComputeLogLikelihood();
    
    // check that the likelihood is neither infinite nor Nan
    if (isnan(ll))
      throw "Image log-likelihood in NaN";
    
    if (isinf(ll))
      throw "Image log-likelihood in Inf";
    
    ll_term1+=ll;
  }

  
  //
  // Add the term for the topic distributions log(w|z)
  //

  unsigned K=topicTable.size(); // number of topics
  unsigned V=wordTopicTable.NumberOfRows(); // number of visual words
  double eta=(samplerImages[0])->GetEta(); // we use the first image since all the eta should be tha same.
  
  // first add the term indipendent from the word assignment
  ll_term2=K*(boost::math::lgamma(V*eta)-V*boost::math::lgamma(eta));

  // then we consider all the word/topic assignment
  for(unsigned k=0; k<K; ++k) {
    for(unsigned w=0; w<V; ++w)
      ll_term2+=boost::math::lgamma(wordTopicTable(w, k)+eta);
    ll_term2-=boost::math::lgamma(topicTable[k]+V*eta);
  }
  
  // check that the likelihood is neither infinite nor Nan
  if (isnan(ll_term2)) throw "Visual words log-likelihood in NaN";
  if (isinf(ll_term2)) throw "Visual words log-likelihood in Inf";
  
  logLikelihood=ll_term1+ll_term2;
  
  return logLikelihood;
}


void CollectSeedsFromSamplers(const vector<ABSamplerTypePr> &samplerImages,
                              mxArray *&outSeeds)
{
  //
  // Collect the seed of the random generator of each sampler (one image, one 
  // seed). The seeds are returned into a cell array, with each cell containing 
  // a scalar (1x1 matrix) of type double.
  //
  
  //create the cell array
  unsigned T=samplerImages.size(); 
  outSeeds=mxCreateCellMatrix(1, T);
  
  
  // get the seed from each sampler and put it in the corresponding cell
  for(unsigned h=0; h<T; ++h) {
    
    //allocate the mxArray and put the pointer into the cell array
    mxArray *seed=mxCreateDoubleMatrix(1, 1, mxREAL);
    mxSetCell(outSeeds, h, seed);   
    mxGetPr(seed)[0]=(samplerImages[h])->GetSeed();
  }

}


void GetWordTopicTable(const table<unsigned> &twTable, mxArray *&outMatrix)
{
  //
  // Copy the current state of the topic table into a mxArray matrix.
  //
  
  // dimension of the table
  const unsigned V=twTable.NumberOfRows(), 
                 K=twTable.NumberOfCols();
  
  // allocate the output matrix
  outMatrix=mxCreateDoubleMatrix(V, K, mxREAL);
  double *d=mxGetPr(outMatrix);
  
  // copy the matrix
  for(unsigned k=0; k<K; ++k)
    for(unsigned h=0; h<V; ++h, ++d)
      (*d)=twTable(h,k);
}


void SaveAllSamplersStates(const vector<ABSamplerTypePr> &samplerImages, 
                           unsigned sampleNumber, const char *outFile)
{
  //
  // Save on the /tmp of the machine a snpshot of the states of the samplers.
  // These states do not contain the seeds of the random number generators.
  // This function is meant for save partial results when the computation 
  // is interrupted.
  //
  
  // the file name is always the same.
  ofstream stateFile(outFile, ios::out);

  if (!stateFile)
    throw "Unable to open tmp files to save the sampler state";
  
  // save the number of the current sample.
  stateFile<<sampleNumber<<endl;
  
  // save the total number of images we are using
  stateFile<<samplerImages.size()<<endl;
  
  // iterate over all the images
  vector<ABSamplerTypePr>::const_iterator si;
  for(si=samplerImages.begin(); si!=samplerImages.end(); ++si) {
    // save the number of observations
    stateFile<<(*si)->GetN()<<" ";
    
    // copy the state into the out put
    copy((*si)->beginState(), (*si)->endState(), 
         ostream_iterator<unsigned>(stateFile, " "));
    
    stateFile<<endl; // close the line.
  }
}
