function Cs_all=ab_sampling_list(As, Xs, alpha, NI, Cs, si, lambda, ...
                                 Ws, Vs, eta_s, xmlFiles)
%
%AB_SAMPLING_LIST
%
% Combine sampling procedure for segmenting multiple images (collections)
%
% SC=ab_sampling_list(A, W, alpha, NI, C, si, lambda, seed)
% 
% All the input, except NI, are cell arrays. Each cell contains the
% corresponding value for the ab_gibbs_sampling function. For example A{1}
% contains the affinity matrix for the first image in the collection
%
% Use 'help ab_gibbs_sampling' for more details about the input.
%
% NI: number of iterations of the sampler for generating a new sample. If
%     it is a vector then length of NI samples are generated and the number
%     of iterations before creating sample s is NI(s).
%
% SC: is a cell array containing all the samples for all the images. For 
%     example SC{1} contains the samples for the first image.
%

% some flags used in debugging
displayJumps=true;
saveTmpSample=false;

% number of images and number of samples
M=length(As);
NS=length(NI);
K=length(alpha{1});

% create empty file names is no fiels were given
if nargin~=11, xmlFiles=repmat({''}, 1, M); end;

% create the variable with all the samples
Cs_all=cell(1, M);

% init the segmentation with a little bit of Gibbs sampling.
Cs=ab_gibbs_sampling_list(As, Xs, alpha, 1, Cs, si, lambda, Ws, Vs, eta_s);

n_accept=0;
n_reject=0;

for s=1:NS
  tic;
  for h=1:NI(s);
    % compute the topic statistics
    topicTable=compTopicTable(Cs, Ws, Vs{1}, length(alpha{1}));

    % run the SW sampler for each image to move blocks
    for m=1:M
      
      for l=1:100
        Cold=Cs{m};
        [Cs{m}, topicTable, acpt]=ab_sw_sampling(As{m}, Cs{m}, Ws{m}, ...
                                                 topicTable, alpha{m}, eta_s{m});
        if (acpt)
          n_accept=n_accept+1;
          
          if (ismember(m, [2 3 4]))
            show_jump(displayJumps, xmlFiles{m}, Cold, Cs{m}, K, m);
          end
        else
          n_reject=n_reject+1;
        end
        
        if (~ismember(m, [2 3 4])), break; end
          

      end
    end

%     Cs=ab_gibbs_sampling_list(As, Xs, alpha, 2, Cs, si, ...
%                               lambda, Ws, Vs, eta_s);

    saveSample(saveTmpSample, Cs, cumsum(NI(1:s))-NI(s)+h);
    putDot(h);
    
    if (mod(h, 50)==0) || (h==NI(s))
      disp(['Acceptance ratio: ', num2str(n_accept/(n_accept+n_reject))]);
    end
  end
  
  % add the sample to the list 
  Cs_all=addSamples(Cs_all, Cs);
  disp(['Generated sample ', num2str(s), '/', num2str(NS), ...
    ' in ', num2str(NI(s)), ' iter.']);
end


%
% Auxiliary functions
%

function topicTable=compTopicTable(Cs, Ws, V, K)

% Compute the topic/table for the current segmentation
topicTable=zeros(K, V);

for m=1:length(Cs)
  for k=1:K
    topicTable(k, :)=topicTable(k, :) + hist(Ws{m}(Cs{m}==k), 1:V); 
  end
end


function Cs_all=addSamples(Cs_all, Cs)

if length(Cs_all) ~= length(Cs)
  error('Wrong number of samples');
end

for l=1:length(Cs_all)
  Cs_all{l}=cat(2, Cs_all{l}, Cs{l});
end

function saveSamples(Cs, n) %#ok<DEFNU,INUSL>

% save the samples into a MAT file

matFileName=sprintf('samples_%04d.mat', n);
save(matFileName, 'Cs');


function show_jump(display, xmlFile, Co, Cn, K, h)

% if display is false we return immediatly
if not(display), return; end

info=xml_load(xmlFile);
[ix, iy]=getGridPositions(info.width, info.height, 4);
col_num=length(unique(ix)); row_num=length(unique(iy));

figure(13);
subplot(2, 2, 1); imshow(info.original_name);
subplot(2, 2, 2); imagesc(reshape(Cn==Co, row_num, col_num)); axis equal; axis off;

segImageMAP=imresize(reshape(Co, row_num, col_num), 4, 'nearest');
segImageMAP=color_segments(segImageMAP, K, [], seg_color(K));
subplot(2, 2, 3); imshow(segImageMAP);

segImageMAP=imresize(reshape(Cn, row_num, col_num), 4, 'nearest');
segImageMAP=color_segments(segImageMAP, K, [], seg_color(K));
subplot(2, 2, 4); imshow(segImageMAP);

% [~, f]=fileparts(info.original_name);
% suptitle(f);

suptitle(['Image # ', num2str(h)]);

% reassigned segment
pause;


function saveSample(saveFlag, Cs_all, timestamp)

if saveFlag
  save('/tmp/marco/seg_results.mat', 'Cs_all', 'timestamp'); 
end
% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
