function Cs_all=ab_sampling_list_sp(As, SPs, alpha, NI, Ci, Ws, Vs, eta_s, ...
                                    xmlFiles) % the xml files are for debug
%
%AB_SAMPLING_LIST
%
% Combine sampling procedure for segmenting multiple images (collections)
%
% SC=ab_sampling_list(A, W, alpha, NI, C, si, lambda, seed)
% 
% All the input, except NI, are cell arrays. 
%
% Use 'help ab_gibbs_sampling' for more details about the input.
%
% NI: number of iterations of the sampler for generating a new sample. If
%     it is a vector then length of NI samples are generated and the number
%     of iterations before creating sample s is NI(s).
%
% SC: is a cell array containing all the samples for all the images. For 
%     example SC{1} contains the samples for the first image.
%

% some flags used in debugging
displayJumps=false;
saveTmpSample=true;

% number of images and number of samples
M=length(As);
NS=length(NI);

% number of topics and number of visual word types.
K=length(alpha{1});
T=cols(Ws{1}); 

% create the variable with all the samples
Cs_all=cell(1, M);

% init at random the superpixel in the collection
Cs=cell(1, M);
for m=1:M
  Cs{m}=initSP(SPs{m}, K, Ci{m});
end

n_accept=0;
n_reject=0;

for s=1:NS
  tic;
  for h=1:NI(s);
    % compute the topic statistics
    topicTables=compTopicTable(Cs, Ws, Vs{1}, K, T);

    % run the SW sampler for each image to move blocks
    for m=1:M
      Cold=Cs{m};
      [Cs{m}, topicTables, acpt]=ab_sw_sampling_sp(As{m}, SPs{m}, Cs{m}, ...
                                                   Ws{m}, topicTables, ...
                                                   alpha{m}, eta_s{m});
      if acpt
        n_accept=n_accept+1;
      else
        n_reject=n_reject+1;
      end
      show_jump(displayJumps, xmlFiles{m}, Cold, Cs{m}, K);
    end
    
    
    saveSample(saveTmpSample, Cs, cumsum(NI(1:s))-NI(s)+h);
    putDot(h);
    
    if (mod(h, 50)==0)
      disp(['Acceptance ratio: ', num2str(n_accept/(n_accept+n_reject))]);
    end
  end
  disp(' ');
  
  % add the sample to the list 
  Cs_all=addSamples(Cs_all, Cs);
  et=toc;
  disp(['Generated sample ', num2str(s), '/', num2str(NS), ' in ', ...
        num2str(NI(s)), ' iter. Elapsed time ', num2str(et)]);
end


%
% Auxiliary functions
%

function topicTables=compTopicTable(Cs, Ws, V, K, T)

% Compute the topic/table for the current segmentation
topicTables=cell(1, T);

for t=1:T
  topicTables{t}=zeros(K, V);
end

for m=1:length(Cs)
  
  for t=1:T
    for k=1:K
      topicTables{t}(k, :)=topicTables{t}(k, :) + hist(Ws{m}(Cs{m}==k, t), 1:V);
    end
  end
end


function Cs_all=addSamples(Cs_all, Cs)

if length(Cs_all) ~= length(Cs)
  error('Wrong number of samples');
end

for l=1:length(Cs_all)
  Cs_all{l}=cat(2, Cs_all{l}, Cs{l});
end


function show_jump(display, xmlFile, Co, Cn, K)

% if display is false we return immediatly
if not(display), return; end

info=xml_load(xmlFile);
[ix, iy]=getGridPositions(info.width, info.height, 4);
col_num=length(unique(ix)); row_num=length(unique(iy));

figure(13);
subplot(2, 2, 1); imshow(info.original_name);
subplot(2, 2, 2); imagesc(reshape(Cn==Co, row_num, col_num)); axis equal; axis off;

segImageMAP=imresize(reshape(Co, row_num, col_num), 4, 'nearest');
segImageMAP=color_segments(segImageMAP, K, [], seg_color(K));
subplot(2, 2, 3); imshow(segImageMAP);

segImageMAP=imresize(reshape(Cn, row_num, col_num), 4, 'nearest');
segImageMAP=color_segments(segImageMAP, K, [], seg_color(K));
subplot(2, 2, 4); imshow(segImageMAP);

[~, f]=fileparts(info.original_name);
suptitle(f);

% reassigned segment
pause(0.5);


function Cs=initSP(SP, K, Ci)

% create a random assignment of topics to superpixels.

Cs=zeros(size(SP));

if numel(Ci)>0
  for s=1:max(SP)
    % pixels in the current superpixel
    idx=(SP==s);
    
    % get the histogram of the labels in the current superpixel
    lh=hist(Ci(idx), 1:K);
    
    % assign the most popular label in the superpixel
    Cs(idx)=argmax(lh);
  end
  
  return;
end

% if we are here then we do not have any input label
for s=1:max(SP)  
  Cs(SP==s)=randi(K);
end


function saveSample(saveFlag, Cs_all, timestamp)

if saveFlag
  save('/tmp/marco/seg_results.mat', 'Cs_all', 'timestamp'); 
end

% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
