function SC=ab_split_merge_sampling(A, As, X, alpha, lambda, NI, C, propType)
%AB_SPLIT_MERGE_SAMPLING
% Split/Merge sampling algorithm for the affinity based clustering
%
% SC=ab_split_merge_sampling(A, alpha, NI)
% 
%  A: set of affinities (NxN matrix, possibly sparse) for the sampler(s)
%
%  A_sym: symmetric matrix of the affinities. It is used to compute the
%     "launch" state using the node potential method of L. Grady.
%
%  X: set of observation to clusters (NxF). Each row is a associated with an
%     observation and each entry is a different features. The affinitieas
%     are possibly computed using a larger set of observations. The one in X
%     are only those ones that are used to compute the global term that
%     describes the cluster, and the volume used for the uniform distribution
%     used as garbage collector.alpha: hyperparameter of the dirichlet process 
%     prior.
%
% NI: number of iterations of the sampler for generating a new sample. If
%     it is a vector then length of NI samples are generated and the number
%     of iterations before creating sample s is NI(s).
%
% C:  initial assignment for the labels
%
% propType: how the split is chosen. There are two possibilities:
%           'random_walk' and 'best_split'. By default we use random_walk.
%
% SC: new labeling of the date after NI iteration of the sampler.
%
% References:
% [1] S. Jain and R. M. Neal, "A Split-Merge Markov Chain Monte Carlo 
%     Procedure for the Dirichlet Process Mixture Model", Tech Report 
%     No. 2003, Department of Statistics, University of Toronto.
% 
% [2] L. Grady, "Space-Variant Machine Vision - A Graph Theoretic
%     Approach", Ph.D. thesis, Boston University.
%
%


% debug flag status
% DEBUG=0;


%
% check for the coherence of the inputs and construct the default ones
%

if (rows(A)~=cols(A)) 
  error('AB:SAMPLER', 'A must be square');
end

if (rows(As)~=cols(As)) 
  error('AB:SAMPLER', 'A_sym must be square');
end

if (rows(X)~=cols(A))
  error('AB:SAMPLER', 'A and X must have the same dimensions');
end

if (rows(As)~=cols(A))
  error('AB:SAMPLER', 'A and As must have the same dimensions');
end

if (any(alpha<=0))
  error('AB:SAMPLER', 'alpha must be positive');
end

% check if a starting clustering is provided
if (nargin<7)
  % By default all the points are in the same custer.
  C=ones(rows(A), 1);
  K=1;
else
  K=max(C);
end

% check if we have a propType
if (nargin<8)
  propType='random_walk';
end

%
% disable the warning for the linear system
%
warning('off', 'MATLAB:nearlySingularMatrix');
warning('off', 'MATLAB:singularMatrix');


%
% dimensions of the data
%
N=rows(A);    % number of points and dimension of the space
NS=length(NI); % number of sample


%
% Set the initial state of the sampler. 
%
SC=zeros(N, NS); % preallocation of the output

% Set the global term. Since at the beginning all tha data are in single
% cluster, then 
allGlobalTerm(K).mu=mean(X);
allGlobalTerm(K).L=chol(cov(X, 1));


%
% Run the sampler and collect the labels
%

for h=1:NS
  
  % run the sampler for NI(h) iterations before collecting a sample
  for l=1:NI(h)
    % select a pair of observations
    [i, j]=SelectPair(N);
    
    % set of indeces of the observation that we are considering 
    % in the split/merge move.
    S=(C==C(i) | C==C(j));

    % extract the restricted matrices that will be used to compute the
    % split/merge and to evaluate the acceptance probability a(C_new | C_old)
    Ar=A(S,S);  % affinities to compute the non-parametric likelihood 
    Asr=As(S,S); % affinity to compute the split/merge
    Xr=X(S, :); % Position of the observations in the space
        
    % indeces of the selected observations in the restricted matrices
    ir=sum(S(1:i)); jr=sum(S(1:j));
    Cr=ones(sum(S), 1); % preallocate labels for the cluster currently considered

    % different behaviour (either split or merge) depending on the labels
    % of the selected pair
    if C(i)==C(j)
      % The two observations are in the same cluster: split move
      
      % obtain the new state by splitting the cluster and compute the 
      % Metropolis-Hasting ratio
      [Csplit, logQsplit]=GetSplit(Asr, Xr, ir, jr, propType);
      ni=sum(Csplit==1); nj=sum(Csplit==2);
      globalTermSplit=ComputeGlobalTerm(Xr, Csplit); % TODO: not used now
      
      % compute the (log) prior ratio of the split. The prior does not
      % depend on the specific obeservation, but only on the number of
      % observation assigned to each cluster.
      logPriorRatio=ComputeLogPriorRatio(alpha, ni, nj);
      
      % compute the (log) likelihood of the split
      logLikelihoodRatio=ComputeLogLikelihood(Ar, Xr, Csplit, lambda, globalTermSplit) - ...
                         ComputeLogLikelihood(Ar, Xr, Cr, lambda, allGlobalTerm(C(i)));

      % we subtruct the logQsplit because: 
      % log(Q(Cmerge, Csplit)/Q(Csplit, Cmerge))=-log(Q(Csplit)
      a=exp(logPriorRatio+logLikelihoodRatio-logQsplit);
      
      % accept or reject the split
      if (rand<a) && (sum(Csplit==1)>1) && (sum(Csplit==2)>1)
        % accept the split
        K=K+1; % we have to add a new cluster
        Csplit(Csplit==1)=C(i);
        Csplit(Csplit==2)=K;
        C(S)=Csplit;
        
        allGlobalTerm(C(i))=globalTermSplit(1);
        allGlobalTerm(K)=globalTermSplit(2);
                
        disp('Split accepted');
      else
        if (sum(Csplit==1)==1) || (sum(Csplit==2)==1)
          disp('Single point cluster!!!');
        end

        disp('Split rejected');
      end
        
    else
      % The two observation are in the different clusters: 
      % we have to do a merge move
      
      % label 2 all the observations that have label C(j) in the original
      % assignment. All the other observations are already labeled 1.
      Cr(C(S)==C(j))=2;
      
      % obtain the new state by splitting the cluster and compute the 
      % Metropolis-Hasting ratio
      [Cmerge, logQmerge]=GetMerge(Asr, Xr, ir, jr, Cr);
      ni=sum(Cr==1); nj=sum(Cr==2);
      globalTermMerge=ComputeGlobalTerm(Xr, Cmerge);
      
      % compute the (log) prior ratio of the split. The prior does not
      % depend on the specific obeservation, but only on the number of
      % observation assigned to each cluster.
      logPriorRatio=-ComputeLogPriorRatio(alpha, ni, nj);
      
      % compute the (log) likelihood of the split
      logLikelihoodRatio=ComputeLogLikelihood(Ar, Xr, Cmerge, lambda, globalTermMerge) - ...
                         ComputeLogLikelihood(Ar, Xr, Cr, lambda, allGlobalTerm([C(i) C(j)]));

      % we subtruct the logQsplit because: 
      % log(Q(Csplit, Cmerge)/Q(Cmerge, Csplit))=log(Q(Cmerge)
      a=exp(logPriorRatio+logLikelihoodRatio+logQmerge);

      % accept or reject the merge
      if (rand<a)
        % accept the merge
        allGlobalTerm(C(i))=globalTermMerge;
        C(S)=C(i);
        disp('Merge accepted');
      else
        disp('Merge rejected');
      end
      
    end
    
  end
  
  % save the samples
  SC(:, h)=C;
%   putDot(h);

end


%%
%
% SelectPair

function [i, j]=SelectPair(N)

%
% select a random pair from the set of points
%

P=1/N*ones(N, 1);

% select the first observation
i=MultinomialRnd(P);

% select the second observation
j=MultinomialRnd(P);

% keep on selecting the second observation until it is different fron the
% first one
while (j==i), j=MultinomialRnd(P); end



%%
% 
% GetSplit
%

function [C_split, logQ]=GetSplit(Asr, Xr, ir, jr, propType)

%
% Compute a split of the dataset and return the split and the probability
% of the particular split
%

% Compute the partition probability state using the node potential method
Pr=netPowCluster(Asr, ir, jr);

switch lower(propType)
  case 'best_split'
    % First possibility: deterministic sample using the max operator
    [map_Pr, U]=max(Pr, [], 2); % indicator vector for the split assignment
    U=labels2ind(U);
    
  case 'random_walk'
    % Second possibility: random sample using the probabilities operator
    U=sample_hist(Pr', 1)'; % indicator vector for the split assignment
    
  otherwise
    error('Unrecognize option for spliting method');
    
end

%
% Compute a proposal split sampling the assignment from the probabilities
% Pr obtained from the Network potential method
%

% compute the (log)-probability of obtaining this split
Q=sum(Pr.*U, 2);
logQ=sum(log(Q+eps));

% for the moment we just consider the current split as the final proposal
% the corresponging prop dist is 1. This does not seem completely coherent
% with the Metropolis-Hasting algorithm, but at the moment it is fine.
C_split=ones(rows(U), 1);

% check which cluster is bigger and label it as 1
if sum(U(:,1))>=rows(U)
  C_split(U(:, 2)==1)=2;
else
  C_split(U(:, 1)==1)=2;
end



%%
%
% GetMerge
% 

function [Cmerge, logQ]=GetMerge(Asr, Xr, ir, jr, Cr)

%
% Compute the probability of having the split that we are merging
%

Pr=netPowCluster(Asr, ir, jr);

% These are the indicator vectors
U=labels2ind(Cr); % indicator vector

%
% Compute a proposal split sampling the assignment from the probabilities
% Pr obtained from the Network potential method
%
Q=sum(Pr.*U, 2);
logQ=sum(log(Q+eps));

Cmerge=ones(rows(Xr), 1);



%%
%
% ComputeGlobalTerms 
% 

function globalTerm=ComputeGlobalTerm(X, C)

K=max(C);
N=cols(X);

for h=1:K  
  idx=(C==h);
  globalTerm(h).mu=mean(X(idx, :));
  CovMatrix=cov(X(idx, :), 1)+0.3*eye(N);
  
  if (any(eig(CovMatrix))<=0)
    globalTerm(h).L=0.3*eye(N);
  else
    globalTerm(h).L=chol(CovMatrix);
  end
end



%%
% 
% ComputeLogPriorRatio
% 

function logPrior=ComputeLogPriorRatio(alpha, n1, n2)

% compute the logProb of the prior. This is the split case, the merge
% case is just the inverse (opposite in sign since we have the log).
logPrior=log(alpha)+logfact(n1-1)+logfact(n2-1)-logfact(n1+n2-1);



%%
% 
% ComputeLogLikelihood
% 

function logLikelihood=ComputeLogLikelihood(Ar, Xr, C, lambda, globalModels)

%
% Compute the non-parametric component of the likelihood
%

% indicator vectors. If C_new has just one label then it correspond to a
% merge move and U_new will be a vector of all ones.
U=labels2ind(C); 
Nk=sum(U); % number of element in each cluster of U_new. Again this 
           % vector has a single value if C_new comes from a split.

% likelihoods of each point to be in each cluster. Some of them can be zero
% in no point belong to that cluster (leftovers from previous operations)
L=Ar*U;
L=L./repmat(Nk, rows(U), 1); % normalize the distribution

% likelihood of each point to be in the cluster currently assigned to it
L=sum(L.*(U==1), 2);

% test if we have to compute the global term
if lambda~=0, 

  if length(globalModels)==1
    
    % we have just one cluster
    logG=logNormal(Xr, globalModels.mu, globalModels.L);
    
  else
    
    % we have two clusters
    logG=zeros(rows(Xr), 1); % preallocation
    
    % process the first cluster
    idx1=(C==1);
    logG(idx1)=logNormal(Xr(idx1, :), globalModels(1).mu, globalModels(1).L);
    
    % process the second cluster
    idx2=(C==2);
    logG(idx2)=logNormal(Xr(idx2, :), globalModels(2).mu, globalModels(2).L);
  end
  
  % add together the two terms
  L=(1-lambda)*L+lambda*exp(logG);

end
  
% compute the log of the product of all the likelihood
logLikelihood=sum(log(L+eps));



% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
