function [C, topicTable, accept]=ab_sw_sampling(A, C, W, topicTable, alpha, eta)
%
% Cs=AB_sw_sampling(A, C, W, topicTable, alpha, eta)
%
% Sampling procedure based on the SW algorithm. Given an image (affinitied 
% and visual words) it reassigns a chunk of pixels to a new cluster.
%
% A: affinity matrix
% 
% C: Current segmentation for the given image
%
% W: Visual words in the image
%
% topicTable: table with the counts of visual words/segment assignment
%
% alpha: hyperparameter for the Dirichlet prior over the mixture coeff.
%
% eta: hyperparameter for the symmetric Dirichlet prior over the topic
%      distribution over visual words.
%
%
% The outputs are the new segmentation and the new likelihood.
%


% generate the cut using the SW algorithm
[idx_Vo, ~]=GenerateCut(A, C);

% obtain the label of selected macro-region in the current segmentation
old_label=C(find(idx_Vo, 1));

% sample a new label for the selected macro-region. The new sample is
% based on the similarity between the visual words.
[new_label, lr, newTopicTable]=sampleTopic(topicTable, eta, ...
                                            W(idx_Vo, :), old_label);

if (old_label==new_label)
  accept=false;
  return;
end
                                       
% Compute assignmet prob.

% we have to collect all the affinities between the selected connected
% component and the verteces in the old and new
log_old_edges=log(nonzeros(A(idx_Vo, ~idx_Vo & C==old_label)));
log_new_edges=log(nonzeros(A(idx_Vo, ~idx_Vo & C==new_label)));

% compute the log of prob. ratio
logQr=sum(log_old_edges)-sum(log_new_edges)+lr;

% compute the likelihood ratio for the old and new conficuration

% compute the likelihood ratio for the old and new configuration
% Cn and topicTable_new are the new value for the 
Cn=C; 
Cn(idx_Vo)=new_label;

% compute the logLikelihood of the old assignment p(x|c)p(w|c)p(c)
ll_old=compLogLikelihood(A, C, topicTable, alpha, eta);

% compute the logLikelihood of the old assignment p(x|c)p(w|c)p(c)
ll_new=compLogLikelihood(A, Cn, newTopicTable, alpha, eta);

% compute the MH ratio and decide if we have to accept the split or not.
a=min(1, exp(logQr+ll_new-ll_old));

if (rand<a)
  C=Cn;
  topicTable=newTopicTable;
  accept=true;
else
  accept=false;  
end

%%

%
% Auxiliary functions
%


% function [idx, P]=GenerateCut(A, C)
% 
% %
% % Generate the cut for a segmentation. The matrix A represents the graph,
% % the vector C represents the segmentation.
% %
% 
% % obtain the edge representation of the graph.
% [a_i, a_j, a_v]=find(A);
% 
% ne=length(a_v); % number of edges in the graph
% nv=size(A, 1); % number of verteces in the graph
% 
% % activate the edges depending on their affinity value and remove all the 
% % edges that connect nodes in different segments
% idx_act=(rand(ne, 1)<a_v);
% 
% % keep only the activated edges 
% a_i=a_i(idx_act);
% a_j=a_j(idx_act);
% 
% idx_act=(C(a_i)==C(a_j));
% a_i=a_i(idx_act);
% a_j=a_j(idx_act);
% 
% 
% % compute the connected components 
% [S, P, CS]=getConnComp(a_i, a_j, nv); % S is the number of connected components and P 
%                                       % is the labeling of the nodes in the graph in
%                                       % different connected components.
% 
% small=true;
% if all(CS<5)
%   cl=randint(1, 1, [1 S]);
%   small=false;
%   idx=(P==cl);
%   
% %   disp('All small!!!');
% end  
% 
% while small
%   % select one connected component
%   cl=randint(1, 1, [1 S]);
% 
%   % get all the verteces in the selected connected component
%   idx=(P==cl);
%   small=sum(idx)<5;
% end



function [idx, P]=GenerateCut(A, C)

%
% Generate the cut for a segmentation. The matrix A represents the graph,
% the vector C represents the segmentation.
%

% obtain the edge representation of the graph.
[a_i, a_j, a_v]=find(A);

ne=length(a_v); % number of edges in the graph
nv=size(A, 1); % number of verteces in the graph

% activate the edges depending on their affinity value and remove all the 
% edges that connect nodes in different segments
idx_act=(rand(ne, 1)<a_v);

% keep only the activated edges 
a_i=a_i(idx_act);
a_j=a_j(idx_act);

idx_act=(C(a_i)==C(a_j));
a_i=a_i(idx_act);
a_j=a_j(idx_act);


% compute the connected components 
[S, P, CS]=getConnComp(a_i, a_j, nv); % S is the number of connected components and P 
                                      % is the labeling of the nodes in the graph in
                                      % different connected components.
small=true;
if all(CS<5)
  cl=randi([1 S], 1, 1);
  small=false;
  idx=(P==cl);
  
%   disp('All small!!!');
end  

while small
  % select one connected component
  cl=randi([1 S], 1, 1);

  % get all the verteces in the selected connected component
  idx=(P==cl);
  small=sum(idx)<5;
end


function [new_label, lr, topicTable]=sampleTopic(topicTable, eta, W, old_label)

V=cols(topicTable);

% compute the sample probability of the topics for the current
betas=scale_rows(topicTable+eta, 1./row_sum(topicTable));

% compute the logP along the rows
logPz=row_sum(log(betas(:,W)));
Pz=exp(logPz-max(logPz));
Pz(old_label)=0; % force to zero the probability of reassigning the old label
Pz=Pz/sum(Pz); % normalize to one. This is important to return the correct
               % ratio between proposal distribution

% generate the new sample
new_label=sample(Pz', 1);

% change the topicTable to reflect the new assignment and compute the
% probability of generating the old label with the updated labels

ht=hist(W, 1:V); % histogram of the visual words in the superpixels

topicTable(old_label, :)=topicTable(old_label, :)-ht;
topicTable(new_label, :)=topicTable(new_label, :)+ht;

% betas for the new configureation. We need to update only the two
% diffenrent labels
betas(old_label, :)=topicTable(old_label, :)./sum(topicTable(old_label, :));
betas(new_label, :)=topicTable(new_label, :)./sum(topicTable(new_label, :));

% compute the proposal distribution for obtaining the old label starting
% from the new state.
logPz_new=row_sum(log(betas(:,W)));
Pz_new=exp(logPz_new-max(logPz_new));
Pz_new(new_label)=0; % again force to zero this probability
Pz_new=Pz_new/sum(Pz_new);

lr=log(Pz(old_label))-log(Pz(new_label));



% function [new_label, lr]=sampleLabel(Wo, Co, curr_label, topicTable)
% 
% K=rows(topicTable);
% V=cols(topicTable);
% 
% % obtain the histogram of the visual words in the selected region
% vw_hist=hist(Wo, 1:V);
% betas_Vo=vw_hist/sum(vw_hist);
% 
% % remove the effect of the current region from the topicTable and compute
% % the betas
% for k=1:K, topicTable(k,:)=topicTable(k, :)-hist(Wo(Co==k), 1:V); end
% curr_betas=topicTable./repmat(row_sum(topicTable), 1, cols(topicTable));
% 
% % compute the assignment prob to a different cluster based on the
% % similarity of the visual word histogram and the KLdiv.
% div=zeros(1, K);
% for k=1:K, div(k)=KLdiv(betas_Vo, curr_betas(k, :)); end
% 
% T=10;
% topic_prob=exp(-div.^2./T);
% topic_prob=(topic_prob+eps)./sum(topic_prob+eps); %#ok<NASGU>
% 
% % at this point we should compute the new label and the ratio between the
% % two proposal probabilities. We select the new label at random, but in
% % principle we could use the statistics of the visual words
% % new_label=sample(topic_prob);
% new_label=sample(ones(1, K)/K);
% 
% lr=log(topic_prob(curr_label)/topic_prob(new_label));
% 
% 
% 
% function d=KLdiv(p, q)
% 
% d=sum(p.*log2(p./(q+eps)+eps));



function [S, P, CP]=getConnComp(a_i, a_j, nv)

% compute the connected components
G=sparse(a_i, a_j, 1, nv, nv);
G=G+speye(nv);
[p, ~, r, s]=dmperm(G);

S=length(r)-1; % number of components
P=zeros(nv, 1); % labeling of the components

for k=1:S
  idx=r(k):(r(k+1)-1);
  P(p(idx))=k;
end

CP=hist(P, 1:S); % cardinality of the components



% function topicTable_new=updateTopicTable(topicTable, W, C, Cn)
% 
% K=rows(topicTable);
% V=cols(topicTable);
% 
% % 'remTable' are the histograms of the visual words 
% % for the old configuration
% remTable=zeros(K, V); 
% 
% % 'addTable' are the histograms of the visual words 
% % for the new configuration
% addTable=zeros(K, V);
% 
% for k=1:K
%   remTable(k, :)=hist(W(C==k), 1:V);
%   addTable(k, :)=hist(W(Cn==k), 1:V);
% end
% 
% % remove the counts for the old configuration and update it with the new
% % configuration.
% topicTable_new=topicTable-remTable+addTable;
% 
% % a sanity check
% if (any(any(topicTable_new<0)))
%   error('Something wrong: negative histogram');
% end



function ll=compLogLikelihood(A, C, topicTable, alpha, eta)

K=rows(topicTable);
V=cols(topicTable);

% size of the segments
Nk=hist(C, 1:K);

% likelihood of the observations in each cluster
U=labels2ind(C, K);
Fk=sum((A*U).*U, 2); % Fk is a column vector of dimension N with 
                     % the likelihod of each observation to be in cluster
                     % assigned cluster.
if any(Fk==0)
  warning('We have a zero likelihood in the affinity term'); %#ok<WNTAG>
end

Fk=Fk./(Nk(C))';

% first likelihood term: log p(x | c)
ll_term1=sum(log(Fk));

% second likelihood term: log p(w | c)
ll_term2=K*(gammaln(V*eta)-V*gammaln(eta))+...
  sum(sum(gammaln(topicTable+eta)))-sum(gammaln(sum(topicTable+eta)));

% prior term: log p(c)
ll_term3=gammaln(sum(alpha))-gammaln(sum(alpha+Nk))+...
         sum(gammaln(alpha+Nk))-sum(gammaln(alpha));

ll=ll_term1+ll_term2+ll_term3;



function display_topic_prob(topic_prob) %#ok<DEFNU>

K=length(topic_prob);

% display the divergence 
seg_color_map=[1 0 0;
               1 1 0;
               0 1 0;
               0 1 1;
               0 0 1];

clf;
for k=1:K
  tmp=zeros(1, K);
  tmp(k)=topic_prob(k);
  hold on;
  bar(1:K, tmp, 'FaceColor', seg_color_map(k, :));
  hold off;
end



function c=label2color(idx) %#ok<DEFNU>

colors={'red', 'yellow', 'green', 'cyan', 'blue'};

c=colors{idx};

% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
