function [C, topicTables, accept]=ab_sw_sampling_sp(A, SP, C, W, ...
                                                   topicTables, alpha, eta)
%
% Cs=AB_sw_sampling_sp(A, C, W, topicTable, alpha, eta)
%
% Sampling procedure based on the SW algorithm. Given an image (affinitied 
% and visual words) it reassigns a chunk of pixels to a new cluster.
%
% A: affinity matrix
%
% SP: superpixel partition of the image. Used for the proposal
% distribution.
% 
% C: Current segmentation for the given image
%
% W: Visual words in the image
%
% topicTable: table with the counts of visual words/segment assignment
%
% alpha: hyperparameter for the Dirichlet prior over the mixture coeff.
%
% eta: hyperparameter for the symmetric Dirichlet prior over the topic
%      distribution over visual words.
%
%
% The outputs are the new segmentation and the new likelihood.
%


% generate the cut using the SW algorithm
idx_Vo=GenerateCut(SP);

% obtain the label of selected macro-region in the current segmentation
old_label=C(find(idx_Vo,1));

% sample a new label for the selected macro-region. The new sample is
% based on the similarity between the visual words.
[new_label, lr, newTopicTables]=sampleTopic(topicTables, eta, ...
                                             W(idx_Vo, :), old_label);
if (old_label==new_label)
  % this may spare us a little work
  accept=true;
  return;
end

% Compute assignmet prob.

% compute the likelihood ratio for the old and new configuration
% Cn and topicTable_new are the new value for the 
Cn=C; 
Cn(idx_Vo)=new_label;

% compute the logLikelihood of the old assignment p(x|c)p(w|c)p(c)
ll_old=compLogLikelihood(A, C, topicTables, alpha, eta);

% compute the logLikelihood of the old assignment p(x|c)p(w|c)p(c)
ll_new=compLogLikelihood(A, Cn, newTopicTables, alpha, eta);

% compute the MH ratio and decide if we have to accept the split or not.
a=min(1, exp(lr+ll_new-ll_old));

if (rand<a)
  C=Cn;
  topicTables=newTopicTables;
  accept=true;
else
  accept=false;  
end


function idx_cut=GenerateCut(SP)

while true
  selectSP=randi(max(SP)); % label of the superpixel we are considering

  % select the pixel we want to use.
  idx_cut=(SP==selectSP);
  
  if any(idx_cut)
    % if there is at least an observation in the superpixel then we can
    % leave the function
    return; 
  end
end



function [new_label, lr, topicTables]=sampleTopic(topicTables, eta, ...
                                                   W, old_label)

T=length(topicTables); % number of visual word topics
V=cols(topicTables{1}); % all the dictionary have the same number of words.
K=rows(topicTables{1});


% compute the sample probability of the topics for the current
betas=cell(1, T);
for t=1:T
  betas{t}=scale_rows(topicTables{t}+eta, 1./row_sum(topicTables{t}));
end

% compute the logP along the rows
logPz=zeros(K, 1);
for t=1:T
  logPz=logPz+row_sum(log(betas{t}(:,W(:, t))));
end
Pz=exp(logPz-max(logPz));
Pz=Pz/sum(Pz); % normalize to one. This is important to return the correct
               % ratio between proposal distribution

% generate the new sample
new_label=sample(Pz', 1);

% change the topicTable to reflect the new assignment and compute the
% probability of generating the old label with the updated labels

for t=1:T
  ht=hist(W(:, t), 1:V); % histogram of the visual words in the superpixels

  topicTables{t}(old_label, :)=topicTables{t}(old_label, :)-ht;
  topicTables{t}(new_label, :)=topicTables{t}(new_label, :)+ht;

  % betas for the new configureation. We need to update only the two
  % diffenrent labels
  betas{t}(old_label, :)=topicTables{t}(old_label, :)./ ...
                          sum(topicTables{t}(old_label, :));
  betas{t}(new_label, :)=topicTables{t}(new_label, :)./ ...
                          sum(topicTables{t}(new_label, :));

end


% compute the proposal distribution for obtaining the old label starting
% from the new state.
logPz_new=zeros(K, 1);
for t=1:T
  logPz_new=logPz_new+row_sum(log(betas{t}(:,W(:, t))));
end
Pz_new=exp(logPz_new-max(logPz_new));
Pz_new=Pz_new/sum(Pz_new);

lr=log(Pz(old_label))-log(Pz(new_label));


function ll=compLogLikelihood(A, C, topicTables, alpha, eta)

T=length(topicTables);
K=rows(topicTables{1});
V=cols(topicTables{1});

% size of the segments
Nk=hist(C, 1:K);

% likelihood of the observations in each cluster
U=labels2ind(C, K);
Fk=scale_cols((A*U), 1./Nk); % Fk is a column vector of dimension N with the 
                             % likelihod of each observation to be in cluster
                             % assigned cluster.
Fk=row_sum(Fk.*U);

if any(Fk==0)
  warning('We have a zero likelihood in the affinity term'); %#ok<WNTAG>
end

% first likelihood term: log p(x | c)
ll_term1=sum(log(Fk));


ll_term2=0;
for t=1:T
  % second likelihood term: log p(w | c)
  ll_term2=ll_term2 +  ...
           K*(gammaln(V*eta)-V*gammaln(eta))+ ...
           sum(sum(gammaln(topicTables{t}+eta)))-...
           col_sum(gammaln(row_sum(topicTables{t}+eta)));
end

% prior term: log p(c)
ll_term3=gammaln(sum(alpha))-gammaln(sum(alpha+Nk))+...
         sum(gammaln(alpha+Nk))-sum(gammaln(alpha));

ll=ll_term1+ll_term2+ll_term3;



function display_topic_prob(topic_prob) %#ok<DEFNU>

K=length(topic_prob);

% display the divergence 
seg_color_map=[1 0 0;
               1 1 0;
               0 1 0;
               0 1 1;
               0 0 1];

clf;
for k=1:K
  tmp=zeros(1, K);
  tmp(k)=topic_prob(k);
  hold on;
  bar(1:K, tmp, 'FaceColor', seg_color_map(k, :));
  hold off;
end



function c=label2color(idx) %#ok<DEFNU>

colors={'red', 'yellow', 'green', 'cyan', 'blue'};

c=colors{idx};

% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
