function [R, g, L]=ab_variational(A, X, alpha, Ni, Z, lambda) %, row_num, col_num)
%AB_VARIATIONAL
% Variational inference for the affinity-based clustering.
%
% [R, g, L]=ab_variational(A, alpha, Z)
% 
% A: set of affinities (NxN)
%
% X: list of feature vectors
%
% alpha: hyperparameter of the Dirichlet prior distribution.
%        its lenght is the number of clusters. 
%
% Ni: Number of iterations
% 
% Z: Initial labeling of the observations. If not present or equal to []
%    then it is randomly generated.
%
% lambda: mixture coefficient for the parametric term. If zero no
%         parametric term is used.
%
% R: Responsabilities for each observation. They are the parameters of the 
%    variational distribution Qz.
%
% g: parameters of the variational distribution Qt. This distribution has
%    the functional form of a Dirichlet distribution.
%
% L: lower bound of the log-likelihood of the model at each iteration.
%
%


%% Check for the coherence of the inputs
if (rows(A)~=cols(A)) 
  error('AB:VARIATIONAL', 'A must be square');
end

% if (rows(A)~=rows(X))
%   error('AB:VARIATIONAL', 'A and X must have the same number of rows');
% end

if (any(alpha<=0))
  error('AB:VARIATIONAL', 'alpha must be strictly positive');
end

% dimension of the problem/model
N=rows(A);  % number of points and dimension of the space
K=length(alpha); % number of clusters

% reshape alpha as a row vector
alpha=reshape(alpha, 1, length(alpha));

% fix the number of iteration for the variational optimizations
Nv=1;


%% Set the default arguments:

% no starting labeling
if (nargin<5), Z=[]; end 

% init the sampler
if (isempty(Z))
  Z=unidrnd(K, N, 1); % vector of assignment;
else
  % the labels are provided as input. we need to check that 
  % they make sense
  if ( (any(Z<1)) || (any(Z>K) ) )
    error('AB:SAMPLER', 'incoherent assignment C');
  end
end

% fix the mixing coefficient for the parametric and non-parametric term
if (nargin<6), lambda=0.0; end

if ((lambda<0) || (lambda>1))
  error('AB:VARIATIONAL', 'lambda must be between 0 and 1');
end


%% Run the variational inference

%
% At the beginning the responsibilities are equal to the assignment Z
%

% transform the labels in indicator vectors
R=labels2ind(Z, K);  % this are the responsibilities
Nk=sum(R); % sum of the responsabilities for each cluster
g=alpha+Nk; % initial setting for the function Qt

% init the global term if necessary
if (lambda~=0)
  % add also the global term.
  [mu, Sigma]=CompParamStat(X, R);
  
  disp('Using the parametric term');
end


% preallocate the log-likelihood
L=zeros(Ni, 1);


for h=1:Ni
  
  %
  % First step: for each observation x_n compute the (approximate)
  % likelihood f^_k(x_n)
  %
  
  Fk=A*R; % we will use these values as surrogate of the true (unkown) Fk
  Fk=Fk./repmat(Nk, N, 1); % normalize the distributions
  
  if (lambda~=0)
    % add also the global term.
    Fk=(1-lambda)*Fk+lambda*CompParmTerm(X, mu, Sigma);
  end

  %
  % Second step minimize the variational problem. This means minimize the
  % KL(q|p) with q=qt*qz
  % 
  
  for m=1:Nv
    % 
    % Compute the new responsabilities R using Fk and g
    %

    R=Fk.*repmat(exp(psi(g)), N, 1);
    R=R./repmat(sum(R, 2), 1, K); % normalize responsabilities

    % compute the total responsibility for each cluster
    Nk=sum(R);


    %
    % Using the new responsabilies update the paramete g of the 
    % variational distribution Qt.
    %

    g=alpha+Nk;
  end
  
  % init the global term
  if (lambda~=0)
    % add also the global term.
    [mu, Sigma]=CompParamStat(X, R);
  end
  
%   % try to assign the MAP
%   [Rm, Z]=max(R, [], 2);  
%   R=labels2ind(Z, K);  % this are the responsabilities
%   Nk=sum(R); % sum of the responsabilities
%   g=alpha+Nk; % initial setting for the function


  %
  % Compute the lower bound of the log-likelihood of the model
  % 
  
%   % precompute E_q [ log \theta ]
%   lnt=digamma(g)-digamma(sum(g));
  
  L(h)=0;
%   L(h)=sum(sum(log(Fk).*R))+...
%        sum(sum(R.*repmat(lnt, N, 1)))+ ...
%        logC(alpha)+sum((alpha-1).*lnt)+...
%        -(sum(sum(R.*log(R))))+...
%        -(logC(g)+sum((g-1).*lnt));

%   %
%   % a little bit of debug
%   %

  figure(11); title('Evolution of the variational inference');

  [foo, Cmap]=max(R, [], 2);
  segImageMAP=reshape(Cmap, 144, 176); % reshape the 
  outImage=color_segments(segImageMAP, K, []);
  imshow(outImage);
  disp(['Iteration number: ', num2str(h)]);
  pause(0.1);

end


%%
function r=logC(alpha)

r=gammaln(sum(alpha))-sum(gammaln(alpha));


%%
function [mu, Sigma]=CompParamStat(X, R)

%
% Compute the mu and Sigma of each cluster using the responsabilities
%

K=cols(R); % number of clusters
F=cols(X); % number of dimensions of the feature space.


% preallocate
mu=zeros(K, F);
Sigma=zeros(F,F, K);

for k=1:K
  
  % weight of the summation
  W=repmat(R(:,k)/sum(R(:,k)), 1, F);
    
  % mean
  mu(k,:)=sum(W.*X, 1);
  
  % covariance
  y=X-repmat(mu(k,:), rows(X), 1); % center observations
  Sigma(:,:,k)=y'*(W.*y);  
end


%%
function F=CompParmTerm(X, mu, Sigma)

% number of clusters
K=size(Sigma, 3);

F=zeros(rows(X), K);

for k=1:K
  F(:,k)=gauss(mu(k,:), Sigma(:,:,k), X);
end  




% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
