function [R, g, betas]=ab_variational_list(A, W, alpha, Ni, eta, V)
%AB_VARIATIONAL
% Variational inference for segmenting multiple images (collections)
%
% [R, g, b, ll]=ab_variational_list(A, W, alpha, Ni, eta, V)
% 
% A: cell array with all the affinity matrices
%
% W: cell array with all the list of visual words
%
% alpha: Dirichlet hyperparameter for the mixture component
% 
% Ni: number of iterations of the sampler for generating a new sample. If
%     it is a vector then length of NI samples are generated and the number
%     of iterations before creating sample s is NI(s).
%
% eta: hyperparameter for the Dirichlet distribution of the topics
%
% V: number of visual words in the dictionary
%
% R: cell array with all the responsibilities for each observation. 
%    They define the variational distribution q(Z) for each image.
%
% g: hyperparameters for the variational distribution q(\theta)
%
% b: hyperparameters for the variational distribution q(\phi)
%
% ll: log-likelihood log p(X, W, Z)
%
% ll_vb: variational lower bound of the log-likelihood log p(X, W)
%

% dimension of the problem/model
M=length(A); % number of images in the collections  
K=length(alpha); % number of segments/topics

% reshape alpha as a row vector
alpha=reshape(alpha, 1, length(alpha));

% image to debug
USE_DEBUG=false; %#ok<NASGU>

% preallocate the cell arrays
% Fk=cell(1, M);
R=cell(1, M);
g=cell(1, M);
Nk=cell(1, M);

% init the topic distribution
betas=eta*ones(K, V);

% init the variational distributions randomly assigning the observation to
% clusters
for m=1:M
%   Fk{m}=zeros(length(W{m}), K);
  R{m}=labels2ind(unidrnd(K, length(W{m}), 1));
  Nk{m}=col_sum(R{m});
  g{m}=alpha+Nk{m};
  betas=betas+sumTopicResp(R{m}, W{m}, V);
end
edBetas=exp(digamma(betas)-repmat(digamma(row_sum(betas)), 1, V));


% disp('Start variational inference');
for s=1:Ni
  
  % a little nit of debugging info
%  displayBlendImage(R{1}, ['Iteration ', num2str(s-1)]);

  
  % decide which type of iteration we are using
  switch iterType(s, Ni)
    
    % run variational alg.
    case 0, [R, g, Nk, betas]=run_variational_with_topic(A, W, R, g, Nk, alpha, ...
                                                  eta, edBetas);

    % run smooth variational alg. (without topic term)
    case 1, [R, g, Nk, betas]=run_variational_smooth(A, W, R, g, Nk, alpha, eta, V);
    
%     % force hard label assignment
%     case 3, [R, g, Nk, betas]=run_variational_hard(R, W, eta, V);
    
  end

  % q(\phi)
  edBetas=exp(digamma(betas)-repmat(digamma(row_sum(betas)), 1, V));
    
  % display the progres bar.
  putDot(s);
end
disp(' ');

% a little nit of debugging info
% displayBlendImage(R{1}, ['Iteration ', num2str(s)]);



% =======================================================================
% Auxiliary functions
% =======================================================================

function type=iterType(s, Ni)

% always smooth the last 10 iterations
if s>(Ni-10), 
  type=1; 
  return; 
end

type=0;

% if (s<50)
%   if mod(s, 20)<16, type=0; return; else type=1; return; end
% 
% else
% %   if mod(s, 25)==0, type=2; return; end
%   if mod(s, 30)<20, type=0; return; else type=1; return; end
% end



% =======================================================================
% Different types of variational step
% =======================================================================
  
function [R, g, Nk, betas]=run_variational_with_topic(A, W, R, g, Nk, alpha, ...
                                                        eta, edBetas)
  
% initial value for this statistics
K=length(alpha);
V=cols(edBetas);
M=length(A);

betas=eta*ones(K, V);

for m=1:M
  % q(Z) This is the most intensive line of code
  R{m}=scale_cols(A{m}*R{m}, exp(digamma(g{m}))./Nk{m});   

  % consider the constribution of the topics
  R{m}=R{m}.*edBetas(:, W{m})'; 

  % normalize the responsabilities.
  R{m}=scale_rows(R{m}, 1./row_sum(R{m}));

  % q(\theta)
  Nk{m}=col_sum(R{m});
  g{m}=alpha+Nk{m};

  % collect the data for the new betas
  betas=betas+sumTopicResp(R{m}, W{m}, V);

end


function [R, g, Nk, betas]=run_variational_smooth(A, W, R, g, Nk, alpha, eta, V)
  
% initial value for this statistics
K=length(alpha);
betas=eta*ones(K, V);
M=length(A);

for m=1:M
  % q(Z) This is the most intensive line of code
  R{m}=scale_cols(A{m}*R{m}, exp(digamma(g{m}))./Nk{m});   

  % normalize the responsabilities.
  R{m}=scale_rows(R{m}, 1./row_sum(R{m}));

  % q(\theta)
  Nk{m}=col_sum(R{m});
  g{m}=alpha+Nk{m};

  % collect the data for the new betas
  betas=betas+sumTopicResp(R{m}, W{m}, V);

end


function [R, g, Nk, betas]=run_variational_hard(R, W, eta, V)

K=cols(R{1});
M=length(R);

betas=eta*ones(K, V);
g=cell(1, M);
Nk=cell(1, M);


for m=1:M
  % q(Z) This is the most intensive line of code
  [~, Z]=max(R{m}, [], 2);
  R{m}=labels2ind(Z, K);

  % q(\theta)
  Nk{m}=col_sum(R{m});
  g{m}=alpha+Nk{m};

  % collect the data for the new betas
  betas=betas+sumTopicResp(R{m}, W{m}, V);
end


% function [R, betas]=run_split_merge(A, W, R, betas, alpha, eta)
% 
% % number of images in the collection
% M=length(A);
% 
% for m=1:M
% 
%   % compute the merge cost Jm for this image.
%   [Jm, k1, k2]=GetMergeHyp(R{m});
%   
%   % compute the split cost Js for each segment
%   topic_dist=scale_rows(betas, 1./row_sum(betas));
%   [Js, k3]=GetSplitHyp(R{m}, W{m}, topic_dist);
%   
%   % obtain the L best hyphotesys
%   ch=bestHyp(k1, k2, k3, 5);
%   
%   % find which one improve the likelihood
%   [R{m}, betas]=compNewPost(A{m}, W{m}, R{m}, betas, ch, alpha, eta);
% end
% 
% 
% 
% % =======================================================================
% % Auxiliary function for the split and merge update
% % =======================================================================
% 
% function [Jm, k1, k2]=GetMergeHyp(R)
% 
% K=cols(R);
% Jm=triu(R'*R, 1);
% [Jm, idx]=sort(Jm(:), 'descend');
% Jm=Jm(1:((K*K-1)/2));
% idx=idx(1:((K*K-1)/2)); 
% [k1, k2]=ind2sub([K, K], idx);  % moving from m to n  
% 
% function [Js, k3]=GetSplitHyp(R, W, betas)
% 
% V=cols(betas);
% 
% t=sumTopicResp(R, W, V); 
% t=scale_rows(t, 1./row_sum(t)); % normalize
% 
% % compute the KL distance between the global topic and the local topics
% Js=row_sum(t.*log(t./betas+eps));
% [Js, k3]=sort(Js, 'descend');
% 
% function c=bestHyp(k1, k2, k3, L)
% 
% % all the possible candidates
% c=[repmat([k1, k2], length(k3), 1), repmat(k3, length(k1), 1)];
% 
% % remove all the tuples with a repetition
% c(c(:,1)==c(:,3) | c(:,2)==c(:,3), :)=[];
% 
% L=min(L, rows(c));
% c=c(1:L,:); 
% 
% function [R, betas]=compNewPost(A, W, R, betas, c, alpha, eta)
% 
% V=cols(betas);
% 
% % current likelihood value
% ll=singleImageLogLikelihood(A, W, R, betas, alpha, eta);
% 
% % iterate over all the candidates. As soon as we find a new configuration
% % that improves the likelihood we stop and do the transformation
% foundImpr=false;
% for h=1:rows(c)
%   
%   k1=c(h, 1);
%   k2=c(h, 2);
%   k3=c(h, 3);
%   
%   Rnew=R;
%   Rnew(:, k1)=Rnew(:, k1)+Rnew(:, k2);
%   Rnew(:, k2)=0;
%   
% %   idx1=R(:, k3) < R(:, k2);
% %   idx2=not(idx1);
% %   Rnew(:, k2)=R(:, k3).*(idx1);
% %   Rnew(idx2, k2)=R(idx2, k3).*(idx2);
%   
%   betas_new=betas-sumTopicResp(R, W, V)+sumTopicResp(Rnew, W, V);
% 
%   if (ll<singleImageLogLikelihood(A, W, Rnew, betas_new, alpha, eta))
%     foundImpr=true;
%   end
%   
% end
% 
% if foundImpr
%   R=Rnew;
%   betas=betas_new;
% end
  

% function ll=singleImageLogLikelihood(A, W, R, betas, alpha, eta)
% 
% K=rows(betas);
% V=rows(betas);
% 
% % hard labeling
% [m, C]=max(R, [], 2);
% 
% % size of the segments
% Nk=hist(C, 1:K);
% 
% % likelihood of the observations in each cluster
% U=labels2ind(C, K);
% Fk=sum((A*U).*U, 2); % Fk is a column vector of dimension N with 
%                      % the likelihod of each observation to be in cluster
%                      % assigned cluster.
% 
% Fk=Fk./(Nk(C))';
% 
% % first likelihood term: log p(x | c)
% ll_term1=sum(log(Fk+eps));
% 
% % second likelihood term: log p(w | c)
% ll_term2=K*(gammaln(V*eta)-V*gammaln(eta))+...
%   sum(sum(gammaln(betas)))-sum(gammaln(sum(betas)));
% 
% % prior term: log p(c)
% ll_term3=gammaln(sum(alpha))-gammaln(sum(alpha+Nk))+...
%          sum(gammaln(alpha+Nk))-sum(gammaln(alpha));
% 
% ll=ll_term1+ll_term2+ll_term3;
% 
% 
% 
% % =======================================================================
% % Compute likelihood and lower bound
% % =======================================================================
% 
% function ll=compLogLikelihood(R, As, Ws, alpha, eta, V) %#ok<INUSD,DEFNU>
% 
% M=length(As);
% Z=cell(1, M);
% for m=1:M, [t, Z{m}]=max(R{m}, [], 2); end
% 
% % make the cell arrays
% Xs=xrepmat({[]}, 1, M);
% alpha=xrepmat({alpha}, 1, M);
% lambda=xrepmat({zeros(size(alpha{1}))}, 1, M);
% eta_s=xrepmat({eta}, 1, M);
% Vs=xrepmat({V}, 1, M);
% si=xrepmat({[]}, 1, M);
% 
% [C, ll]=ab_gibbs_sampling_list(As, Xs, alpha, 0, Z, si, lambda, Ws, Vs, eta_s);
% 
% 
% function ll_vb=varLowerBound(R, A, W, alpha, edBetas, V) 
% 
% % number of images
% M=length(R);
% 
% % iterate over all the images
% ll_vb=zeros(1, M);
% for m=1:M
%   ll_vb(m)=single_image_ll_vb(R{m}, A{m}, W{m}, alpha, edBetas, V);
% end
% 
% function ll_vb=single_image_ll_vb(R, A, W, alpha, betas, g, eta)
% 
% % 5 terms from E_q[ log P(x, w, z, theta, phi | alpha, eta)]
% 
% % term 1:  E_q[ log p(x|z) ]
% Fk=scale_cols(A*R, 1./col_sum(R)); % we approximate Fk as the average density
% ll(1)=sum(sum(R.*log(Fk+eps)));
% 
% % term 2: E_q[ log p(w|z, phi) ]
% ll(2)=sum(sum(R.*log(edBetas(:, W)'))); %#ok<COLND>
% 
% % term 3: E_q[ log p( z| theta) ]
% ll(3)=sum(sum(scale_rows(R, g)));
% 
% % term 4: E_q[ log p(theta | alpha)]
% ll(4)=gammaln(sum(alpha))-sum(gammaln(alpha))+sum( (alpha-1) .* E_log_t(g));
% 
% % term 5: E_q[ log p(phi | eta)]
% ll(5)=0;
% 
% % 3 terms from the E_q[ log q(theta, phi, z) ]
% 
% % term 6: E_q[ log q(theta) ]
% ll(6)=gammaln(sum(g))-sum(gammaln(g))-sum((gamma-1).*E_log_t(g));
% ll(6)=-ll(6);
% 
% % term 7: E_q[ log q(phi) ]
% ll(7)=0;
% ll(7)=-ll(7);
% 
% % term 8: E_q[ log q(z) ]
% ll(8)=sum(sum(betas.*log(betas+eps)));
% ll(8)=-ll(8);
% 
% % add together all the terms to obtain the variational lower bound
% ll_vb=sum(ll);
  


% =======================================================================
% Debug subfunctions
% =======================================================================

function displayBlendImage(R, titleStr) 
%
% displayBlendImage 
%
% Given the reposabilities 'R' display an image with different segments 
% blended together accordingly to 'R'.
%

% number of segments topics
K=cols(R);

% dimension of the image
N=79;
M=rows(R)/N;

%  probability planes
P=zeros(M, N, K);
for k=1:K
  P(:,:,k)=reshape(R(:,k),M, N);
end

% obtain the blend
blend=blend_segments(P, K, [], seg_color(K));

% display and wait.
figure(13); imshow(blend);
title(titleStr);
pause(0.1);


function displayResp(R, Fk, W, b, m, s, USE_DEBUG) %#ok<DEFNU>

% skip the display if we are not debugging
if ~USE_DEBUG, return; end

K=cols(R);
L=ceil(K/3);

figure(11); clf;

for k=1:K
  subplot(L, 3, k);
  
  I=reshape(R(:, k), 59, 79);
  I(1,1)=0;
  I(end, end)=1;  
  
  imagesc(I);
  colormap gray;
  axis equal;
  axis off;
  
  title(['Im. ', num2str(m), ', topic ', num2str(k)]);
end
suptitle(['Iter ', num2str(s)]);

% display the responsibilities for the pixel (7, 44)

ix=19;
iy=7;

idx=sub2ind([59, 79], iy, ix);
w=W(idx); % this is the visual word 


figure(17);

subplot(1, 4, 1); 
bar(Fk(idx, :));
title(['Fk for (', num2str(ix), ', ', ...
       num2str(iy), ') in image ', num2str(m)]);

load /common/marco/clustering/datasets/sim_noise_images/topic_dist.mat betas
subplot(1, 4, 2); 
bar(betas(:, w));
title('True empirical topic dist.');

subplot(1, 4, 3); 
bar(b(:, w));
title('Likelihood of being in each topic');

subplot(1, 4, 4);
bar(R(idx, :));
title(['Responsabilities for (', num2str(ix), ', ', ...
       num2str(iy), ') in image ', num2str(m)]);
suptitle(['Iter ', num2str(s)]);

pause(0.1);


function saveResponsibilities(R, iter, baseName)  %#ok<INUSL,DEFNU>

% by default we saving in this family of files.
if nargin==2, baseName='tmp_variational_'; end

% construct the filename
fileName=[baseName, num2str(iter, '%04d')];

% save the responsibilities.
save(fileName, 'R', 'iter');

% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
