function [R, g, betas]=ab_variational_single(A, W, alpha, Ni, eta, V)
%AB_VARIATIONAL
% Variational inference for segmenting a single image. This implementation
% is different from the one in 'ab_variational.m', because is uses visual
% words as global term instead of Gaussian distributions.
%
% R=ab_variational_single(A, W, alpha, Ni, eta, V)
% 
% A: affinity matrix
%
% W: list of visual words
%
% alpha: Dirichlet hyperparameter for the mixture component. This is not
% estimated during inference.
% 
% Ni: number of iterations for the variational algorithm. We could also 
%     test for convergence of the lower bound, but this is easier to code.
%
% eta: hyperparameter for the Dirichlet distribution of the topics. This is
%      also not estimated
%
% V: number of visual words in the dictionary
%
% R: cell array with all the responsibilities for each observation. 
%    They define the variational distribution q(Z) for each image.
%

% dimension of the problem/model
N=length(A); % number of observation in the image
K=length(alpha); % number of segments/topics

% reshape alpha as a row vector
alpha=reshape(alpha, 1, length(alpha));

% image to debug
USE_DEBUG=false; %#ok<NASGU>

% init the variational distributions randomly assigning the observation to
% each one of the K clusters.
R=labels2ind(unidrnd(K, N, 1)); 
Nk=col_sum(R);
g=alpha+Nk;

% variational hyperparameter for the topic distributions
betas=eta+sumTopicResp(R, W, V);
edBetas=exp(digamma(betas)-repmat(digamma(row_sum(betas)), 1, V));


% disp('Start variational inference');
for s=1:Ni
  
  % a little bit of debugging info
  displayBlendImage(R, ['Iteration ', num2str(s-1)], USE_DEBUG);

  % decide which type of iteration we are using
  switch iterType(s, Ni)
    
    % run variational alg.
    case 0, [R, betas, g, Nk]=run_variational_with_topic(A, W, R, g, Nk, alpha, ...
                                                         eta, edBetas);

    % run smooth variational alg. (without topic term)
    case 1, [R, betas, g, Nk]=run_variational_smooth(A, W, R, g, Nk, alpha, ...
                                                     eta, V);
    
    % force hard label assignment
    case 3, [R, betas, g, Nk]=run_variational_hard(R, W, eta, V);
      
    otherwise, error('Unrecognized type of iterations');
    
  end

  % q(\phi)
  edBetas=exp(digamma(betas)-repmat(digamma(row_sum(betas)), 1, V));
    
  % saveResponsibilities(R, s);
  
  % display the progres bar.
  putDot(s);
end
disp(' ');

% a little bit of debugging info
displayBlendImage(R, ['Iteration ', num2str(s)], USE_DEBUG);



% =======================================================================
% Auxiliary functions
% =======================================================================

function type=iterType(s, Ni)

% always smooth the last 10 iterations
if s>(Ni-10), type=1; return; end

if (s<50)
  if mod(s, 20)<16, type=0; return; else type=1; return; end

else
%   if mod(s, 25)==0, type=2; return; end
  if mod(s, 30)<20, type=0; return; else type=1; return; end
end



% =======================================================================
% Different types of variational step
% =======================================================================
  
function [R, betas, g, Nk]=run_variational_with_topic(A, W, R, g, Nk,  ...
                                                      alpha, eta, edBetas)
  
% initial value for this statistics
V=cols(edBetas);

% q(Z) This is the most computationaly intensive line of code
R=scale_cols(A*R, exp(digamma(g))./Nk);   

% consider the constribution of the topics
R=R.*edBetas(:, W)'; 

% normalize the responsabilities.
R=scale_rows(R, 1./row_sum(R));

% q(\theta)
Nk=col_sum(R);
g=alpha+Nk;

% collect the data for the new betas
betas=eta+sumTopicResp(R, W, V);


function [R, betas, g, Nk]=run_variational_smooth(A, W, R, g, Nk, alpha, eta, V)
  
% q(Z) This is the most computationaly intensive line of code
R=scale_cols(A*R, exp(digamma(g))./Nk);

% normalize the responsabilities.
R=scale_rows(R, 1./row_sum(R));

% q(\theta)
Nk=col_sum(R);
g=alpha+Nk;

% collect the data for the new betas
betas=eta+sumTopicResp(R, W, V);


function [R, betas, g, Nk]=run_variational_hard(R, W, eta, V)

K=cols(R);

% q(Z) 
[~, Z]=max(R, [], 2);
R=labels2ind(Z, K);

% q(\theta)
Nk=col_sum(R);
g=alpha+Nk;

% collect the data for the new betas
betas=eta+sumTopicResp(R, W, V);


% =======================================================================
% Compute likelihood and lower bound
% =======================================================================

function ll=compLogLikelihood(R, As, Ws, alpha, eta, V) %#ok<INUSD,DEFNU>

M=length(As);
Z=cell(1, M);
for m=1:M, [~, Z{m}]=max(R{m}, [], 2); end

% make the cell arrays
Xs=xrepmat({[]}, 1, M);
alpha=xrepmat({alpha}, 1, M);
lambda=xrepmat({zeros(size(alpha{1}))}, 1, M);
eta_s=xrepmat({eta}, 1, M);
Vs=xrepmat({V}, 1, M);
si=xrepmat({[]}, 1, M);

[~, ll]=ab_gibbs_sampling_list(As, Xs, alpha, 0, Z, si, lambda, Ws, Vs, eta_s);


function ll_vb=varLowerBound(R, A, W, alpha, edBetas, V)  %#ok<DEFNU>

% number of images
M=length(R);

% iterate over all the images
ll_vb=zeros(1, M);
for m=1:M
  ll_vb(m)=single_image_ll_vb(R{m}, A{m}, W{m}, alpha, edBetas, V);
end

function ll_vb=single_image_ll_vb(R, A, W, alpha, betas, g, eta) %#ok<INUSD>

% 5 terms from E_q[ log P(x, w, z, theta, phi | alpha, eta)]

% term 1:  E_q[ log p(x|z) ]
Fk=scale_cols(A*R, 1./col_sum(R)); % we approximate Fk as the average density
ll(1)=sum(sum(R.*log(Fk+eps)));

% term 2: E_q[ log p(w|z, phi) ]
ll(2)=sum(sum(R.*log(edBetas(:, W)')));  %#ok<NODEF>

% term 3: E_q[ log p( z| theta) ]
ll(3)=sum(sum(scale_rows(R, g)));

% term 4: E_q[ log p(theta | alpha)]
ll(4)=gammaln(sum(alpha))-sum(gammaln(alpha))+sum( (alpha-1) .* E_log_t(g));

% term 5: E_q[ log p(phi | eta)]
ll(5)=0;

% 3 terms from the E_q[ log q(theta, phi, z) ]

% term 6: E_q[ log q(theta) ]
ll(6)=gammaln(sum(g))-sum(gammaln(g))-sum((gamma-1).*E_log_t(g));
ll(6)=-ll(6);

% term 7: E_q[ log q(phi) ]
ll(7)=0;
ll(7)=-ll(7);

% term 8: E_q[ log q(z) ]
ll(8)=sum(sum(betas.*log(betas+eps)));
ll(8)=-ll(8);

% add together all the terms to obtain the variational lower bound
ll_vb=sum(ll);
  


% =======================================================================
% Debug subfunctions
% =======================================================================

function displayBlendImage(R, titleStr, USE_DEBUG)
%
% displayBlendImage 
%
% Given the reposabilities 'R' display an image with different segments 
% blended together accordingly to 'R'.
%

if ~USE_DEBUG, return; end

% number of segments topics
K=cols(R);

% dimension of the image
N=79;
M=rows(R)/N;

%  probability planes
P=zeros(M, N, K);
for k=1:K
  P(:,:,k)=reshape(R(:,k),M, N);
end

% obtain the blend
blend=blend_segments(P, K, [], seg_color(K));

% display and wait.
figure(13); imshow(blend);
title(titleStr);
pause(0.1);


function displayResp(R, Fk, W, b, m, s, USE_DEBUG) %#ok<DEFNU>

% skip the display if we are not debugging
if ~USE_DEBUG, return; end

K=cols(R);
L=ceil(K/3);

figure(11); clf;

for k=1:K
  subplot(L, 3, k);
  
  I=reshape(R(:, k), 59, 79);
  I(1,1)=0;
  I(end, end)=1;  
  
  imagesc(I);
  colormap gray;
  axis equal;
  axis off;
  
  title(['Im. ', num2str(m), ', topic ', num2str(k)]);
end
suptitle(['Iter ', num2str(s)]);

% display the responsibilities for the pixel (7, 44)

ix=19;
iy=7;

idx=sub2ind([59, 79], iy, ix);
w=W(idx); % this is the visual word 


figure(17);

subplot(1, 4, 1); 
bar(Fk(idx, :));
title(['Fk for (', num2str(ix), ', ', ...
       num2str(iy), ') in image ', num2str(m)]);

load /common/marco/clustering/datasets/sim_noise_images/topic_dist.mat betas
subplot(1, 4, 2); 
bar(betas(:, w));
title('True empirical topic dist.');

subplot(1, 4, 3); 
bar(b(:, w));
title('Likelihood of being in each topic');

subplot(1, 4, 4);
bar(R(idx, :));
title(['Responsabilities for (', num2str(ix), ', ', ...
       num2str(iy), ') in image ', num2str(m)]);
suptitle(['Iter ', num2str(s)]);

pause(0.1);


function saveResponsibilities(R, iter, baseName)  %#ok<INUSL,DEFNU>

% by default we saving in this family of files.
if nargin==2, baseName='tmp_variational_'; end

% construct the filename
fileName=[baseName, num2str(iter, '%04d')];

% save the responsibilities.
save(fileName, 'R', 'iter');

% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
