function [errors, mapping, class_labels]=analyze_error_MSRCv2(ct, nt)
%
% Analyze the errors for the MSRCv2 dataset and compute matching between
% topics and categories in the dataset.
%


if nargin<2, nt=2; end

% disp('Compute the labeling errors for the MSRCv1 dataset');

% Selected categories. 
% In the order below they are: cow, grass, cars, sky, foliage, sea, bikes, 
% faces. There is also another category that collect all the pixels that 
% are not marked.
idx=2:cols(ct);

% the confusion table may not have all the selected categories. We keep
% only the ones available.
idx=idx(idx<=cols(ct));

ct1=ct(:, idx); % get only the columns corresponding to the selected categories.
first_col=sum(ct, 2)-sum(ct1, 2); % whatever is left out is put in the first 
                                  % column (the clutter category).

% we add the 'clutter' category
ct=[first_col ct1];
idx=[1 idx]; 

% get the name of each categories and keep only the one we actually used.
% list of names for the 
class_labels={'nolabel', ...           
        'building', ...           
        'grass', ...           
        'tree', ...               
        'cow', ...                
        'horse', ...              
        'sheep', ...              
        'sky', ...                
        'mountain', ...           
        'aeroplane', ...           
        'water', ...              
        'face', ...               
        'car', ...                
        'bicycle', ...            
        'flower', ...                     
        'sign', ...                     
        'bird', ...                     
        'book', ...                     
        'chair', ...                     
        'road', ...                     
        'cat', ...                     
        'dog', ...                    
        'body', ...                     
        'boat', ...           
};
class_labels=class_labels(idx);

% number of topics
K=rows(ct);

% disp(['Total number of topics ', num2str(K)]);
% disp(['Up to ', num2str(nt), ' topics per class']);

% % display confusion table
% display_confusion_table(1, ct, class_labels);

% here we match topics to classes. We do this by computing a "good"
% assignemnt of disjoint subset of topics to class labels as provided by
% the ground truth.
[ct, idx_topics, idx_classes, mapping, u]=compute_mapping(ct, class_labels, nt);

% at this point we can prune the empty topics. This has also the effect of
% matching topics with cathegories in such a way that the confusion table
% has a nice diagonal.
s=col_sum(ct(u, :));
ct=cat(1, s, ct(idx_topics, :));
idx_classes=[1 idx_classes];

% % display confusion table
% display_confusion_table(11, ct, class_labels);

% Compute the detection rate and the false_allarm rate. These are the two
% measures used in Wang and Grimson, other measures are possible.
errors=cell(1+length(idx_classes), 6);

% write the header and the first coloum
errors(1, 2:end)={'Det', 'False Pos', 'Prec.', 'Recall', 'F'};
errors(:, 1)=[{'Class'}, class_labels(idx_classes)]';


% total number of pixels in each class (considering also the non-specific class) 
% and total number of pixels in the dataset.
tot_classes=sum(ct);
tot_ct=sum(tot_classes);

for h=1:rows(ct)
  
  % compute the table (see en.wikipedia.org/wiki/Precision_and_recall)
  
  tp=ct(h, idx_classes(h)); % true positives
  fp=sum(ct(h, :))-tp; % false positive
  fn=sum(ct(:, idx_classes(h)))-tp; % false negative
  tn=tot_ct-sum(ct(h, :))-sum(ct(:, idx_classes(h)))+tp; % true negative
  
  % check that the total makes sense
  if (tot_ct~=(tp+tn+fp+fn)), error('Something wrong'); end
  
  % add the data to the error table
  errors{h+1, 2}=tp/(tp+fn); % detection rate
  errors{h+1, 3}=fp/(fp+tn); % false alarm rate

  errors{h+1, 4}=tp/(tp+fp); % precision 
  errors{h+1, 5}=tp/(tp+fn); % recall
  
  errors{h+1, 6}=2/(1/errors{h+1, 4}+1/errors{h+1, 5}); % F measure
  
%   % plot the errors 
%   print_errors(class_labels, errors(h+1, :));
end
% disp(' ');
% disp(' ');

% display the precision recall with the corresponding labels near each
% marker
recall=cell2mat(errors(3:end, 5));
precision=cell2mat(errors(3:end, 4));

% plot_precision_recall('MSRCv1', recall, precision, '+r');
% h=text(recall, precision+0.01, errors(3:end, 1)); set(h, 'FontSize', 15);


function [ct, idx_topics, idx_classes, mapping, unused]=compute_mapping(ct, class_labels, nt)

% compute detection rate for each class
idx_classes=2:length(class_labels);

% flags for unused topics. All the observations whose label is not used are
% moved to the 'nolabel\ column of the confusion table.
unused=true(1, rows(ct));

% cost matrix
M=sum(sum(ct(:, 2:end)))-ct(:, 2:end)';

% compute the topic class assignment
idx_topics=assignmentoptimal(M)';

mapping=idx_topics; % this is the initial mapping.
unused(idx_topics)=false;

idx=idx_topics;
for h=1:(nt-1)
  M(:, idx)=+Inf; % remove the links to the already used topics
  idx=assignmentoptimal(M)'; % compute the new assignment
  mapping=cat(1, mapping, idx);
  
  % consolidate the ct matrix
  for l=1:length(idx)
    if idx(l)~=0
      ct(idx_topics(l), :)=ct(idx_topics(l), :)+ct(idx(l), :);
      ct(idx(l), :)=0;
    end
  end
  
  % remove all the zeros from the indeces
  idx(idx==0)=[];
  unused(idx)=false;
end


function print_errors(class_labels, errors)

disp(['Class: ', class_labels{idx_classes(h)}, ...
      ' D=',  num2str(errors{1}, '%.4f'), ...
      ' FA=', num2str(errors{2}, '%.4f'), ...
      ' P=',  num2str(errors{3}, '%.4f'), ...
      ' R=',  num2str(errors{4}, '%.4f'), ...
      ' F=',  num2str(errors{5}, '%.4f')]);

%   fprintf('%10s: P=%1.3f R=%1.3f\n', ...
%     class_labels{idx_classes(h)}, precision, recall);


function display_confusion_table(figNum, ct, class_labels) %#ok<*DEFNU>

% normalize confusion table
ctn=ct./repmat(sum(ct), rows(ct), 1);

figure(figNum); 
imagesc(ctn); colormap hot;

% display labels and other informations
set(gca, 'xtick', 1:rows(ct));
set(gca, 'xticklabel', class_labels);
ylabel('Topics'); xlabel('Classes');
set(gca, 'FontSize', 12)

% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
