function [errors, mapping]=analyze_error_sim(ct, nt)

if nargin<2, nt=1; end

disp('Compute the labeling errors for the simulation images');

% get the name of each categories and keep only the one we actually used.
class_labels={'sky', 'grass', 'cow', 'bush', 'sheep'};

% number of topics
K=rows(ct); 

disp(['Total number of topics ', num2str(K)]);
disp(['Up to ', num2str(nt), ' topics per class']);

% % display confusion table
display_confusion_table(1, ct, class_labels);

% here we match topics to classes. We do this by computing a "good"
% assignemnt of disjoint subset of topics to class labels as provided by
% the ground truth.
[ct, idx_topics, idx_classes, mapping, u]=compute_mapping(ct, class_labels, nt);

% Compute the detection rate and the false_allarm rate. These are the two
% measures used in Wang and Grimson, other measures are possible.
errors=cell(length(class_labels)+1, 6);

% write the header and the first coloum
errors(1, 2:end)={'Det', 'False Pos', 'Prec.', 'Recall', 'F'};
errors(:, 1)=[{'Class'}, class_labels(idx_classes)]';


% total number of pixels in each class (considering also the non-specific class) 
% and total number of pixels in the dataset.
tot_classes=sum(ct);
tot_ct=sum(tot_classes);

for h=1:cols(ct)
  m=ct(h, idx_classes(h));  % total number of pixels correctly recoved
  
  % compute the detection rate as the ration between the number of
  % correctly retrived pixes divided by the total number of pixel in the 
  % current class h.
  detection=m/tot_classes(idx_classes(h));
  
  % compute the false alarm as the number of time we wrongly labeled a pixels
  % divided by the total number of pixel not label as assigned the current
  % class in the background
  false_alarm=(sum(ct(h, :))-m)/(tot_ct-tot_classes(idx_classes(h)));
  
  disp(['Class: ', class_labels{idx_classes(h)}, ...
        ' D=', num2str(detection, '%.4f'), ...
        ' FA=', num2str(false_alarm, '%.4f')]);
      
   % add the data to the error table
   errors{h+1, 2}=detection;
   errors{h+1, 3}=false_alarm;
      
end
disp(' ');


% compute also the precision/recal
for h=1:cols(ct)
  % true positives
  tp=ct(h, idx_classes(h)); 
  
  % false positive
  fp=sum(ct(h, :))-tp;
  
  % false negative
  tn=sum(ct(:, idx_classes(h)))-tp;

  % true negative
  fn=tot_ct-sum(ct(h, :))-sum(ct(:, idx_classes(h)))+tp;
  
  % check that the total makes sense
  if (tot_ct~=(tp+tn+fp+fn))
    error('Something wrong');
  end
  
  % compute the precision/recall 
  precision=tp/(tp+fp);
  recall=tp/(tp+fn);
  
  % compute the F value
  F=2*(precision*recall)/(precision+recall);
  
%   disp(['Class: ', class_labels{idx_classes(h)}, ...
%         ' P=', num2str(precision, '%.4f'), ...
%         ' R=', num2str(recall, '%.4f'), ...
%         ' F=', num2str(F, '%.4f')]);

  % add the data to the error table
  errors{h+1, 4}=precision;
  errors{h+1, 5}=recall;
  errors{h+1, 6}=F;
     
end
disp(' ');

disp(' ');


function [ct, idx_topics, idx_classes, mapping, unused]=compute_mapping(ct, class_labels, nt) %#ok<*INUSD>

% compute detection rate for each class
idx_classes=1:cols(ct);

% flags for unused topics. All the observations whose label is not used are
% moved to the 'nolabel\ column of the confusion table.
unused=true(1, rows(ct));

% cost matrix
M=sum(sum(ct))-ct';

% compute the topic class assignment
idx_topics=assignmentoptimal(M)';

mapping=idx_topics; % this is the initial mapping.
unused(idx_topics)=false;

idx=idx_topics;
for h=1:(nt-1)
  M(:, idx)=+Inf; % remove the links to the already used topics
  idx=assignmentoptimal(M)'; % compute the new assignment
  mapping=cat(1, mapping, idx);
  
  % consolidate the ct matrix
  for l=1:length(idx)
    if idx(l)~=0
      ct(idx_topics(l), :)=ct(idx_topics(l), :)+ct(idx(l), :);
      ct(idx(l), :)=0;
    end
  end
  
  % remove all the zeros from the indeces
  idx(idx==0)=[];
  unused(idx)=false;

  % display confusion table
  display_confusion_table(1, ct, class_labels);
end



function display_confusion_table(figNum, ct, class_labels)

% normalize confusion table
ctn=ct./repmat(sum(ct), rows(ct), 1);

figure(figNum); 
imagesc(ctn); colormap hot;

% display labels and other informations
set(gca, 'xtick', 1:rows(ct));
set(gca, 'xticklabel', class_labels);
ylabel('Topics'); xlabel('Classes');
set(gca, 'FontSize', 12)

% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
