function colorSegmentations(xmlFile, pattern, colors, displayOnScreen, baseOutName)

%
% color the segmentation maps contained in the files describe by 'pattern'
% with the color map given by 'colors'.
% 
% If 'colors' is not specified then the function 'seg_color' is used.
%
% If 'colors' is a string then it specified a type of colormap.
%

if nargin<3, colors='standard'; end
if nargin<4, displayOnScreen=false; end

disp('Color the segmentations');

% info about the experiment
expInfo=readXMLfile(xmlFile);

if isa(colors, 'char')
  switch (lower(colors))
    case 'standard', colors=seg_color(expInfo.K);
    case 'clutter', colors=seg_color(expInfo.K, 1);
    case 'class', colors=GetClassColors;
    case 'sim', colors=[zeros(1, 3); seg_color(5)]; 
      % five are the number of classes in the sim_images dataset.
      % the first row of zeros takes care of the 'clutter' class
      % that is not present in the dataset.
      
    otherwise, error('colorSegmentations: unrecognized color map');
  end
end

% total number of different colors
nc=rows(colors);

% collect all the specified files from result directory
segFiles=getAllFilesInDir(expInfo.destDir, pattern);

% check the number of images from the variable 'pattern' vs. the
% number of images used in the experiments.
if (length(segFiles) ~= length(expInfo.data_file_info))
  error('Wrong number of segmentations!' );
end

for h=1:length(segFiles)
  
  % load the segmentation 
  load(segFiles{h}, 'segImageMAP');
  
  % generate the segmentation images
  segImageMAP=color_segments(segImageMAP, nc, [], colors);
  
  if (nargin<5)
    outFileName=[segFiles{h}(1:(end-4)), '.png'];
  else
    outFileName=sprintf('%s/%s_%04d.png', expInfo.destDir, baseOutName, h);
  end
  imwrite(segImageMAP, outFileName);

  if displayOnScreen,
    % display the segmentation images (for debugging porpuse)
    figure(1);
    subplot(1,2,1); imshow(expInfo.data_file_info{h}.original_name);
    subplot(1,2,2); imshow(outFileName);
    suptitle(num2str(h));
    pause(0.25);
  end

  putDot(h);
end
disp(' ');

disp(' ');


function class_colors=GetClassColors

class_colors= ...
  [ 0.7 0.7 0.7; ... % no label / junk
    1.0 0.0 0.0; ... % cows
    0.0 1.0 0.0; ... % grass
    0.0 0.0 0.5; ... % cars
    0.0 1.0 1.0; ... % sea
    0.5 0.8 0.3; ... % buildings
    0.1 0.5 0.1; ... % foliage
    1.0 1.0 0.0; ... % sky
    1.0 1.0 1.0; ... % airplanes
    0.8 0.0 0.8; ... % bikes
    1.0 0.5 0.0];    % faces
  
% function map=seg_color(nSeg, useGB)
% %
% % map=seg_color(nSeg, useGB) returns a color map to visualize segmentation
% % results.
% %
% % nSeg: number of segments, and number of rows of the color map. If it is
% % bigger than 11 then the colorcube function is used
% %
% % useGB: force the first segment to be black (garbage collector)
% %
% 
% % default to background cluster
% if nargin<2, useGB=0; end
% 
% 
% % construct the color map for 11 segments
% map11=[
%      255 255   0; % full yellow
%      255   0   0; % full red
%      0   255   0; % full green;
%      0     0 255; % full blue
%      255   0 255; % magenta
%      0   255 255; % cyan
%      126 100 172; % violet
%      255 110   0; % orange
%       63 128  63; % dark green
%      195 195 195; % light gray
%       87  87  87; % dark gray
%      ];
% map11=map11/255;  
% 
% map14=[
%          0         0         0;
%          0         0    0.5020;
%          0    0.5020         0;
%          0    0.5020    0.5020;
%     0.2510         0         0;
%     0.2510         0    0.5020;
%     0.2510    0.5020         0;
%     0.5020         0         0;
%     0.5020         0    0.5020;
%     0.5020    0.5020         0;
%     0.5020    0.5020    0.5020;
%     0.7529         0         0;
%     0.7529         0    0.5020;
%     0.7529    0.5020         0;
% ];
% 
% 
% if nSeg<=rows(map11)  
%   map=map11(1:nSeg, :);
% elseif nSeg<=rows(map14)
%   map=map14(1:nSeg, :);
% else
%   map=colorcube(nSeg);
% end
% 
% % set the garbage collector cluster to zero if present
% if useGB, map(1,:)=[0.7 0.7 0.7]; end
% 

% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
