/* AUTORIGHTS
Copyright (C) 2008-2012 Marco Andreetto
      
This file is part of AB_SAMPLER.

AB_SAMPLER is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#include <iostream>
#include <cmath>

#include <boost/lexical_cast.hpp>

#include "mex.h"

using namespace std;
using namespace boost;

class MatrixWrapper
{
  double *data; // data in the image
  
  // dimension of the matrix and number of elements
  long nrows, ncols, nelem;
  
  public: 
  
  MatrixWrapper(const mxArray *pm) {
    nrows=mxGetM(pm);
    ncols=mxGetN(pm);
    nelem=ncols*nrows;
    
    data=mxGetPr(pm);
  }
  
  long GetM() const { return nrows; }
  long GetN() const { return ncols; }
  
  void CheckRange(const long i,  const long j) const {
    // the range for i and j are [0, nrows-1] and [0, ncols-1]
    if ((i<0) || (i>=nrows)) 
      throw "Out of range row index: got " + lexical_cast<string>(i) 
            + " while the range is [0, " + lexical_cast<string>(nrows-1) 
            + "]";
      
    if ((j<0) || (j>=ncols))
      throw "Out of range column index: got " + lexical_cast<string>(j) 
            + " while the range is [0, " + lexical_cast<string>(ncols-1) 
            + "]";
  }
  
  
  const double &operator()(const long i, const long j) const {
    CheckRange(i, j); // if the check fails an exception in thrown.
  
    // return the image
    return data[j*nrows+i];
  }

  double &operator()(const long i, const long j) {
    CheckRange(i, j); // if the check fails an exception in thrown.

    // return the image
    return data[j*nrows+i];
  }
  
  const double &operator[](const long i) const {
    // the range for i is [0, nelem]
    if ((i<0) || (i>=nelem)) 
      throw "Out of range linear index";
            
    // return the image
    return data[i];
  }

  double &operator[](const long i) {
    // the range for i is [0, nelem]
    if ((i<0) || (i>=nelem)) 
      throw "Out of range linear index";

    // return the image
    return data[i];
  }
};


double findMaxContourOnLine(const MatrixWrapper &contour,
                            long Px, long Py, long Qx, long Qy)
{
  // compute the unit length vector for the direction  
  double nx=(Qx-Px), ny=(Qy-Py);
  double l=sqrt(nx*nx+ny*ny);
  nx/=l;
  ny/=l;
  
  // move along the line starting from P and going toward
  double maxCont=0;
  
  long Rx, Ry;
  const double step=0.5;
  
  for(double h=0; h<l; h+=step) {
    Rx=Px+static_cast<long>(nx*h);
    Ry=Py+static_cast<long>(ny*h);
    maxCont=max(maxCont, contour(Ry, Rx));
  }
  
  // check also the end value
  maxCont=max(maxCont, contour(Qy, Qx));
  
  return maxCont;
}


// aliases for the input
#define iCONT_IMAGE 0
#define iNEIGH_X    1
#define iNEIGH_Y    2

// aliases for the output
#define oCONT_RESP  0

// gataway function for mex version
void mexFunction(int n_out, mxArray *out[],
                 int n_in, const mxArray *in[])
{ 
  //
  // MEX function for computing intervening coutour. The input parameters are:
  // 
  // - An image in double format with values between 0-1.
  // 
  // - Two vector Nx and Ny, of dimension Nx2, containing the x and y 
  //   coordinates respectively of each pixels used in the computation. Each 
  //   row contains the coordinates of the end points used to compute the 
  //   intervening contour.
  //
  
  try {
    // check the input/output arguments
    if (n_in!=3) 
      throw "Two input required";
    
    if (mxGetClassID(in[iCONT_IMAGE])!=mxDOUBLE_CLASS)
      throw "Contour image must be of type double";

    if (n_out!=1)
      throw "Only output requires";
    
    // construct the image data wrapper from the mex file.
    // the indeces are in xy format rather than ij.
    MatrixWrapper contour(in[iCONT_IMAGE]);
    
    // construct the neighbor matrix
    MatrixWrapper xNeigh(in[iNEIGH_X]);
    MatrixWrapper yNeigh(in[iNEIGH_Y]);
    
    long M=xNeigh.GetM(); // number of intervening contour
    
    if (xNeigh.GetN()!=2)
      throw "The xNeigh matrix must have two columns";
    
    if (yNeigh.GetN()!=2)
      throw "The yNeigh matrix must have two columns";
    
    // allocate the output
    out[oCONT_RESP]=mxCreateDoubleMatrix(M, 1, mxREAL);
    MatrixWrapper contResp(out[oCONT_RESP]);
    
    // iterate over all the points and for each 
    long Px, Py, Qx, Qy;
    double ic_PQ, ic_QP;
    for(long m=0; m<M; m++){
      // starting pixel (index 0)
      Px=static_cast<long>(xNeigh(m, 0)-1);
      Py=static_cast<long>(yNeigh(m, 0)-1);
      
      Qx=static_cast<long>(xNeigh(m, 1)-1);
      Qy=static_cast<long>(yNeigh(m, 1)-1);
        
      // because of quantization the path between P and Q is not the same 
      // as the path between Q and P. About 99.6% of the time is the same.
      // We run in both direction and take the max of the two.
      ic_PQ=findMaxContourOnLine(contour, Px, Py, Qx, Qy);
      ic_QP=findMaxContourOnLine(contour, Qx, Qy, Px, Py);
      
      contResp[m]=max(ic_PQ, ic_QP);
    }
  }
  
  // these catches my exceptions
  catch (const char *exp_msg) { mexErrMsgTxt(exp_msg); }
  
  catch (const string &exp_msg) { mexErrMsgTxt(exp_msg.c_str()); }
  
  // this catches all the exceptions from standard libraries
  catch (exception &e) { mexErrMsgTxt(e.what()); }

  // this catches whatever remains
  catch (...) { mexErrMsgTxt("Unidentified exception"); }
}
