function [phi, theta, vwHist]=compSegmentationStat(xmlFiles, segFiles, ...
                                                   V, K, eta, alpha)
% 
%[phi, theta, vwHist]=compSegmentationStat(xmlFiles, segFiles, V, K, eta, alpha)
%
% Compute the topic distributions 'phi' and the segment mixture coefficients 
% 'theta' given the segmentation of a collection of images. 
% The "global histogram" of the visual words is also computed ('vwHist'); this 
% quantity is specific collection specific and does not depend on the 
% segmentation.
%
% xmlFiles: are the files containig all the image informations
%
% segFiles: are the files containing the segmentation
%
% V: is the number of visual 
% 
% K: is the number of topics/segments
% 
% eta: is the prior for the topic distributions.
%
% alpha: is the prior for the theta distributions. If it is a scalar then
%        it is replicated K times.
%
% phi: the K topic/segment distributions
%
% theta: the mixture coefficient for the segments in each image of the
%        collection.
%
% vwHist: histogram of the visual words in the collection
%

if (length(xmlFiles)~=length(segFiles))
  error('Mismatching in the number of files');
end

% if we are given a scalar for alpha create a vector of K entries all 
% set to alpha.
if isscalar(alpha)
  alpha=alpha*ones(1, K);
else
  
  % we are given a vector: we check for consistency with the other input
  % parameter K.
  if (K ~= length(alpha))
    error('Mismatch in the number of topic and dirichlet prior.');
  end
  
end

% number of images in the collection
M=length(segFiles);

% at the beginning all the topics are set to the prior
phi=eta*ones(K, V);
theta=repmat(alpha, M, 1);

% get the visual dictionary
[centers, mu, L, vwInfo]=getVisualWordCenters('texture', 'msrc_normalized', ...
                                               1024, xmlFiles);

% run over all the segmentations and the xml files
for m=1:length(xmlFiles)
  
  % load the visual words
  [W, ix, iy]=xmlLoadPatchFilters(xmlFiles{m}, 4, centers, mu, L);
  if (max(W)>V), disp('Unrecognized visual word'); end
  
  % load the segmentation for the image and collect the segment labels
  load(segFiles{m}, 'segImageMAP');
  
  C=segImageMAP(sub2ind(size(segImageMAP), iy, ix));
  if (max(C)>K), disp('Unrecognized segment label'); end
  
  % iterate over all the topics and compute the sample distributions
  for k=1:K
    if sum(C==k)~=0
      phi(k, :)=phi(k, :)+hist(W(C==k), 1:V);
    end
  end
  
  % compute the cardinality of each segment
  theta(m, :)=theta(m, :)+hist(C, 1:K);
  
  putDot(m);  
end
disp(' ');

vwHist=col_sum(phi);
vwHist=vwHist/sum(vwHist);

% normalize the topic distributions
phi=scale_rows(phi, 1./row_sum(phi));

% normalize the mixture coeficients
theta=scale_rows(theta, 1./row_sum(theta));

% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
