function [d, varargout]=compare_partitions(E, F, method)
%
% d = compare_partitions(E, F, method)
%
% compute the distance between to data partitions E and F according to the
% distance measure selected by method.
%
% E and F are the two partitions to compare. Every column in E or F is an 
% indicator vector for one cluster.
%
% Method can be:
% CE classification error.
% BJ Bach & Jordan distance as presented in the corresponding paper.
% VI variation of information as presented By Marina Meila
%
% for a discusion on how to compare paertition see:
% "See Comparing clustering - an axiomate view" by Marina Meila
%
%
% Lihi Zelnik-Manor, Apr 2005, Caltech
% Marco Andreetto, May 2006, Caltech
%

%first check congruency of the input
if (rows(E)~=rows(F))
  error('compare_partitions: E and F must have the same number of observation');
end;

% then we select the method for the comparison
if (nargin==2)
  method='ce';
elseif (nargin~=3)
  error('compare_partitions: either 2 or 3 inputs required');
end

switch lower(method)
  case 'ce'
    [d, m_ef, m_fe, M, C]=compare_partitions_CE(E, F);
  case 'vi'
    d=compare_partitions_VI(E, F);
  case 'bj'
    d=compare_partitions_BJ(E, F);
end

% if we are using the CE method we may have to return also the matching
% information

switch nargout
  case 3, varargout{1}=m_ef; varargout{2}=m_fe;
  case 4, varargout{1}=m_ef; varargout{2}=m_fe; varargout{3}=M; 
  case 5, varargout{1}=m_ef; varargout{2}=m_fe; varargout{3}=M; varargout{4}=C;
end


%%
function [d, m_ef, m_fe, M, C]= compare_partitions_CE(E,F)

% first we have to solve the matching problem
[N, R] = size(E);
S=size(F, 2);

% set the cost matrix
C=zeros(R, S);
for r=1:R
  for s=1:S
    C(r,s)=sum(E(:,r) & F(:,s));
  end
end

% set the constraint of the linear problem
A=zeros(R+S, R*S);

l=1;
T=zeros(R,S);
for r=1:R
  T(r,:)=ones(1, S);  A(l,:)=T(:);  T(r,:)=zeros(1, S);
  l=l+1;
end
for s=1:S
  T(:,s)=ones(R, 1);  A(l,:)=T(:);  T(:,s)=zeros(R, 1);
  l=l+1;
end

% this is the matching 
options = optimset('LargeScale', 'off', 'Simplex', 'on', 'Display', 'off');
X=linprog(-C(:), A, ones(R+S, 1), [], [], zeros(R*S, 1), ones(R*S, 1), [], options);
M=reshape(X, R, S);

% cost of this matching
d=1-sum(sum(M.*C))/N;

% from E to F matching
m_ef=zeros(R, 1);
for r=1:R
  if (~isempty(find(M(r,:)==1, 1)))
    m_ef(r)=(find(M(r,:)==1, 1));
  end
end

% from F to E matching
m_fe=zeros(S, 1);
for s=1:S
  if (~isempty(find(M(:,s)==1, 1)))
    m_fe(s)=(find(M(:,s)==1, 1));
  end
end;



%%
function [d] = compare_partitions_VI(E, F)

N = rows(E); % number of observations

% entropy of E
Nk_E = sum(E);
Pk_E = Nk_E/N;

ind=find(Pk_E>0);
logPk_E=Pk_E;
logPk_E(ind)=log2(Pk_E(ind));
H_E = -sum(Pk_E.*logPk_E);

% entropy of F
Nk_F = sum(F);
Pk_F = Nk_F/N;

ind=find(Pk_F>0);
logPk_F=Pk_F;
logPk_F(ind)=log2(Pk_F(ind));
H_F = -sum(Pk_F.*logPk_F);

% mutual information between E and F
Pkk = E'*F/N;

% avoid taking log of 0
R = Pkk./(Pk_E'*Pk_F);
ind = find(R>0);
logR = R; 
logR(ind) = log2(R(ind));
I_EF = sum(sum(Pkk.*logR));

% Variation of Information
d = H_E + H_F - 2*I_EF;



%%
function [d] = compare_partitions_BJ(E,F)

R = cols(E);
S = cols(F);

d = 0;
for r=1:R, d = d + (E(:,r)*E(:,r)')/(E(:,r)'*E(:,r)); end
for s=1:S, d = d - (F(:,s)*F(:,s)')/(F(:,s)'*F(:,s)); end
d = (1/2)*(norm(d,'fro')^2);

% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
