function A = imAffSampler(IM, R, scale)
% IMAFFSAMPLER 
%   A=IMAFFSAMPLER(IM, R, S) Compute the affinities between the pixels of 
%   an image using a radius R for the space and a scale S for the color.
%   

% Default scale is zero (adaptative scale)
if nargin < 3, scale = 0; end;

nc=size(IM, 3); % get the number of channels, this defines the dimension
                % of the feature space

% build the matrix of the squared distances. R defines the dimension of 
% the patch connected to the central pixel.

% make sure the image in in the 0-1 range. Lihi's code uses an hard-coded
% threshold for the distance that makes sense only in this range.
if max(max(max(IM)))>1, IM=IM/255; end 

D  = imdist(IM, R);

% get the (squared) distances in the feature space
[i, j, d]=find(D);  
[m, n]=size(D);

% This part is different depending on the scaling factor.
% check if we are using a fixed scale or an adaptive scale
if( scale > 0 )  
  % compute the affinities for the given scale. The scale is the sigme of
  % the gaussian. We need to square it.
  sigma=scale;
  a=exp(-d./(sigma))./sqrt(sigma.^nc);
  
  % create the sparse matrix for the affinities
  A=sparse(i, j, a, m, n);
  
else
  % compute the local scale for each observation
  [~, ~, ~, ~, LS] = scale_dist(D, 3);
  
  [i2, j2, ls]=find(LS); % (square) local scales
  ls=max(0.004, ls); % threshold operation as in lihi

  % by construction should be i=i2 and j=j2. We check this just to be sure
  if ( sum(i~=i2)>0 || sum(j~=j2)>0 )
    error('Something wrong with the local scale matrix');
  end
  
  % compute the affinities
  a=exp(-d./ls)./sqrt(ls.^nc);
  
  % create the sparse matrix for the affinities
  A=sparse(i, j, a, m, n); 

end

% other normalization
A=A./(R-1); % x normalization
A=A./(R-1); % y normalization
A=A./sqrt(2*pi^nc); % color normalization


% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
