function [D2, ind_non_zero, rows_nonz, cols_nonz] = imdist(IM, R)
%IMDIST
% [D2, ind_non_zero, rows_nonz, cols_nonz] = imdist(IM, R)
% build a sparse distance matrix D2 for image IM connecting pixels within
% the neighborhood define by R. The value in D2 are square of the 
% distances.
%
% Lihi Zelnik-Manor, March 2005, Caltech
% Marco Andreetto, March 2008, Caltech

% dimensions of the image and number of channels
[~, ~, channels] = size(IM);

% make sure the image is of class double. It is OK if the image does not
% have the range 0-1. We are interest in the distances, therefore any scale
% factor that applies to the whole image is also a scaling factor to the
% matrix distance D2 (the only catch is that there is a square operation).
IM=double(IM);

% build distance matrix for first channel
[D2, rows_nonz, cols_nonz] = mkImDist(IM(:,:,1), R);

% the matrix D2 is symmetric.
ind_non_zero = sub2ind(size(D2), rows_nonz, cols_nonz);
    
% if there is more than one channel sum all the (squared) distances
% note, that all channels have the same fill pattern in the distance
% matrix.
for c = 2:channels,
    D2 = D2+mkImDist(IM(:,:,c), R);
end


%%
function [D, ind_row, ind_col, Dff]=mkImDist(im, R)
%function D=mkDistmatrix(im, R)
% use block processing to make a distance matrix
% D is a matrix size numberofpixels x numberofpixels
% D(i,j)=(im(i)-im(j)).^2
% where i and j are pixel locations
% R defines the neighborhood size, i.e. the neighborhood has 
% size (RxR), with the pixel defining the neighborhood.

% Get the dimension of the image
[sx, sy]=size(im);

% construct a new image where each pixel contains the index
% in the linear indexing of the image.
% For example, given an image 2x2 IndImage=[1 3; 2 4];
IndImage=reshape(1:(sx*sy), sx, sy);

% zero padding. It's OK to put zeros in the 'IndImage' matrix, since those
% elements will never be used as indeces.
im=addborder(im, (R-1)/2, (R-1)/2, 0);
IndImage=addborder(IndImage, (R-1)/2, (R-1)/2, 0);

% Generate the matrix of the neighbors.
A=im2col(im,[R R],'sliding'); 
Ind=im2col(IndImage,[R R],'sliding');
% now each colunn of A is a neighborhood of im
% we want to subtract the values of the center row

% Get the center column
xA=size(A, 1); % this should alway be equal to RxR
centerPixel=(xA+1)/2;

B=ones(xA, 1);
Ac=B*A(centerPixel,:);     % equivalent to repmat(A(centerPixels, :), xA, 1)
Indc=B*Ind(centerPixel,:); % equivalent to repmat(Ind(centerPixels, :), Xa, 1)
I=find(Ind(:)>0 & Indc(:)>0); % The second condition should be redundant

ind_col = Ind(I);   clear Ind;
ind_row = Indc(I);  clear Indc;
a = A(I);        clear A;
ac = Ac(I);      clear Ac;

Dff=(a-ac).^2 + eps/10;   % avoid true zeros, put eps instead
D=sparse(ind_row, ind_col, Dff);

% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
