function [R, g, betas]=lda_variational_list(W, alpha, Ni, eta, V)
%AB_VARIATIONAL
% Variational inference for segmenting multiple images (collections)
%
% [R, g, b]=ab_variational_list(W, alpha, Ni, eta, V)
% 
% All the input, except alpha, Ni, are cell arrays. Each cell contains the 
% corresponding value for the 'ab_variational' function. For example A{1} 
% contains the affinity matrix for the first image in the collection
%
% Ni: number of iterations of the sampler for generating a new sample. If
%     it is a vector then length of NI samples are generated and the number
%     of iterations before creating sample s is NI(s).
%
% eta: hyperparameter for the Dirichlet distribution over topics
%
% V: number of visual words in the dictionary
%
% R: all the responsibilities for each pixel in each image. They define the
%    variational distribution q(Z).
%
% g: hyperparameters for the variational distribution q(\theta)
%
% b: hyperparameters for the variational distribution q(\phi)
%
%

% dimension of the problem/model
M=length(W); % number of images in the collections  
K=length(alpha); % number of segments/topics

% reshape alpha as a row vector
alpha=reshape(alpha, 1, length(alpha));

% number of iterations for the variational e-step
Nv=10;
R=cell(1, M);
g=cell(1, M);


%% Run the variational inference

% init the topic distribution
betas=rand(K, V);
betas=betas./repmat(row_sum(betas), 1, V);

% init the variational distributions
for m=1:M
  R{m}=(1/K)*ones(length(W{m}), K);
  g{m}=alpha+col_sum(R{m});
end


disp('Start variational inference');
for s=1:Ni
  
  % run the variational inference
  for h=1:Nv
    
    new_betas=zeros(K, V);
    for m=1:M
      % q(Z)
      R{m}=betas(:, W{m})'.*repmat(exp(digamma(g{m})), length(W{m}), 1);
      R{m}=R{m}./repmat(row_sum(R{m}), 1, K);
      
      % q(\theta)
      g{m}=alpha+col_sum(R{m});      
      
      % collect the data for the new betas
      new_betas=new_betas+sumTopicResp(R{m}, W{m}, V);
    end
    
    % q(\phi);
    betas=new_betas./repmat(row_sum(new_betas), 1, V);
  end
  
  putDot(s);
end

disp(' ');


% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
