"""
Create a set of HTML pages to display the segmentation results of the birds
"""

import sys, os, os.path
from jinja2 import Environment, FileSystemLoader
from glob import *



# ============================================================================
# Parameters of the result pages
# ============================================================================

# title of the web page
webPageTitle= '''Blue bird segmentation results'''

# list of images to put in the table (a reg expression identifies the images
regExpList=["blue_bird_images/image_*.png", 
            "20100913a/seg_topic_004_image_*.png",
            "20100913a/prob_topic_004_image_*.png",
            "20100913a/seg_fern_004_image_*.png",
           ]

# description for the columns of the table
descList = ["Original Image",
            "Unsupervised seg (topic4)", 
            "Posterior prob. (topic4)", 
            "Classification from ferns.", 
           ]

expDesc = "Unsupervised segmentation of the bird dataset and fern classification"
expTag="20100913a"



# ============================================================================
# Global variables fixed during the whole set of experiments. These are all 
# the variable that have to be modified to create a new webpage for the results
# ============================================================================

# base URL for the results:
baseURL = "http://www.vision.caltech.edu/marco/projects/clustering/Journal/"

# base path in the filesystem
basePath = "/common/marco/clustering/results/Journal/"

srcPath = "/home/marco/research/projects/AB_Clustering/src"

# number of images in each webpage
imagePerPage = 50



# ============================================================================
# Simple subroutines
# ============================================================================

def makeLinkString(htmlFile, text):
  """
  Given the name of the file we want to link returns the string containing
  the HTML code for the link
  """
  return "<a href=" + baseURL + htmlFile + "> " + text + "</a>"


def getFileName(htmlFile, num):

  if num == 0:
    return htmlFile
    
  # remove the extension if present
  [fileName, ext] = os.path.splitext(htmlFile)
  
  # add the number and return
  fileName += "-" + str(num) + ext 
  
  return fileName



# ============================================================================
# Main block of the script
# ============================================================================

# check the command line arguments
usageMsg="Usage: "+sys.argv[0]+" <htmlFile>"
if len(sys.argv)!=2:
  print >> sys.stderr, usageMsg
  sys.exit(1)

# check the number of descriptions matches the number of image types
if len(regExpList) != len(descList):
  print >> sys.stderr, """Error: mismatch between the number of image 
  types and the number of description"""
  sys.exit(1)


# collect the images for the webpage
lists_of_images = []
for h in range(len(regExpList)):
  # collect all the images for the different image template and sort them
  print basePath+'/'+regExpList[h]
  image_list=glob(basePath+'/'+regExpList[h])
  image_list.sort()
  
  # remove the base directory from all the entries of the list
  image_list=[x[len(basePath)+1:] for x in image_list]
  
  # append the list
  lists_of_images.append(image_list)


# check that all the image lists have the same length
nImage = len(lists_of_images[0])
if nImage == 0:
  print >> sys.stderr, "Error: no images"
  sys.exit(1)
  
for h in range(len(lists_of_images)):
  if len(lists_of_images[h]) != nImage:
    print >> stderr, "Error: wrong number of images for " + regExpList[h]
  

# repack them in a different order for the web page
table = []
for k in range(len(lists_of_images[0])):
  row = []; # an empty list for the row
  for h in range(len(lists_of_images)):
    row.append(lists_of_images[h][k])
  
  # add the row to the table
  table.append(row)
  

# define variables for the webpage
env = Environment(loader=FileSystemLoader(\
                  os.path.join(srcPath, 'html_templates')))
template = env.get_template('image_list.html')


# generate and save all the webpages (multiple pages for multiple results

done = False # flag for terminating the cycle
pos = 0; # index of the block we want to print
page =0; # page we are currently generating

# get html file names from the command line argument and the page number
currHtmlFile = getFileName(sys.argv[1], page)
nextHtmlFile = getFileName(sys.argv[1], page + 1)

prevlink = "Prev" # the last page has no "Next" link
nextlink = makeLinkString(nextHtmlFile, "Next")

while not(done):

  # extract all the entries for the current page
  subTable = table[pos:(pos+imagePerPage)]
  
  # move to the next block of images and to the next page
  pos += imagePerPage
  page += 1
  
  # check if we don't have other rows in the table
  if pos>=len(table):
    nextlink = "Next." # the last page has no "Next" link
    done = True
  
  # render the current page and save it in a file
  htmlFile=file(os.path.join(basePath, currHtmlFile), "w")
  print >> htmlFile, template.render(WebPageTitle = webPageTitle, \
                                     expDesc = expDesc, \
                                     prevlink=prevlink, \
                                     nextlink=nextlink, \
                                     descList=descList, \
                                     table=subTable)
  htmlFile.close()

  # Generate new prev, next and file names
  prevlink=makeLinkString(currHtmlFile, "Prev")
  currHtmlFile = nextHtmlFile
  nextHtmlFile = getFileName(sys.argv[1], page + 1)
  nextlink = makeLinkString(nextHtmlFile, "Next")

