/* AUTORIGHTS
Copyright (C) 2008-2012 Marco Andreetto
      
This file is part of AB_SAMPLER.

AB_SAMPLER is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#include <iostream>
#include <iterator>
#include <vector>

#include "mex.h"


#define iMASK 0 // the input mask with the connected componets
#define iK    1 // the number of connected components
#define iT    2 // the threshold for the "dust" area in the segment.

#define N_IN  3 // number of inputs


#define oMASK 0 // the output segmentation after dusting

#define N_OUT 1 // number of outputs


using namespace std;


//
// A simple structure for easy acces to 2D arrays
//

struct maskType
{
  maskType(double *iData, int iM, int iN)
  { data=iData; M=iM; N=iN; }
  
  double & operator() (int i, int j)
  { if ( (i<0) & (i>=M) ) 
      mexErrMsgTxt("Error in row index");
  
    if ( (j<0) & (j>=N) ) 
      mexErrMsgTxt("Error in column index");
  
    return ( data[i+j*M] );
  }
  
  int M; // number of rows
  int N; // number of cols
  
  double *data;
};



void mexFunction (int n_out, mxArray* out[], 
                  int n_in, const mxArray* in[])
{
  //
  // Checking the inputs
  //
  
  if (n_in != N_IN)
    mexErrMsgTxt("3 inputs required");
  
  if (n_out != N_OUT)
    mexErrMsgTxt("1 output required");
  
  
  // 
  // Performe the computation
  //
  
  // Get the dimensions of the segmentation mask, the number of segments, 
  // and the threshold value.
  int M=mxGetM(in[iMASK]), N=mxGetN(in[iMASK]), 
      K=static_cast<unsigned>(mxGetScalar(in[iK])),
      T=static_cast<unsigned>(mxGetScalar(in[iT]));
  
  // create the output mask and copy the input. From now on we use only this
  // mxArray for all the computations.
  out[oMASK]=mxDuplicateArray(in[iMASK]);
  
  
  // create an auxiliary object for easy access
  maskType mask(mxGetPr(out[oMASK]), M, N);

  // count the size of each segment (connected component)
  vector<int> segSizes(K, 0);
  
  for(int m=0; m<M; ++m)
    for(int n=0; n<N; ++n){
      --mask(m, n); // decreas by one for C/MATLAB convention
      ++segSizes[static_cast<int>(mask(m, n))];
    }
  
  // Iterate over all the segments and for the ones smaller than the threshold 
  // count the number of neighbors. Then change the segment label to the one 
  // of the more frequent neighbors.
  for(int s=0; s<K; ++s) {
    
    // if s is a large segment then we have nothing to do
    if ( (segSizes[s] > T) || 
         (segSizes[s]==0) )
      continue; 
    
    // we are dealing with a small segment. We need to count the number of 
    // neighbors of this segment.
    vector<int> neighbors(K, 0);
    
    for(int m=1; m<(M-1); ++m)
      for(int n=1; n<(N-1); ++n) {
        if ( (mask(m  ,n+1) == s) || (mask(m+1,n  ) == s) || 
             (mask(m  ,n-1) == s) || (mask(m-1,n  ) == s) )
          ++neighbors[static_cast<int>(mask(m, n))];
      }
    
    // we do not consider the current segment
    neighbors[s]=0;
    
    // get the most frequent neighbors
    int maxLabel=0, maxN=neighbors[0];
    for(int k=1; k<K; ++k)
      if (maxN<neighbors[k]) {
        maxLabel=k;
        maxN=neighbors[k];
      }
    
    // change the current label to the new one
    for(int m=0; m<M; ++m)
      for(int n=0; n<N; ++n) {
        if (mask(m, n) == s)
          mask(m, n)=maxLabel;
      }
    
    // we are done for this segment
  }
  
  // before returning we have to increase by one for the Matlab/C convention
  for(int m=0; m<M; ++m)
    for(int n=0; n<N; ++n)
      ++mask(m, n);
  
  // now we are done!!!
}
