/* AUTORIGHTS
Copyright (C) 2008-2012 Marco Andreetto
      
This file is part of AB_SAMPLER.

AB_SAMPLER is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#include <iostream>
#include <fstream>
#include <iterator>
#include <cmath>
#include <list>

#include <boost/foreach.hpp>

#include "ublas_utils.h"

#include "my_distributions.h"
#include "mex.h"

using namespace std;
using namespace boost::numeric;

// Microsoft Visual Studio 8 (2005) does not find the macro M_PI.
// The following ifdef fix the problem
#ifndef M_PI
#define M_PI       3.14159265358979323846
#endif


/****************************************************************************
 * GaussianDist member functions
 *
 ****************************************************************************/

void GaussianDist::SetInvWishart(const ublas_vector &inMu, double inPrec, 
                                unsigned inNu, const ublas_matrix &inS)
{
  mu_prior=inMu;
  precision_prior=inPrec;
  nu_prior=inNu;
  S_prior=inS;
  
  // check the validity of the Normal-Wishart prior hyperparameters
  if (precision_prior<=0)
    throw "Non-positive precision for Normal-Wishart prior";

  if (nu_prior<=0)
    throw "Invalid number of degree of freedom for Wishart prior";
  
  if (ublas_utils::det(S_prior)<=0)
    throw "Invalid scale parameter for Wishart prior";
  
  // signal that the hyperparameters are ready
  hyper_parameters_ready=true;
}


void GaussianDist::Init(const vector<ublas_vector> &data, 
                        const vector<bool> &mask)
{
  Dim=data[0].size(); // number of dimensions of the space of observations
  normFactor=1/(pow(2.0*M_PI, 0.5*Dim)); // fixed normalization function
  
  // compute the two sums
  Nd=0;
  SX=ublas_zero_vector(Dim);
  SX2=ublas_zero_matrix(Dim, Dim);

  for(unsigned i=0; i<mask.size(); ++i) {
    if (mask[i]){
      ++Nd; 
      noalias(SX)+=data[i]; 
      noalias(SX2)+=outer_prod(data[i], data[i]);
    }
  }
  
  UpdateParam();
}


void GaussianDist::Init(const vector<ublas_vector> &data)
{
  // select all the observations
  vector<bool> mask(data.size(), true);
  
  // call the other version
  Init(data, mask);
}


void GaussianDist::UpdateParam()
{
  // Updating the parameters is essentially computing the MAP of the 
  // posterior distribution given the data. This function assume the 
  // sufficient statistics for all the data are given.

  // Check that we have all the data
  if (!hyper_parameters_ready)
    throw "No hyperparameter ready for the update";

  // Update the distribution parameters
  if (Nd==0) {
    // no points are assigned to this cluster. Return the prior
    mean=mu_prior;
    cov=S_prior/(nu_prior+Dim+1.0);
  }
  else {
    // sample mean and sample covariance
    ublas_vector ybar=SX/Nd; 
    ublas_matrix Sbar=SX2/Nd-outer_prod(ybar, ybar);

    // Set the mean paramter. This is the convex combination of the sample 
    // means and the prior mean parameter.
    mean=(mu_prior*precision_prior+ybar*Nd)/(precision_prior+Nd);

    // compute the sample mean prior mean difference
    ublas_vector d=ybar-(mu_prior);

    // Set the covariance.
    cov=(S_prior+Nd*Sbar+
         (precision_prior*Nd/(precision_prior+Nd))*outer_prod(d, d))
         /(nu_prior+Nd+Dim+1.0);
  }

  // compute the remaining quantities.
  ComputeAuxElements();
}


void GaussianDist::UpdateParam(const vector<ublas_vector> &data, 
                               const vector<bool> &valid)
{
  Init(data, valid);
}
  

void GaussianDist::ComputeAuxElements()
{
  // Given mean and covariance, compute the auxiliary quantities used for
  // the evaluation of the Gaussian density function.

  // compute the determinant and the matrix using the same function
  double determinant;
  boost::numeric::ublas_utils::inv_det(cov, invCov, determinant);

  // check that the determinant is strictly positive (actually smaller than 1E-16)
  if (determinant<=1E-16)
    throw "Determinant of the covariance matrix too small or zero";

  sqrtInvDet=sqrt(1/determinant); 
}



/****************************************************************************
 * StudentDist member functions
 *
 ****************************************************************************/

void StudentDist::SetInvWishart(const ublas_vector &inMu, double inPrec, 
                                unsigned inNu, const ublas_matrix &inS)
{
  mu_prior=inMu;
  precision_prior=inPrec;
  nu_prior=inNu;
  S_prior=inS;
  
  // check the validity of the Normal-Wishart prior hyperparameters
  if (precision_prior<=0)
    throw "Non-positive precision for Normal-Wishart prior";

  if (nu_prior<=0)
    throw "Invalid number of degree of freedom for Wishart prior";
  
  if (ublas_utils::det(S_prior)<=0)
    throw "Invalid scale parameter for Wishart prior";
}


void StudentDist::Init(const vector<ublas_vector> &data, 
                       const vector<bool> &mask)
{
  // Initialize the distribution parameter using the valid observations in
  // Data. The parameters are computed using the formulas for the Gaussian.
  // Once the position and scale parameters are computed the flag 
  // 'paramAvailable' is set and the update method is called.
  
  // number of dimensions of the space of observations. The shape parameter 
  // is already set.
  Dim=data[0].size();
  
  // Once the dimension D and the shape parameter ni are known this is fixed.
  //
  //    Gamma((ni+D)/2)
  // -------------------------
  // Gamma(ni/2) (pi*ni)^(D/2)
  //
  normFactor=exp(lgamma((ni+Dim)/2.0)-(lgamma(ni/2.0)+(Dim/2.0)*log(M_PI*ni)));
  
  // Compute the initail position and scale parameter as sample mean and 
  // covariance. Then call the update function to estimate them again.
  unsigned Nd=0;
  ublas_vector accVector=ublas_zero_vector(Dim);
  ublas_matrix accMatrix=ublas_zero_matrix(Dim, Dim);

  for(unsigned i=0; i<mask.size(); ++i) {
    if (mask[i]){
      ++Nd; 
      noalias(accVector)+=data[i]; 
      noalias(accMatrix)+=outer_prod(data[i], data[i]);
    }
  }
  
  // set the position and scale parameter.
  if (Nd==0) {
    mu=mu_prior;
    sigma=S_prior;
  }
  else {
    mu=accVector/Nd;
    sigma=accMatrix/Nd;
  }
  
  // We have to compute also the auxiliary elements of the function
  ComputeAuxElements();
  
  // Now the parameters are availables.
  paramAvailable=true;
  
  // estimate again the parameters.
  //UpdateParam(data, mask);
}


void StudentDist::UpdateParam(const vector<ublas_vector> &data, 
                              const vector<bool> &valid)
{
  // Update the parameters of the distribution using the maximum likelihood
  // estimators and the input data. To compute the weights of the input data
  // the current position and scale parameters are used.

  // Check that we have already the parameters ready.
  if (!paramAvailable)
    throw "Parameters not available for updating";
  
  // check that the two input vectors have the same size.
  if (valid.size()!=data.size())
    throw "Mismatching dimension for data and valid vectors";
  
  // Compute the weights and the number of elements. To compute the weights
  // we use the current value of mu and Sigma (they are used by the 
  // member function mDist).
  unsigned Nd=0;
  vector<double> weight(data.size());
  for(unsigned h=0; h<data.size(); ++h)
    if (valid[h]) {
      ++Nd;
      weight[h]=(ni+Dim)/(ni+mDist(data[h]));
    }
  
  // Compute the new position parameter mu
  double totWeight=0;
  mu=ublas_zero_vector(Dim);
  for(unsigned h=0; h<data.size(); ++h)
    if (valid[h]) {
      noalias(mu)+=data[h]*weight[h];
      totWeight+=weight[h];
    }
  mu/=totWeight; // new position parameter mu;
  
  // we may not have any valid element 
  if  (Nd==0) {
    cerr<<"Warning: empty cluster"<<endl;
    sigma=S_prior;
    ComputeAuxElements();
    
    return;
  }
  
  //Compute the new scale parameter sigma. Use the current mu.
  sigma=ublas_zero_matrix(Dim, Dim);
  for(unsigned h=0; h<data.size(); h++) 
    if (valid[h])
      noalias(sigma)+=weight[h]*outer_prod(data[h]-mu, data[h]-mu);
  sigma/=Nd;
  
  // We need to update the other elements of the class
  ComputeAuxElements();
}


void StudentDist::ComputeAuxElements()
{
  // Given position and scale parameter, compute the auxiliary quantities 
  // used for the evaluation of the gaussian density function: the inverse 
  // of the scale matrix and its determinant.

  // compute the determinant and the matrix using the same function
  double determinant;
  boost::numeric::ublas_utils::inv_det(sigma, invSigma, determinant);

  // check that the determinant is strictly positive
  if (determinant<=1E-16) 
    throw "Determinant of the covariance matrix too small";

  // if we are here then the inverse matrix is well defined.
  sqrtInvDet=std::sqrt(1/determinant);
}

/***************************************************************************
 * Code for the random permutation
 ***************************************************************************/

// declarations for the random permutation function
typedef std::pair<unsigned, unsigned> viType;


struct genRndVI {
  unsigned n;
  
  // init the seed 
  genRndVI() : n(0) { 
    srand( static_cast<unsigned> ( std::time(0)+std::clock()) );
  } 
  
  // generate and return a random number
  viType operator() () { viType p(rand(), ++n); return p; }
};


struct copySecondInArray : public std::unary_function<viType, void> {
  std::vector<unsigned> *pv;
  
  copySecondInArray(std::vector<unsigned> &v) : pv(&v) {}
  void operator() (const viType &a) { pv->push_back(a.second); }
};


void GenRandomPerm(unsigned N, vector<unsigned> &perm) {
  
  // construct the list to sort
  list<viType> viList;
  generate(viList.begin(), viList.end(), genRndVI());
  
  // sort the list using the first element of the pair
  viList.sort();

  // copy the indeces in the permutation vector
  perm.reserve(N);
  BOOST_FOREACH( viType p, viList) {
    perm.push_back(p.second);
  }
};
