/* AUTORIGHTS
Copyright (C) 2008-2012 Marco Andreetto
      
This file is part of AB_SAMPLER.

AB_SAMPLER is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#ifndef _MY_DISTRIBUTIONS_H_
#define _MY_DISTRIBUTIONS_H_

#include <iostream>
#include <vector>
#include <cmath>
#include <algorithm>
#include <ctime>

#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/io.hpp>

#include "ublas_utils.h"

#include "mex.h"

//
//define the types from ublas
//
typedef boost::numeric::ublas::matrix<double> ublas_matrix;
typedef boost::numeric::ublas::vector<double> ublas_vector;

typedef boost::numeric::ublas::zero_vector<double> ublas_zero_vector;
typedef boost::numeric::ublas::zero_matrix<double> ublas_zero_matrix;

typedef boost::numeric::ublas::identity_matrix<double> ublas_identity_matrix;



/***************************************************************************
 * GuassianDist
 *
 * Definition of the class for a normal distribution with prior 
 * distribution over the parameters given by a Normal-InvWishart
 *
 ***************************************************************************/
class GaussianDist
{
  // hyperparameters of the Normal-InvWishart prior. These will be used when 
  // computing the distribution parameters from the sufficient statistics.
  ublas_vector mu_prior;  // mean of the gaussian prior 
  double precision_prior; // precision for the gaussian prior 
  unsigned nu_prior;      // Degree of freedom of the Inv-Wishart
  ublas_matrix S_prior;   // Scale matrix of the Inv-Wishart

  bool hyper_parameters_ready; // used to check that the object is ready

  unsigned Dim; // number of dimension of the distribution
  
  // Sufficient statistics and distribution parameters
  ublas_vector SX;  // sum of a the observations (vector)
  ublas_matrix SX2; // sum of a the squared observations (matrix)
  unsigned Nd;      // number of observations used to compute the parameters
  
  // mean and covariance matrix defining the Gaussian distribution
  ublas_vector mean;
  ublas_matrix cov;
  
  // internal variable to speed-up the distribution evaluation.
  ublas_matrix invCov;
  double normFactor; // 1/((2*pi)^(D/2))
  double sqrtInvDet; // 1/(det(Cov)^(1/2))
    
  public:
  
  // public constructors
  GaussianDist() : hyper_parameters_ready(false), Dim(0) {}
  
  // Set the Gaussian Inv-Wishart prior on the position and scale
  void SetInvWishart(const ublas_vector &, double, unsigned, const ublas_matrix &);

  // Initialize the distribution parameter using a set of observations
  void Init(const std::vector<ublas_vector> &, const std::vector<bool> &);
  void Init(const std::vector<ublas_vector> &);
  
  // Accessing members for the statistics. These are usefull when we share
  // information among multiple images.
  ublas_vector & SumX() { return SX; }
  const ublas_vector & SumX() const { return SX; }
  
  ublas_matrix & SumX2() { return SX2; }
  const ublas_matrix & SumX2() const { return SX2; }
  
  unsigned & NumObs() { return Nd; }
  const unsigned NumObs() const { return Nd; }
  
  // Update the distribution paramters based on the statistics and on the 
  // parameters priors.
  void UpdateParam();
  void UpdateParam(const std::vector<ublas_vector> &, const std::vector<bool> &);
  
  // Compute other auxiliary quantities like the inverse covariance and the 
  // determinant of the covariance.
  void ComputeAuxElements();

  // The following two are the most critical functions for the speed of the 
  // sampler when the parametric term is used. We should try to optimize them.
  
  // Compute the Mahalanobis distance for the input X
  double mDist(const ublas_vector &X) const {
    return inner_prod(X-mean, prod(invCov, X-mean));
  }
  
  // Evaluate the Gaussian density in X.
  double Evaluate(const ublas_vector &X) const {
    return (std::exp(-0.5*mDist(X))*normFactor*sqrtInvDet);
  }
};



/***************************************************************************
 * StudentDist
 *
 * Definition of the class for a Student T distribution with prior 
 * distribution over the parameters given by a Normal-InvWishart
 * This distributions is an alternative to the usual multivariate normal
 * distribution.
 *
 ***************************************************************************/

class StudentDist
{
  // Hyperparameters of the Normal-InvWishart prior. These will be used when 
  // computing the distribution parameters from the sufficient statistics.
  ublas_vector mu_prior;  // mean of the gaussian prior 
  double precision_prior; // precision for the gaussian prior 
  unsigned nu_prior;      // Degree of freedom of the Inv-Wishart
  ublas_matrix S_prior;   // Matrix of the Inv-Wishart

  // Parameters of the multivariate student T distributions
  unsigned Dim; // number of dimension of the distribution
  static const double ni=5; // shape parameter/number of degree of freedom. 
  
  ublas_vector mu;    // position parameter and scale matrix parameter.
  ublas_matrix sigma; // we do not call this mean and covariance!
  
  bool paramAvailable; // this flag indicates if the position and scale 
                       // parameters are available.

  // Internal variable to speed-up the distribution evaluation at a certain 
  // point (precomputed and preallocated).
  ublas_matrix invSigma;
  double normFactor, sqrtInvDet;
  

  public:
  
  // default concor, set the dimension to zero and the flag for the 
  // parameter to false.
  StudentDist() : Dim(0), paramAvailable(false){}
  
  // Set the Gaussian Inv-Wishart prior
  void SetInvWishart(const ublas_vector &, double, 
                     unsigned, const ublas_matrix &);

  void Init(const std::vector<ublas_vector> &, 
            const std::vector<bool> &);
  
  void UpdateParam(const std::vector<ublas_vector> &, 
                   const std::vector<bool> &);
  
  // Compute other auxiliary quantities like the inverse 
  // covariance and its determinant.
  void ComputeAuxElements();
  
  // Compute the Mahalanobis distance for the input X
  double mDist(const ublas_vector &X) const {
    return inner_prod(X-mu, prod(invSigma, X-mu));
  }
  
  // Evaluate the Gaussian density in X.
  double Evaluate(const ublas_vector &X) const {
    return normFactor*sqrtInvDet*pow((1+mDist(X)/ni), -0.5*(ni+Dim));
  }
};


/*****************************************************************************
 * Other auxiliary functions
 *
 ****************************************************************************/

void GenRandomPerm(unsigned N, std::vector<unsigned> &);


// Declaration of the isinf and isnan for the MSVC++
#if defined(WIN32) || defined(_WIN32) || defined(__WIN32__) 
  inline int isnan(double x) { return _isnan(x); }
  inline int isinf(double x) { return !_finite(x); }

#endif // WIN32

#endif // _MY_DISTRIBUTIONS_H_
