function pruning_small_topics(expTag, minP)
% 
% remove all the small segment topics present in an image and assign labels
% only when the corresponding probability is larger than 'minP'. The
% function generate a new "virtual" experiment with tag:
%
% 'newExpTag= [expTag, '_pruned'];
%
% The new experiment has an additional topic for the background altough
% this is also used for the matching of classes and topics.
%
% By defualt minP is set to zero, so that no pruning based on posterior
% probability is done.
%

% set minP to zero
if nargin<2, minP=0; end

% number of topic to keep in each image
T=4;

% read the xmlFile describing the experiment.
expInfo=readXMLfile(['experiment_info_', expTag, '.xml']);
newExpInfo=expInfo;

% change the new xml data structure with the new number of topics and the
% use of a background topic
newExpInfo.K=expInfo.K+1;
newExpInfo.use_bg=true;

% create a new directory for the "virtual" experiment
newExpInfo.destDir=fullfile(expInfo.destDir, 'pruned');
mkdir(newExpInfo.destDir);

% load the samples from the experiment
load([expInfo.destDir, '/seg_results.mat'], 'Cs_all');

% iterate over all the images and change the samples in such a way that the
% resulting segmentation has background for all the samples
newCs_all=cell(1, length(Cs_all)); %#ok<NODEF>

for h=1:length(Cs_all)
  newCs_all{h}=Cs_all{h}+1; % the new samples are shifted by one
  
  % compute the MAP and probabilities matrices from the samples of image h
  [Cmap, P]=extractMAP(Cs_all{h}, expInfo.K);
  
  % compute the histogram of the topics in image h
  ch=hist(Cmap, 1:expInfo.K);
  [~, sc]=sort(ch);
  
  % remove small topics
  idx=false(size(Cmap));
  for k=1:(expInfo.K-T)
    idx=idx | (Cmap==sc(k)); % indeces of the smaller label
  end
  newCs_all{h}(idx, :)=1; % the label 1 corresponds to the background
  
  % iterate over all the topics and set to zero the labels of 
  % all the observations
  idx=false(size(Cmap));
  for k=1:expInfo.K
    % collect all the observation with label k and posterior probability
    idx = idx | ((Cmap==k) & (P(:, k)<minP));
  end
  newCs_all{h}(idx, :)=1; 
  
  putDot(h);
end
disp(' ');

% save the new samples and the experiment info into a new file
Cs_all=newCs_all;
save(fullfile(newExpInfo.destDir, 'seg_results.mat'), 'Cs_all');

xmlFileName=fullfile(newExpInfo.destDir, 'experiment_info.xml');
xml_save(xmlFileName, newExpInfo);
copyfile(xmlFileName, ...
  fullfile('/common/marco/clustering/results/Journal/', ...
           ['experiment_info_', expTag, '_pruned.xml']));

% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
