/* AUTORIGHTS
Copyright (C) 2008-2012 Marco Andreetto
      
This file is part of AB_SAMPLER.

AB_SAMPLER is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#include <iostream>

#include <vector>
#include <string>
#include <sstream>

#include <boost/numeric/ublas/io.hpp>

#include "ublas_utils.h"

#include "my_parallel_library.h"

using namespace std;
using boost::asio::ip::tcp; // we use a tcp socket


// 
// Macros for the command line arguments
// 

#define COMMAND 0
#define MASTER  1
#define ID      2
#define NARG    3

// Usage message
const char* usageMessage="Usage: <command> <master> <ID>";



// 
// TCP port number for all the communications. This is fixed to 5173 because 
// it is the only port currently not used by any other service and not under
// firewall control. The constant is a string since we were passing a string 
// as command argument.
//

const char *PORT="5173";

// Dimension of the matrices and vectors exchanged
const unsigned F=3;


// global variable for the sequence number of the master
unsigned seqNumber=0;
unsigned S=10;


// define the types from ublas
typedef boost::numeric::ublas::matrix<double> ublas_matrix;
typedef boost::numeric::ublas::vector<double> ublas_vector;

typedef boost::numeric::ublas::zero_vector<double> ublas_zero_vector;
typedef boost::numeric::ublas::zero_matrix<double> ublas_zero_matrix;

typedef boost::numeric::ublas::identity_matrix<double> ublas_identity_matrix;


// main function 
int main(int argc, char **argv)
{
  try {
    if (argc!=NARG) {
      cerr<<usageMessage<<endl;
      exit(-1); 
    }
  
    // main object for the io_service
    boost::asio::io_service io_service;

    // resolve the connection to the server/port
    tcp::resolver resolver(io_service);
    tcp::resolver::query query(tcp::v4(), argv[MASTER], PORT);
  
    // connect to the master using 'socket'
    tcp::socket socket(io_service);
    socket.connect(*resolver.resolve(query));
  
    // set the matrices SX_local, SX_all, SX_local and SX2_all;
    ublas_vector SX_local(F), SX_all(F);
    for(unsigned h=0; h<F; ++h) SX_local[h]=h+1;
    
    ublas_matrix SX2_local(F, F), SX2_all(F, F);
    SX2_local=atof(argv[ID])*ublas_identity_matrix(F);
    
    boost::asio::write(socket, 
      boost::asio::buffer(reinterpret_cast<const char *>(&F), sizeof(unsigned)));
    
    unsigned Nd_local=0, Nd_all=0;
    messageType msg;
    
    for(unsigned s=0; s<S; ++s){
      msg.seqNumber=seqNumber;
      
      // Send all the local values to the Master for combining them.
      CompactStats(SX_local, SX2_local, Nd_local, msg);
      boost::asio::write(socket, 
          boost::asio::buffer(reinterpret_cast<const char *>(&msg), MSG_SIZE));  

      // Receive all the information from the master about the global terms
      boost::asio::read(socket, 
         boost::asio::buffer(reinterpret_cast<char *>(&msg), MSG_SIZE));
      ExtractStats(msg, SX_all, SX2_all, Nd_all);
      
      // copy the statistics for all the data into the local ones
      Nd_local=Nd_all;
      SX_local=SX_all;
      SX2_local=SX2_all;

      // check that the message we got back has the correct sequence number
      if (msg.seqNumber!=seqNumber) {
        cerr<<msg.seqNumber<<" "<<seqNumber<<endl;
        cerr<<"Error communication out of sequence"<<endl;
        exit(-1);
      }
 
      // print the matrices on the screen
      cerr<<"========================================================="<<endl;
      cerr<<"Data received from the master"<<endl;
      cerr<<Nd_all<<endl;
      cerr<<SX_all<<endl;
      cerr<<SX2_all<<endl;
      cerr<<endl;

      // increase the seqNumber
      ++seqNumber;

    }
    
    cerr<<endl<<endl<<"Completed all the iterations"<<endl;
  }

  catch (exception &e) { 
    std::cerr<<"Catch all: "<<e.what()<<std::endl;
  }
  
  return 0;
}
