function ferns=trainClassifierImages(expInfo)

% extract the most confident pixels and train a fern classifier on them.
% The resulting classifier is then applied to all the images to obtain a
% new classification

segFiles=getAllFilesInDir(expInfo.destDir, 'seg_map_*.mat');
K=expInfo.K;

% parameter of the fern classifier and of the training set
pT=0.95; % probability to accept a pixel in a given category
fernPrm=struct('S', 7, 'M', 50, 'thrr', [0 1], 'bayes', 1); 

% iterate over all the images and collect all the patches strongly assigned
% to the topic 'k'
allFeatures=[];
allLabels=[];

trainIdx=randperm(length(segFiles));
for h=1:2:length(trainIdx)
  l=trainIdx(h);
  
  % load the segmentation and the assignment prob
  load(segFiles{l}, 'segImageMAP', 'probImage');
  
  % load the image
  I=im2double(imread(expInfo.data_file_info{l}.original_name));
  
  % collect the patches and their topic assignment
  [features, label]=selectFeatures(I, segImageMAP, probImage, pT, 'plain');
  allFeatures=cat(1, allFeatures, features);
  allLabels=cat(1, allLabels, label);
  
  putDot(h);
end
disp(' ');

% train the classifier
[ferns, allLabelsPr]=fernsClfTrain(allFeatures, allLabels, fernPrm);

% remove seemingly redudant fiend in the fern structure
ferns.inds=0;


function [features, labels]=selectFeatures(I, segImage, probImage, pT, type)

switch lower(type)
  
  case 'plain' 
    [features, labels]=selectFeaturesPlain(I, segImage, probImage, pT);
    
  case 'hist'
    [features, labels]=selectFeaturesHist(I, segImage, probImage, pT);
    
  otherwise
    error('Unrecognized feature type');
end


function [features, labels]=selectFeaturesPlain(I, segImage, probImage, pT)

% generate all the possible patches of size (L, L)
[R, L]=getPatchDim;

% create the patches and collect their corresponding labels.
features=[];
for c=1:size(I, 3)
  p=im2col(addborder(I(:,:,c), R, R, -1), [L L], 'sliding');
  
  % select only half of the points, but keep all the pixels
  features=cat(1, features, p);
end
labels=segImage(:);

% reshape the 3D array to obtain a topic plane in each column
probImage=reshape(probImage, rows(I)*cols(I), []);
unusedPatchesIdx=all(probImage<pT, 2);

features(:, unusedPatchesIdx)=[];
labels(unusedPatchesIdx)=[];

% we should remove also patches near the border
borderPatchesIdx=any(features==-1, 1);
features(:, borderPatchesIdx)=[];
labels(borderPatchesIdx)=[];

% remove 70% of them
idxRand=rand(1, cols(features))<0.7;
features(:, idxRand)=[];
labels(idxRand)=[];

features=features';


function [features, labels]=selectFeaturesHist(I, segImage, probImage, pT)

% generate all the possible patches of size (L, L)
[R, L]=getPatchDim;

% create the patches and collect their corresponding labels.
features=[];
for c=1:size(I, 3)
  p=im2col(addborder(I(:,:,c), R, R, -1), [L L], 'sliding');
  
  % select only half of the points, but keep all the pixels
  features=cat(1, features, p);
end
labels=segImage(:);

% reshape the 3D array to obtain a topic plane in each column
probImage=reshape(probImage, rows(I)*cols(I), []);
unusedPatchesIdx=all(probImage<pT, 2);

features(:, unusedPatchesIdx)=[];
labels(unusedPatchesIdx)=[];

% we should remove also patches near the border
borderPatchesIdx=any(features==-1, 1);
features(:, borderPatchesIdx)=[];
labels(borderPatchesIdx)=[];

% remove half of them
idxRand=rand(1, cols(features))<0;
features(:, idxRand)=[];
labels(idxRand)=[];

features=features';


function [R, L]=getPatchDim

R=6;
L=2*R+1;


% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
