/* AUTORIGHTS
Copyright (C) 2008-2012 Marco Andreetto
      
This file is part of AB_SAMPLER.

AB_SAMPLER is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#ifndef _UBLAS_UTILS_
#define _UBLAS_UTILS_

#include <exception>

#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/vector_proxy.hpp>
#include <boost/numeric/ublas/matrix.hpp>

#include <boost/numeric/ublas/triangular.hpp>
#include <boost/numeric/ublas/lu.hpp>

#include <cmath>
#include <cfloat>
#include "mex.h"


// namespace for additional function for the ublas library
namespace boost { namespace numeric { namespace ublas_utils {


// define some exceptions for the 
class matrix_not_square: public std::exception
{
  virtual const char* what() const throw()
  {
    return "ublas_utils::exception: determinant/inversion of rectangular matrix";
  }
};


class singular_matrix: public std::exception
{
  virtual const char* what() const throw()
  {
    return "Error: inversion of singular matrix";
  }
};




// Interface functions between MATLAB mxArray and uBLAS matrices and vector

inline 
const ublas::matrix<double> 
ConvertMxArray(const mxArray *inMatrix){
  // Copy the content of the mxArray into a "ublas" matrix object
  
  double *d=mxGetPr(inMatrix); 
  
  int M=mxGetM(inMatrix); //  number of rows of the matrix
  int N=mxGetN(inMatrix); //  number of columns of the matrix

  // create the ublas matrix
  ublas::matrix<double> outMatrix(M, N);

  for(int j=0; j<N; ++j)
    for(int i=0; i<M; ++i, ++d)
#ifdef _MSC_VER
      if (_finite(*d))
        outMatrix(i, j)=(*d);
      else
        exit(-1);
#else
      if (std::isfinite(*d))
        outMatrix(i, j)=(*d);
      else
        exit(-1);
#endif  
  
  return outMatrix;
}


inline
mxArray * ConvertUBlas(const ublas::matrix<double> &inMatrix)
{
  // Copy the content of the "ublas" matrix object into an mxArray

  int M=inMatrix.size1();    //  number of rows of the matrix
  int N=inMatrix.size2();    //  number of columns of the matrix

  // allocate the mxArray
  mxArray *outMatrix=mxCreateDoubleMatrix(M, N, mxREAL);
  double *d=mxGetPr(outMatrix);

  for(int j=0; j<N; ++j)
    for(int i=0; i<M; ++i, ++d)
      (*d)=inMatrix(i, j);
  
  return outMatrix;
}


// some simple matrix operations

inline int inv(const ublas::matrix<double> &A, ublas::matrix<double> &iA)
{
  // check if the matrix is square
  if (A.size1() != A.size2()) throw matrix_not_square();
  
  // create a permutation matrix (type defined in lu.hpp)
  ublas::permutation_matrix<std::size_t> pivots(A.size1());

  // create a working copy to compute the LU factorization
  ublas::matrix<double> lu(A);
  
  // perform LU-factorization. The matrix 'lu' has the L in the lower
  // triangular part and the U in the upper triangular and the diagonal.
  // 'pivots' represent the permutation matrix.
  int c_rank = ublas::lu_factorize(lu, pivots);  
  if( c_rank != 0 ) return c_rank;
  
  // create the inverse matrix and compute it using the LU substitution
  iA.assign(ublas::identity_matrix<double>(lu.size1()));
  lu_substitute(lu, pivots, iA);
  
  return c_rank;
}


inline double det(const ublas::matrix<double> &A)
{
  // check if the matrix is square
  if (A.size1() != A.size2()) throw matrix_not_square();
  
  // create a permutation matrix (type defined in lu.hpp)
  ublas::permutation_matrix<std::size_t> pivots(A.size1());

  // create a working copy to compute the LU factorization
  ublas::matrix<double> lu(A);
  
  // perform LU-factorization. The matrix 'lu' has the L in the lower
  // triangular part and the U in the upper triangular and the diagonal.
  // 'pivots' represent the permutation matrix.
  int c_rank = ublas::lu_factorize(lu, pivots);  
  if( c_rank != 0 ) return 0.0;
  
  double d = 1.0;

  for(std::size_t i=0; i<pivots.size(); ++i) {
    if (pivots(i)!=i)
      d*=-1.0;
    
    d*=lu(i,i);
  }
  
  if (fabs(d)<1E-18)
    return 0.0;
  
  return d; 
}


// compute the inverse of a matrix and the determinant

inline int inv_det(const ublas::matrix<double> &A, ublas::matrix<double> &iA, 
                   double &det)
{
  // check if the matrix is square
  if (A.size1() != A.size2()) throw matrix_not_square();
  
  // create a permutation matrix (type defined in lu.hpp)
  ublas::permutation_matrix<std::size_t> pivots(A.size1());

  // create a working copy to compute the LU factorization
  ublas::matrix<double> lu(A);
  
  // perform LU-factorization. The matrix 'lu' has the L in the lower
  // triangular part and the U in the upper triangular and the diagonal.
  // 'pivots' represent the permutation matrix.
  int c_rank = ublas::lu_factorize(lu, pivots);
  
  if( c_rank != 0 ){
    det=0.0;
    return c_rank;
  }
  
  // compute the determinant
  det=1.0;
  for(std::size_t i=0; i<pivots.size(); ++i) {
    if (pivots(i)!=i)
      det*=-1.0;
    det*=lu(i,i);
  }
  
  // create the inverse matrix and compute it using the LU substitution
  iA=ublas::identity_matrix<double>(lu.size1());
  lu_substitute(lu, pivots, iA);
  
  return c_rank;
}


}}} // close the boost::numeric::ublas namespace

#endif // _UBLAS_UTILS_
