function [SP, A, cx, cy]=ucm2Superpixels(ucm, T)
%
% [SP, A, cx, cy]=ucm2Superpixels(U, T)
%
% Given the ultrametric contour map U compute the superpixel segmentation
% by considering only the regions that are dinstinct at level 'T'. The
% function also return an affinity matrix A  between superpixels and the
% coordinates of the "center" of each pixel cx and cy.
% The affinities are computed using the intervening contour method and the
% ultrametric colormap gray
% 
% 'U' is the input contour map.
%
% 'T' is the threshold for merging the superpixels.
%
% 'SP' is the output mask with a different label for each superpixel.
%
% 'A' is the affinity matrix for the affinities between superpixels.
%
% 'cx' are the x coordinates of the centers of all superpixels.
%
% 'cy' are the y coordinates of the centers of all superpixels.
%

% fist compute the hard contour and the superpixels segment.
bnd=ucm>=T;
SP=bwlabel(ucm<=T)+2; % we add one to be sure that no superpixel has 
                      % label equal to one.
numSP=max(max(SP)); % number of superpixels

% one or more superpixels correspond to the boundaries. We label them zero 
% and remove them from the image leaving only the superpixels.
SP(bnd==1)=1;

% make sure all the contour pixels have the same label
newSP=ones(size(SP));

% expand all the superpixels to remove all the contour pixels.
for h=2:numSP % we skip the number one
  
  sp_element=(SP==h); % pixels in super pixel h
  sp_element=imdilate(sp_element, strel('disk', 1)); % expand one pixel
  
  % assign the label to the pixels of this superpixels.
  newSP(sp_element)=h;
end

% compact the segmentation
[newSP, numSP]=compactSuperPixels(newSP);

% merge small components, i.e. smaller than 5 pixels.
minSize=5;
newSP=mergeSmallComp(newSP, numSP, minSize);

% finally we reassign the components label to be in the 1-K range with K
% the final number of superpixels.
[SP, numSP]=compactSuperPixels(newSP);

% now the map is ready we compute the centers
% compute the center of equivalent fo reach superpixels
[cx, cy]=getSPCenters(SP);

% and finally we can compute the affinities using the IC method and the
% ultrametric contour map ad contour operator.

% Create all the possible pairs of coordinates.
[xPos, yPos]=meshgrid(cx, cy);

if any(cx<=0)
  error('Something wrong with the x centers');
end

if any(cy<=0)
  error('Something wrong with the y centers');
end

xCoord=[reshape(xPos, numel(xPos), 1), reshape(xPos', numel(xPos), 1)];
yCoord=[reshape(yPos', numel(yPos), 1), reshape(yPos, numel(yPos), 1)];

% intervening contour for the center of each superpixels
intContour=compIntContour(ucm, xCoord, yCoord);

% these are the affinities.
A=reshape(exp(-(intContour/0.1)), length(cx), length(cx));



% =========================================================================
%  Auxiliary function
% =========================================================================

function [sp_cx, sp_cy]=getSPCenters(SP)

% Compute the center of each super pixels. We first try using the mean of
% the coordinates. If this lies outside we then use a random point in the
% super pixels.

% pixels coordinates
[X, Y]=meshgrid(1:cols(SP), 1:rows(SP));

% number of superpixels
numSP=max(max(SP));
sp_cx=zeros(1, numSP);
sp_cy=zeros(1, numSP);

for h=1:numSP
  
  % pixels in this superpixels
  idx=(SP==h);
  
  % all the pixels in the current superpixels
  sp_cx(h)=round(mean(X(idx)));
  sp_cy(h)=round(mean(Y(idx)));
  
  % there is the possibility this center is not in the superpixel. In this
  % case we randomly assign a pixel inside the superpixel as center.
  if SP(sp_cy(h), sp_cx(h))~=h
    l=find(idx); % linear indeces of all the pixels in this superpixels.
    l=l(randperm(length(l))); 
    
    sp_cx(h)=round(X(l(1)));
    sp_cy(h)=round(Y(l(1)));
  end
end


function [newSP, numSP]=compactSuperPixels(SP)

% remove all the labels with no pixels in the SP.

% number of super pixles.
numSP=max(max(SP));

t=1;
newSP=zeros(size(SP));
for h=1:numSP
  
  % index with all the pixel in this super pixel.
  idx=(SP==h);
  
  % if there are pixels in this superpixels then relabel them
  if sum(sum(idx))>0
    newSP(idx)=t;
    t=t+1;
  end
end

% this is the new number of segments.
numSP=t-1;

% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
