function [Ae, Xe, rn, cn, Contours]=xmlExtractAff(pos, xmlFile, type, recomp)
%
% Extract the Intervening countout for the points of position 'pos'. The
% matrix 'pos' has two columns the first one contains the x coordinates and
% the second the y coordinate. 'xmlFile' is the XML file with all the
% info about the auxiliary data. 'type' is the type of contour operator:
% 'edge' is the one use by Ncut code, 'Pb' is the berkeley one.
% The 'recomp' prompts the recomputation of the auxiliary data.
%

% by default we use the Normalize cut code to compute the edge response.
if nargin<3, type='edge'; end

% by defualt we use the cached values.
if nargin<4, recomp=false; end

% first decide which type of contour operator we are going to use
switch lower(type)
  
  % affinity from lihi
  case 'lihi'
    [Ae, Xe, Contours]=extract_aff_lihi(pos, xmlFile, recomp);
  
  % contour operator from Jianbo Shi code
  case 'edge'
    [Ae, Xe, Contours]=extract_aff_edge(pos, xmlFile, recomp);
    
  % 'pb' contour operator from the segmentation software of Charless Fowlkes
  case 'pb'
    [Ae, Xe, Contours]=extract_aff_pb(pos, xmlFile, recomp);

  % pb operator from the superpixel code
  case 'pb2'
    [Ae, Xe, Contours]=extract_aff_pb2(pos, xmlFile, recomp);

  % Global pb from the Berkeley group (see Maire CVPR2008)
  case 'gpb'
    [Ae, Xe, Contours]=extract_aff_gpb(pos, xmlFile, recomp);

  % ucm from the Berkeley group (see Arbelaez CVPR2009)
  case 'ucm'
    [Ae, Xe, Contours]=extract_aff_ucm(pos, xmlFile, recomp);

  % a fixed affinity graph with constant affinities
  case 'fixed'
    [Ae, Xe, Contours]=extract_aff_fixed(pos, xmlFile, recomp);
    
  otherwise
    error('Unrecognized contour operator');
    
end

% we assume that the dimension of the contour image is the same of the
% original image
rn=rows(Contours);
cn=cols(Contours);


% ======================================================================
% Auxiliary functions used only by xmlExtractAff
% ======================================================================

function [Ae, Xe, Contours]=extract_aff_lihi(curr_pos, xmlFile, recomp)

% structure with all the information
imageInfo=xml_load(xmlFile);

% the boolean function 'compute_affinities' checks if we have to recompute
% the affinities or if we can use the cached values.
if compute_affinities(imageInfo, 'lihi_affinities_name', curr_pos, recomp)
  
  % Compute the affinities and save them in the file
  
  % name of the image
  imageFileName=imageInfo.original_name;
  
  % load the image and convert in double format 0-1.
  I=im2double(imread(imageFileName));
  
  % compute the lihi affinities
  disp('Extracting lihi affinities');
  disp(imageFileName);
  tic; [A, X, Contours]=TA_lihi_affinities(I); toc;
  
  % get only the affinites and observation that in 'curr_pos'
  [Ae, Xe, pos]=extract_affinity_and_observation(I, curr_pos, A, X);
  
  % save the affinity file
  imageInfo.lihi_affinities_name=[imageInfo.original_name, '.lihi_aff.mat'];
  save(imageInfo.lihi_affinities_name, 'Ae', 'Xe', 'Contours', 'pos');
  imageInfo.lihi_affinities_date=getFileDate(imageInfo.lihi_affinities_name);
  
  % save the update xml file for the cached values
  xml_save(imageInfo.xml_name, imageInfo);
  
end

% we load the affinities values
load(imageInfo.lihi_affinities_name, 'Ae', 'Xe');
Contours=[];


function [Ae, Xe, Contours]=extract_aff_edge(curr_pos, xmlFile, recomp)

% structure with all the information
imageInfo=xml_load(xmlFile);

% the boolean function 'compute_affinities' checks if we have to recompute
% the affinities or if we can use the cached values.
if compute_affinities(imageInfo, 'ic_affinities_name', curr_pos, recomp)
  
  % Compute the affinities and save them in the file
  
  % name of the image
  imageFileName=imageInfo.original_name;
  
  % load the image and convert in double format 0-1.
  I=im2double(imread(imageFileName));
  
  % compute the IC affinities
  disp('Extracting IC affinities');
  disp(imageFileName);
  tic; [A, X, Contours]=TA_IC_affinities(I); toc;
  
  % get only the affinites and observation that in 'curr_pos'
  [Ae, Xe, pos]=extract_affinity_and_observation(I, curr_pos, A, X);
  
  % save the affinity file
  imageInfo.ic_affinities_name=[imageInfo.original_name, '.ic_aff.mat'];
  save(imageInfo.ic_affinities_name, 'Ae', 'Xe', 'Contours', 'pos');
  imageInfo.ic_affinities_date=getFileDate(imageInfo.ic_affinities_name);
  
  % save the update xml file for the cached values
  xml_save(imageInfo.xml_name, imageInfo);
  
end

% we load the affinities values
load(imageInfo.ic_affinities_name, 'Ae', 'Xe', 'Contours');
   

function [Ae, Xe, Contours]=extract_aff_pb(curr_pos, xmlFile, recomp)

% structure with all the information
imageInfo=xml_load(xmlFile);

if compute_affinities(imageInfo, 'pb_affinities_name', curr_pos, recomp)
  
  % name of the image
  imageFileName=imageInfo.original_name;
  
  % load the image and convert in double format 0-1.
  I=im2double(imread(imageFileName));
  
  % compute the Pb.
  disp('Extracting Pb affinities');
  disp(imageFileName);
  tic; [A, X, Contours]=TA_Pb_affinities(I); toc;
  
  % get only the affinites and observation that in 'curr_pos'
  [Ae, Xe, pos]=extract_affinity_and_observation(I, curr_pos, A, X);
  
  % save the affinity file
  imageInfo.pb_affinities_name=[imageInfo.original_name, '.pb_aff.mat'];
  save(imageInfo.pb_affinities_name, 'Ae', 'Xe', 'Contours', 'pos');
  imageInfo.pb_affinities_date=getFileDate(imageInfo.pb_affinities_name);
  
  % save the update xml file for the cached values
  xml_save(imageInfo.xml_name, imageInfo);
    
end

load(imageInfo.pb_affinities_name, 'Ae', 'Xe', 'Contours');


function [Ae, Xe, Contours]=extract_aff_gpb(curr_pos, xmlFile, recomp)

% structure with all the information
imageInfo=xml_load(xmlFile);

% check if we have to recompute the affinities
if compute_affinities(imageInfo, 'gpb_affinities_name', curr_pos, recomp)
  
  % dimension of the neighbors
  R=13;
  
  % name of the image
  imageFileName=imageInfo.original_name;
  
  % load the image and convert in double format 0-1.
  I=im2double(imread(imageFileName));
  
  % compute the global Pb
  disp('Extracting globalPb affinities');
  disp(imageFileName);

  % name of the mat file where to save the conour map and the affinities.
  imageInfo.gpb_affinities_name=[imageInfo.original_name, '.gpb_aff.mat'];
  
  tic;
  % check if the file exist in this case we assume it contain the contour
  % map from the UCM operator.
  if (exist(imageInfo.gpb_affinities_name, 'file'))
    disp('Using cached countour map');
    load(imageInfo.gpb_affinities_name, 'Contours');
    [A, X, Contours]=TA_gPb_affinities(I, R, Contours); %#ok<NODEF>
  else
    [A, X, Contours]=TA_gPb_affinities(I, R); 
  end
  toc;
  
  % get only the affinites and observation that in 'curr_pos'
  [Ae, Xe, pos]=extract_affinity_and_observation(I, curr_pos, A, X);
  
  % get only the affinites and observation that in 'curr_pos'
  [Ae, Xe, pos]=extract_affinity_and_observation(I, curr_pos, A, X); %#ok<NASGU>
  
  % save the affinity file
  imageInfo.gpb_affinities_name=[imageInfo.original_name, '.gpb_aff.mat'];
  save(imageInfo.gpb_affinities_name, 'Ae', 'Xe', 'Contours', 'pos');
  imageInfo.gpb_affinities_date=getFileDate(imageInfo.gpb_affinities_name);
  
  % save the update xml file for the cached values
  xml_save(imageInfo.xml_name, imageInfo);
    
end

load(imageInfo.gpb_affinities_name, 'Ae', 'Xe', 'Contours');


function [Ae, Xe, Contours]=extract_aff_ucm(curr_pos, xmlFile, recomp)

% structure with all the information
imageInfo=xml_load(xmlFile);


if compute_affinities(imageInfo, 'ucm_affinities_name', curr_pos, recomp)
    
  % dimension of the neighbors
  R=13;
  
  % name of the image
  imageFileName=imageInfo.original_name;
  
  % load the image and convert in double format 0-1.
  I=im2double(imread(imageFileName));
  
  % compute the ultrametric map
  
  disp('Extracting ucm affinities');
  disp(imageFileName);
  
  % name of the mat file where to save the conour map and the affinities.
  imageInfo.ucm_affinities_name=[imageInfo.original_name, '.ucm_aff.mat'];
  
  tic;
  % check if the file exist in this case we assume it contain the contour
  % map from the UCM operator.
  if (exist(imageInfo.ucm_affinities_name, 'file'))
    disp('Using cached countour map');
    load(imageInfo.ucm_affinities_name, 'Contours');
    [A, X, Contours]=TA_ucm_affinities(I, R, Contours); %#ok<NODEF>
  else
    [A, X, Contours]=TA_ucm_affinities(I, R); 
  end
  toc;
  
  % get only the affinites and observation that in 'curr_pos'
  [Ae, Xe, pos]=extract_affinity_and_observation(I, curr_pos, A, X);
  
  % save the affinity file
  save(imageInfo.ucm_affinities_name, 'Ae', 'Xe', 'Contours', 'pos');
  imageInfo.ucm_affinities_date=getFileDate(imageInfo.ucm_affinities_name);
  
  % save the update xml file for the cached values
  xml_save(imageInfo.xml_name, imageInfo);
end

load(imageInfo.ucm_affinities_name, 'Ae', 'Xe', 'Contours');


function [Ae, Xe, Contours]=extract_aff_fixed(curr_pos, xmlFile, recomp)

% structure with all the information
imageInfo=xml_load(xmlFile);


if compute_affinities(imageInfo, 'fixed_affinities_name', curr_pos, recomp)
    
  % dimension of the neighbors
  R=13;

  % name of the image
  imageFileName=imageInfo.original_name;
  
  % load the image and convert in double format 0-1.
  I=im2double(imread(imageFileName));
  
  % compute the global Pb
  disp('Extracting fixed affinities');
  disp(imageFileName);
  tic; [A, X, Contours]=TA_fixed_affinities(I); toc;
  
  % get only the affinites and observation that in 'curr_pos'
  [Ae, Xe, pos]=extract_affinity_and_observation(I, curr_pos, A, X);
  
  % save the affinity file
  imageInfo.fixed_affinities_name=[imageInfo.original_name, '.fixed_aff.mat'];
  save(imageInfo.fixed_affinities_name, 'Ae', 'Xe', 'Contours', 'pos');
  imageInfo.fixed_affinities_date=getFileDate(imageInfo.fixed_affinities_name);
  
  % save the update xml file for the cached values
  xml_save(imageInfo.xml_name, imageInfo);
    
end

load(imageInfo.fixed_affinities_name, 'Ae', 'Xe', 'Contours');

   
function comp_aff=compute_affinities(imageInfo, field_name, curr_pos, recomp)

% before any check the value of 'comp_aff' is equal to 'recomp'
comp_aff=recomp;

% check if we need to compute the data, either because we do not have the
% field in the XML file or because we have the 'recomp' flag.
comp_aff=comp_aff || ~isfield(imageInfo, field_name);

% even if we do not have to recompute the affinities it may be the case
% that the file does not exist. We also check for this case.
comp_aff=comp_aff || ~exist(eval(['imageInfo.', field_name]), 'file');

% finally, even if we have everything cached we may still need to recompute
% the affinities because we are using a different set of points. If
% 'curr_pos' is empty then we assume that the affinities will be recomputed
% by the calling function.
if (~comp_aff) && (~isempty(curr_pos))
  
  % this operation may raise a warning if 'pos' is not present in the mat
  % file. If 'pos' do not exist then we have always to compute the . If it
  % exist then we check if we have that the variable 'curr_pos' and 'pos'
  % are equal.
  load(eval(['imageInfo.', field_name]), 'pos');
  
  % the "short circuit" logical 'or' operator avoids errors
  comp_aff=~exist('pos', 'var') || ...
          rows(pos)~=rows(curr_pos) || ...
          any(pos(:)~=curr_pos(:));  %#ok<NODEF>
end


function  [Ae, Xe, pos]=extract_affinity_and_observation(I, curr_pos, A, X)

% Extract the affinities of the given points

% get also the dimension of the image
row_num=rows(I);
col_num=cols(I);

% convert the positions 'curr_pos' into pixels coordinates. We assume that the
% coordinates are referred to the top left corner of the image. We also
% assume that the coordinates are spanning the same range of the image.

% the first coordinate is the x which correspond to the column of the
% image. The second coordinate is the y which correspond to the rows.
c_idx=round(curr_pos(:, 1));
r_idx=round(curr_pos(:, 2));

% we need to convert the indeces to the linear case
idx=sub2ind([row_num, col_num], r_idx, c_idx);

% when we extract the affinities we may have "holes". These "holes" are
% caused by the implementation of the code for IC from Jianbo Shi. For the
% moment we just try to fill those holes.
Ae=extract_and_fill(A, idx, length(unique(r_idx)), length(unique(c_idx)));
% Ae=A(idx, idx);
Xe=X(idx, :);

pos=curr_pos; 


function Ae=extract_and_fill(A, idx, row_num_e, col_num_e)

% extract the matrix row and column we are interes in
Ae=A(idx, idx);

% check which elements have no affinities except for themselves.
na_idx=find((sum(Ae)'-diag(Ae))==0); 

if (numel(na_idx)>0)
  disp('We have some isolated point');
  % iterate over all the row/col that gives problem
  for h=1:length(na_idx)
    
    % extract the problematic row/column 
    Ae_col=Ae(:,  na_idx(h));
    
    % convert the corresponding linear index to a matrix index and compute
    % the nearest neighbor
    [m, n]=ind2sub([row_num_e, col_num_e], na_idx(h));
    nn_m=m+[ 0  0 -1  1];
    nn_n=n+[-1  1  0  0];
    
    % clip if necessary
    clip_idx=((nn_m<1) | (nn_m>row_num_e) | (nn_n<1) | (nn_n>col_num_e));
    nn_m(clip_idx)=[];
    nn_n(clip_idx)=[];
    
    % set to one the 4 nearest neighbors. 
    nn_idx=sub2ind([row_num_e, col_num_e], nn_m, nn_n);
    
    Ae_col(nn_idx)=1;
    
    % inser this column into the matrix
    Ae(:, na_idx(h))=Ae_col;
    Ae(na_idx(h), :)=Ae_col';
  end

end
  
% check again if we have any empty row/col
na_idx=find((sum(Ae)'-diag(Ae))==0);
if (numel(na_idx)>0)
  disp('We were not able to solve the problem');
end

% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
