function [Ae, Xe, row_num, col_num, imageEdges]=xmlExtract_IC(pos, xmlFileName, ...
                                                              forceComp)
%
% Extract the Intervening countout for the points of position 'pos'.
%


%
% Check if there is a file with the affinities from IC contour.
%

if nargin<3
  forceComp=false;
end


imageInfo=xml_load(xmlFileName);

if ~isfield(imageInfo, 'ic_affinities_name') || (forceComp)
  %
  % compute the affinities and save them in the file
  %
  
  % name of the image
  imageFileName=imageInfo.original_name;
  
  % load the image and convert in double format 0-1.
  I=im2double(imread(imageFileName));
  
  % compute the IC.
  if (forceComp), disp('Recompute the affinities'); end
  disp('Extracting IC affinities');
  disp(imageFileName);
  tic;
  [A, X, imageEdges]=TA_IC_affinities(I);
  toc;
  
  % get also the dimension of the image
  row_num=rows(I);
  col_num=cols(I);
  
  % save the affinity file
  imageInfo.ic_affinities_name=[imageInfo.original_name, '.ic_aff.mat'];
  save(imageInfo.ic_affinities_name, ...
    'A', 'X', 'imageEdges', 'row_num', 'col_num');
  imageInfo.ic_affinities_date=getFileDate(imageInfo.ic_affinities_name);
  
  % save the update xml file
  xml_save(imageInfo.xml_name, imageInfo);
  
end

load(imageInfo.ic_affinities_name, 'A', 'X', 'row_num', 'col_num');
X=double(X);


%
% Extract the affinities of the given points
% 

% convert the positions 'pos' into pixels coordinates. We assume that the
% coordinates are referred to the top left corner of the image. We also
% assume that the coordinates are spanning the same range of the image.

% the first coordinate is the x which correspond to the column of the
% image. The second coordinate is the y which correspond to the rows.
c_idx=round(pos(:, 1));
r_idx=round(pos(:, 2));

% we need to convert the indeces to the linear case
idx=sub2ind([row_num, col_num], r_idx, c_idx);

% when we extract the affinities we may have "holes". These "holes" are
% caused by the implementation of the code for IC from Jianbo Shi. For the
% moment we just try to fill those holes.
Ae=extract_and_fill(A, idx, length(unique(r_idx)), length(unique(c_idx)));
Xe=X(idx, :);


%%

function Ae=extract_and_fill(A, idx, row_num_e, col_num_e)

% extract the matrix row and column we are interes in
Ae=A(idx, idx);

% check which elements have no affinities except for themselves.
na_idx=find((sum(Ae)'-diag(Ae))==0); 

if (numel(na_idx)>0)
  % iterate over all the row/col that gives problem
  for h=1:length(na_idx)
    
    % extract the problematic row/column 
    Ae_col=Ae(:,  na_idx(h));
    
    % convert the corresponding linear index to a matrix index and compute
    % the nearest neighbor
    [m, n]=ind2sub([row_num_e, col_num_e], na_idx(h));
    nn_m=m+[ 0  0 -1  1];
    nn_n=n+[-1  1  0  0];
    
    % clip if necessary
    clip_idx=((nn_m<1) | (nn_m>row_num_e) | (nn_n<1) | (nn_n>col_num_e));
    nn_m(clip_idx)=[];
    nn_n(clip_idx)=[];
    
    
    % set to one the 4 nearest neighbors. 
    nn_idx=sub2ind([row_num_e, col_num_e], nn_m, nn_n);
    
    Ae_col(nn_idx)=1;
    
    % inser this column into the matrix
    Ae(:, na_idx(h))=Ae_col;
    Ae(na_idx(h), :)=Ae_col';
  end

end
  

% check again if we have any empty row/col
na_idx=find((sum(Ae)'-diag(Ae))==0);
if (numel(na_idx)>0)
  disp('We were not able to solve the problem');
end

% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
