function [segMasks, probImages, seg_sizes]=xmlGenerateSegmentations(xmlFile, idx)
%
% Generate the segmentations for the experiment described in 'xmlFile'.
% if the additional input 'idx' is provided the segmentation refers to the 
% sample of index 'idx'. By default the last sample is used.
%

disp('Generate segmentations');

% load the file structure with all the info about the experiment
expInfo=readXMLfile(xmlFile);

% load the segmentation results
if size(whos('-file', [expInfo.destDir, '/seg_results.mat'], 'P_all'), 1)==1
  load([expInfo.destDir, '/seg_results.mat'], 'Cs_all', 'P_all');
else
  load([expInfo.destDir, '/seg_results.mat'], 'Cs_all');
end

% this will contain the sizes of all the segments
seg_sizes=zeros(1, expInfo.K);

segMasks=cell(1, length(expInfo.data_file_info));
probImages=cell(1, length(expInfo.data_file_info));

% iterate over all the files used in this experiment and generate and
% generate the segmentation images
for h=1:length(expInfo.data_file_info)
  
  % get the grid positions
  [ix, iy]=getGridPositions(expInfo.data_file_info{h}.width, ...
                            expInfo.data_file_info{h}.height, ...
                            expInfo.grid_step);
  
  % get the assignemnt for image h
  if (nargin==2)
    if idx<=0
      Cmap=Cs_all{h}(:, end); %#ok<USENS>
    else
      Cmap=Cs_all{h}(:, idx); 
    end
    P=labels2ind(Cmap, expInfo.K);
  else
    [Cmap, P]=extractMAP(Cs_all{h}, expInfo.K);
  end
  
  if exist('P_all', 'var'), P=(P_all{h}); end %#ok<USENS>
  
  % check for possible errors...
  if (expInfo.K<max(Cmap))
    error('Unrecognize topic label'); 
  end

  % reshape the segmentation
  col_num=length(unique(ix));
  row_num=length(unique(iy));

  segImageMAP=reshape(Cmap, row_num, col_num);
  segImageMAP=imresize(segImageMAP, expInfo.grid_step, 'nearest');

  % at this point the image is not in the right position, we center the
  % image and propagate the border
  segImageMAP=center_and_replicate(segImageMAP, expInfo.grid_step, ...
                                   expInfo.data_file_info{h}.height,...
                                   expInfo.data_file_info{h}.width);

  % expand also the probabilities
  probImage=zeros(expInfo.data_file_info{h}.height, ...
                  expInfo.data_file_info{h}.width, ...
                  expInfo.K);
                  
  for k=1:expInfo.K
    pp=reshape(P(:, k), row_num, col_num);
    pp=imresize(pp, expInfo.grid_step, 'nearest');
    
    % at this point the image is not in the right position, we center the
    % image and propagate the border
    probImage(:,:,k)=center_and_replicate(pp, expInfo.grid_step, ...
                                     expInfo.data_file_info{h}.height,...
                                     expInfo.data_file_info{h}.width);
  end

  % generate the segmentation and save it in a MAT file
  outMatFileName=sprintf('%s/seg_map_%04d.mat', expInfo.destDir, h);
  save(outMatFileName, 'segImageMAP', 'probImage');
  
  segMasks{h}=segImageMAP;
  
  if strcmp(expInfo.model_type, 'AB')
    segMasks{h}=removeDust(segMasks{h}, expInfo.K, 5);
  end
  
  probImages{h}=[];
  putDot(h);
end
disp(' ');

% disp('Sizes of all the segments:');
% disp(num2str(seg_sizes));

disp(' ');


function outSeg=center_and_replicate(inSeg, grid_step, M, N)

% TODO: this may be off of one pixel. In the unlikeliy case the
% segmentation resuts improve to pixel accuracy this may be an issue.

for h=1:(grid_step-1)
  inSeg=[inSeg(:, 1) inSeg inSeg(:, end)]; %#ok<AGROW>
  inSeg=[inSeg(1, :); inSeg; inSeg(end, :)]; %#ok<AGROW>
end

outSeg=inSeg(1:M, 1:N);

% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
