function [W, ix, iy, hueDesc]=xmlLoadPatchHue(xmlFile, step, hue_words, recompute)
%
% Load the visual words for the hue descriptor. See Joost van de Weijer,
% Cordelia Schmid "Coloring Local Feature Extraction", Proc. ECCV06, Graz, 
% Austria, 2006.
% (http://lear.inrialpes.fr/people/vandeweijer/color_descriptors.html)
%
% if recompute is set to true then the filter bank responce is recompute
%

% by default we use the cached results if they exist
if nargin<4, recompute=false; end

% load the xmlFile
imageInfo=xml_load(xmlFile);

% generate the grid positions
[ix, iy, ix_rng, iy_rng]=getGridPositions(imageInfo.width, imageInfo.height, step);

% get the hue descriptors
hueDesc=loadHueDesc(imageInfo, ix, iy, recompute);

% compute the visual words labels
[~, W]=mindist(hue_words', hueDesc');
W=W(:);


function hueDesc=loadHueDesc(imageInfo, ix, iy, recompute)

% check if the hue descriptor is already available, if not compute it and save it. 
if (~isfield(imageInfo, 'hue_desc_name') || recompute)
  
  disp('Computing hue descriptor');
  disp(imageInfo.original_name);
  
  % load the image (range 0-1).
  I=im2double(imread(imageInfo.original_name));
  
  % there is no field 'hue_desc_name', or we are forced to recompute it.
  imageInfo.hue_desc_name=[imageInfo.original_name, '.hue.mat'];
  
  % construct the rectangle. The dimension is fixed to 20x20 for now
  Rects=[ix-10 iy-10 19*ones(length(ix), 2)];
  
  % extract the patches
  [patches_R, patches_G, patches_B]=GetPatches(I, Rects);
  
  % compute the hue descriptors
  hueDesc=HueDescriptor(patches_R, patches_G, patches_B)';
  
  % save the hue descriptors
  save(imageInfo.hue_desc_name, 'hueDesc', 'ix', 'iy');
  
  % updated xmlFile
  imageInfo.hue_desc_date=getFileDate(imageInfo.hue_desc_name);
  xml_save(imageInfo.xml_name, imageInfo);
else
  load(imageInfo.hue_desc_name, 'hueDesc');
end


function [patches_R, patches_G, patches_B]=GetPatches(I, Rects)

% iterate over all the rectangles. Extract the patches separate them in
% red, green, and blue. 

offset=12;

% Pad the patches if necessary.
Ip=[repmat(I(:,1,:),  [1, offset, 1]), I,  repmat(I(:,end,:), [1, offset, 1])];
Ip=[repmat(Ip(1,:,:), [offset, 1, 1]); Ip; repmat(Ip(end,:,:),  [offset, 1, 1])];

% move the position of the rectangles
Rects=Rects+[offset*ones(rows(Rects), 2) zeros(rows(Rects), 2)];

patches_R=zeros(400, rows(Rects));
patches_G=zeros(400, rows(Rects));
patches_B=zeros(400, rows(Rects));

for h=1:rows(Rects)
  P=imcrop(Ip, Rects(h, :));  
  patches_R(:,h)=reshape(P(:,:,1), 400, 1);
  patches_G(:,h)=reshape(P(:,:,2), 400, 1);
  patches_B(:,h)=reshape(P(:,:,3), 400, 1);
end

% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
