function [W, ix, iy, siftDesc]=xmlLoadPatchSift(xmlFile, step, sift_words, ...
                                                recompute)
%
% Load the SIFT descriptor and compute the quantized visual words for
% the specified grid.
%
% if recompute is set to true then the SIFT descriptors are recompute
%
if nargin<4, recompute=false; end

% load the xmlFile
imageInfo=xml_load(xmlFile);

% generate the grid positions
[ix, iy]=getGridPositions(imageInfo.width, imageInfo.height, step);

% get the sift descriptors
siftDesc=loadSiftDesc(imageInfo, ix, iy, recompute);
[~, W]=min(distance(sift_words, double(siftDesc)));
siftDesc=siftDesc';
W=W';

% a small sanity check on the labeling of W
if any(W==0), error('Null label not allowed in the dictionary'); end
if any(W>cols(sift_words)), error('Out of range label'); end



function siftDesc=loadSiftDesc(imageInfo, ix, iy, recompute)


% check if the sift descriptor is already available, if not compute it
% and save it. 
if (~isfield(imageInfo, 'sift_desc_name') || recompute)

  % there is no field 'sift_desc_name', or we are forced to recompute it.
  imageInfo.sift_desc_name=[imageInfo.original_name, '.sift.mat'];
  
  % compute the SIFT descriptors
  I=imread(imageInfo.original_name);
  switch size(I, 3)
    case 1, I=im2single(I);
    case 3, I=im2single(rgb2gray(I));
    otherwise, error('Problem with the image format');
  end
      
  disp('Computing SIFT descriptor');
  disp(imageInfo.original_name);
  
  % compose the frames for computing the SIFT descriptor
  frames=[ix(:), iy(:), 8*ones(numel(ix), 1), zeros(numel(ix), 1)];
  
  % compute the SIFT descriptors using vl_dsift function.
  [pos, siftDesc] = vl_dsift(im2single(I), 'size', 2, 'step', 4);
  pos=pos';
  siftDesc=double(siftDesc);

  % check if the position returned from the vl_dsift function are
  % compatible with the ix and iy coordinates
  if any(pos(:, 1) ~= ix), error('Wrong ix coordinate'); end
  if any(pos(:, 2) ~= iy), error('Wrong iy coordinate'); end
  
  % save the frame and the sift descriptor into the mat file
  save(imageInfo.sift_desc_name, 'siftDesc', 'frames');
  
  % updated xmlFile
  imageInfo.sift_desc_date=getFileDate(imageInfo.sift_desc_name);
  xml_save(imageInfo.xml_name, imageInfo);
else
  % we just need to load the descriptor.
  load(imageInfo.sift_desc_name, 'siftDesc');
end



% function scales=genRandomScales(imageInfo, ix)
% 
% % limit of the scales values   
% rmin=6;
% rmax=min(imageInfo.height, imageInfo.width)/4 ;
% r_range=rmin:rmax ;
% 
% % construct the distribution for the random variable scale
% p=imageInfo.height*imageInfo.width ...
%  -2*(imageInfo.height*imageInfo.width)*r_range...
%  +4*r_range.^2 ;
% P=cumsum(p./sum(p)); % cumulative distribution function
% P=[0 P(1:end-1)]; % from zero to one. one excluded.
% 
% % generate random scales
% scales=[];
% for h=1:numel(ix)
%   r=r_range(find(P<rand, 1, 'last'));
%   scales = cat(1, scales, r/6);
% end

% AUTORIGHTS
% Copyright (C) 2008-2012 Marco Andreetto
%       
% This file is part of AB_SAMPLER.
% 
% AB_SAMPLER is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
